@@ -997,12 +997,13 @@ assert_equal 'ok', %q{
 
 assert_normal_exit %q{
   begin
-    Process.setrlimit(Process::RLIMIT_STACK, 1024*1024)
+    Process.setrlimit(Process::RLIMIT_STACK, 4_202_496)
+    # FreeBSD fails this less than 4M + 8K bytes.
   rescue Exception
     exit
   end
   class C
-    attr "a" * (2*1024*1024)
+    attr "a" * (10*1024*1024)
   end
 }, '[ruby-dev:31818]'
 
@@ -1057,3 +1058,125 @@ assert_equal '[false, false, false, false, true, true]', %q{
   D.new.bar{}
   [C.new.foo, C.new.foo{}, D.new.m1, D.new.m1{}, D.new.m2, D.new.m2{}]
 }, '[ruby-core:14813]'
+
+assert_equal 'ok', %q{
+  class Foo
+    define_method(:foo) do |&b|
+      b.call
+    end
+  end
+  Foo.new.foo do
+    break :ok
+  end
+}, '[ruby-dev:36028]'
+
+assert_equal '[1, 2, [3, 4]]', %q{
+  def regular(a, b, *c)
+    [a, b, c]
+  end
+  regular(*[], 1, *[], *[2, 3], *[], 4) 
+}, '[ruby-core:19413]'
+
+assert_equal '[1, [:foo, 3, 4, :foo]]', %q{
+  def regular(a, *b)
+    [a, b]
+  end
+  a = b = [:foo]
+  regular(1, *a, *[3, 4], *b)
+}
+
+assert_equal '["B", "A"]', %q{
+  class A
+    def m
+      'A'
+    end
+  end
+
+  class B < A
+    define_method(:m) do    
+      ['B', super()]
+    end
+  end
+
+  class C < B
+  end
+
+  C.new.m
+}
+
+assert_equal 'ok', %q{
+  module Foo
+    def foo
+      begin
+        super
+      rescue NoMethodError
+        :ok
+      end
+    end
+    module_function :foo
+  end
+  Foo.foo
+}, '[ruby-dev:37587]'
+
+assert_equal 'Object#foo', %q{
+  class Object
+    def self.foo
+      "Object.foo"
+    end
+    def foo
+      "Object#foo"
+    end
+  end
+
+  module Foo
+    def foo
+      begin
+        super
+      rescue NoMethodError
+        :ok
+      end
+    end
+    module_function :foo
+  end
+  Foo.foo
+}, '[ruby-dev:37587]'
+
+assert_normal_exit %q{
+  class BasicObject
+    remove_method :method_missing
+  end
+  begin
+    "a".lalala!
+  rescue NoMethodError => e
+    e.message == "undefined method `lalala!' for \"a\":String" ? :ok : :ng
+  end
+}, '[ruby-core:22298]'
+
+assert_equal 'ok', %q{
+  "hello"[0] ||= "H"
+  "ok"
+}
+
+assert_equal 'ok', %q{
+  "hello"[0, 1] ||= "H"
+  "ok"
+}
+
+assert_equal 'ok', %q{
+  class C
+    define_method(:foo) do
+      C.class_eval { remove_method(:foo) }
+      super()
+    end
+  end
+  begin
+    C.new.foo
+  rescue NoMethodError
+    'ok'
+  end
+}
+assert_equal 'ok', %q{
+  [0][0, &proc{}] += 21
+  'ok'
+}, '[ruby-core:30534]'
+
