@@ -3,34 +3,6 @@
  * included by eval.c
  */
 
-const char *
-rb_sourcefile(void)
-{
-    rb_thread_t *th = GET_THREAD();
-    rb_control_frame_t *cfp = vm_get_ruby_level_cfp(th, th->cfp);
-
-    if (cfp) {
-	return RSTRING_PTR(cfp->iseq->filename);
-    }
-    else {
-	return 0;
-    }
-}
-
-int
-rb_sourceline(void)
-{
-    rb_thread_t *th = GET_THREAD();
-    rb_control_frame_t *cfp = vm_get_ruby_level_cfp(th, th->cfp);
-
-    if (cfp) {
-	return vm_get_sourceline(cfp);
-    }
-    else {
-	return 0;
-    }
-}
-
 static void
 warn_printf(const char *fmt, ...)
 {
@@ -79,6 +51,12 @@ get_backtrace(VALUE info)
     return rb_check_backtrace(info);
 }
 
+VALUE
+rb_get_backtrace(VALUE info)
+{
+    return get_backtrace(info);
+}
+
 static void
 set_backtrace(VALUE info, VALUE bt)
 {
@@ -88,14 +66,17 @@ set_backtrace(VALUE info, VALUE bt)
 static void
 error_print(void)
 {
-    VALUE errat = Qnil;		/* OK */
-    VALUE errinfo = GET_THREAD()->errinfo;
+    volatile VALUE errat = Qnil;		/* OK */
+    rb_thread_t *th = GET_THREAD();
+    VALUE errinfo = th->errinfo;
+    int raised_flag = th->raised_flag;
     volatile VALUE eclass, e;
-    char *einfo;
-    long elen;
+    const char *volatile einfo;
+    volatile long elen;
 
     if (NIL_P(errinfo))
 	return;
+    rb_thread_raised_clear(th);
 
     PUSH_TAG();
     if (EXEC_TAG() == 0) {
@@ -109,10 +90,12 @@ error_print(void)
     if (NIL_P(errat)) {
 	const char *file = rb_sourcefile();
 	int line = rb_sourceline();
-	if (file)
-	    warn_printf("%s:%d", file, line);
-	else
+	if (!file)
 	    warn_printf("%d", line);
+	else if (!line)
+	    warn_printf("%s", file);
+	else
+	    warn_printf("%s:%d", file, line);
     }
     else if (RARRAY_LEN(errat) == 0) {
 	error_pos();
@@ -181,7 +164,7 @@ error_print(void)
 	long len = RARRAY_LEN(errat);
 	VALUE *ptr = RARRAY_PTR(errat);
         int skip = eclass == rb_eSysStackError;
-	
+
 #define TRACE_MAX (TRACE_HEAD+TRACE_TAIL+5)
 #define TRACE_HEAD 8
 #define TRACE_TAIL 5
@@ -199,6 +182,7 @@ error_print(void)
     }
   error:
     POP_TAG();
+    rb_thread_raised_set(th, raised_flag);
 }
 
 void
@@ -210,7 +194,7 @@ ruby_error_print(void)
 void
 rb_print_undef(VALUE klass, ID id, int scope)
 {
-    char *v;
+    const char *v;
 
     switch (scope) {
       default:
@@ -237,7 +221,7 @@ error_handle(int ex)
     int status = EXIT_FAILURE;
     rb_thread_t *th = GET_THREAD();
 
-    if (rb_thread_set_raised(th))
+    if (rb_threadptr_set_raised(th))
 	return EXIT_FAILURE;
     switch (ex & TAG_MASK) {
       case 0:
@@ -269,8 +253,7 @@ error_handle(int ex)
 	error_pos();
 	warn_printf(": unexpected throw\n");
 	break;
-      case TAG_RAISE:
-      case TAG_FATAL: {
+      case TAG_RAISE: {
 	VALUE errinfo = GET_THREAD()->errinfo;
 	if (rb_obj_is_kind_of(errinfo, rb_eSystemExit)) {
 	    status = sysexit_status(errinfo);
@@ -283,10 +266,13 @@ error_handle(int ex)
 	}
 	break;
       }
+      case TAG_FATAL:
+	error_print();
+	break;
       default:
 	rb_bug("Unknown longjmp status %d", ex);
 	break;
     }
-    rb_thread_reset_raised(th);
+    rb_threadptr_reset_raised(th);
     return status;
 }
