@@ -110,7 +110,7 @@ static VALUE ec_instance(VALUE klass, EC_KEY *ec)
 {
     EVP_PKEY *pkey;
     VALUE obj;
-	
+
     if (!ec) {
 	return Qfalse;
     }
@@ -163,7 +163,7 @@ static VALUE ossl_ec_key_initialize(int argc, VALUE *argv, VALUE self)
     EC_KEY *ec = NULL;
     VALUE arg, pass;
     VALUE group = Qnil;
-	
+
     GetPKey(self, pkey);
     if (pkey->pkey.ec)
         rb_raise(eECError, "EC_KEY already initialized");
@@ -187,23 +187,27 @@ static VALUE ossl_ec_key_initialize(int argc, VALUE *argv, VALUE self)
             ec = PEM_read_bio_ECPrivateKey(in, NULL, NULL, NULL);
             if (!ec) {
                 (void)BIO_reset(in);
+                (void)ERR_get_error();
                 ec = PEM_read_bio_EC_PUBKEY(in, NULL, NULL, NULL);
             }
             if (!ec) {
                 (void)BIO_reset(in);
+                (void)ERR_get_error();
                 ec = d2i_ECPrivateKey_bio(in, NULL);
             }
             if (!ec) {
                 (void)BIO_reset(in);
+                (void)ERR_get_error();
                 ec = d2i_EC_PUBKEY_bio(in, NULL);
             }
 
             BIO_free(in);
 
             if (ec == NULL) {
-                const char *name = STR2CSTR(arg);
+                const char *name = StringValueCStr(arg);
                 int nid = OBJ_sn2nid(name);
 
+                (void)ERR_get_error();
                 if (nid == NID_undef)
                     ossl_raise(eECError, "unknown curve name (%s)\n", name);
 
@@ -274,7 +278,7 @@ static VALUE ossl_ec_key_get_group(VALUE self)
  *  the group.
  *
  *  Setting the group will immediately destroy any previously assigned group object.
- *  The group is internally copied by OpenSSL.  Modifying the original group after 
+ *  The group is internally copied by OpenSSL.  Modifying the original group after
  *  assignment will not effect the internal key structure.
  *  (your changes may be lost).  BE CAREFUL.
  *
@@ -463,8 +467,10 @@ static VALUE ossl_ec_key_to_string(VALUE self, int format)
     BIO *out;
     int i = -1;
     int private = 0;
+#if 0  /* unused now */
     EVP_CIPHER *cipher = NULL;
     char *password = NULL;
+#endif
     VALUE str;
 
     Require_EC_KEY(self, ec);
@@ -484,13 +490,18 @@ static VALUE ossl_ec_key_to_string(VALUE self, int format)
     switch(format) {
     case EXPORT_PEM:
     	if (private) {
+#if 0  /* unused now */
     	    if (cipher || password)
 /* BUG: finish cipher/password key export */
     	        rb_notimplement();
             i = PEM_write_bio_ECPrivateKey(out, ec, cipher, NULL, 0, NULL, password);
+#endif
+            i = PEM_write_bio_ECPrivateKey(out, ec, NULL, NULL, 0, NULL, NULL);
     	} else {
+#if 0  /* unused now */
     	    if (cipher || password)
                 rb_raise(rb_eArgError, "encryption is not supported when exporting this key type");
+#endif
 
             i = PEM_write_bio_EC_PUBKEY(out, ec);
         }
@@ -498,13 +509,17 @@ static VALUE ossl_ec_key_to_string(VALUE self, int format)
     	break;
     case EXPORT_DER:
         if (private) {
+#if 0  /* unused now */
     	    if (cipher || password)
                 rb_raise(rb_eArgError, "encryption is not supported when exporting this key type");
+#endif
 
             i = i2d_ECPrivateKey_bio(out, ec);
         } else {
+#if 0  /* unused now */
     	    if (cipher || password)
                 rb_raise(rb_eArgError, "encryption is not supported when exporting this key type");
+#endif
 
             i = i2d_EC_PUBKEY_bio(out, ec);
         }
@@ -670,7 +685,7 @@ static VALUE ossl_ec_key_dsa_sign_asn1(VALUE self, VALUE data)
 
 /*
  *  call-seq:
- *     key.dsa_verify(data, sig)   => true or false
+ *     key.dsa_verify_asn1(data, sig)   => true or false
  *
  *  See the OpenSSL documentation for ECDSA_verify()
  */
@@ -774,7 +789,7 @@ static VALUE ossl_ec_group_initialize(int argc, VALUE *argv, VALUE self)
             BIO_free(in);
 
             if (!group) {
-                const char *name = STR2CSTR(arg1);
+                const char *name = StringValueCStr(arg1);
                 int nid = OBJ_sn2nid(name);
 
                 if (nid == NID_undef)
@@ -1012,7 +1027,7 @@ static VALUE ossl_ec_group_set_asn1_flag(VALUE self, VALUE flag_v)
 }
 
 /*  call-seq:
- *     group.point_conversion_form  => :uncompressed | :compressed | :hybrid 
+ *     group.point_conversion_form  => :uncompressed | :compressed | :hybrid
  *
  *  See the OpenSSL documentation for EC_GROUP_get_point_conversion_form()
  */
@@ -1032,7 +1047,7 @@ static VALUE ossl_ec_group_get_point_conversion_form(VALUE self)
     case POINT_CONVERSION_HYBRID:	ret = ID_hybrid; break;
     default:	rb_raise(eEC_GROUP, "unsupported point conversion form: %d, this module should be updated", form);
     }
-    
+
    return ID2SYM(ret);
 }
 
@@ -1259,7 +1274,7 @@ static VALUE ossl_ec_point_initialize(int argc, VALUE *argv, VALUE self)
             rb_raise(rb_eArgError, "1st argument must be OpenSSL::PKey::EC::Group");
         group_v = arg1;
         SafeRequire_EC_GROUP(group_v, group);
-        
+
         if (rb_obj_is_kind_of(arg2, cBN)) {
             const BIGNUM *bn = GetBNPtr(arg2);
 
