@@ -1,7 +1,7 @@
 #
 # extconf.rb
 #
-# $Id: extconf.rb 16679 2008-05-29 11:23:36Z knu $
+# $Id: extconf.rb 26353 2010-01-19 05:14:29Z usa $
 #
 
 require 'mkmf'
@@ -56,6 +56,9 @@ if %w'z libz zlib1 zlib zdll'.find {|z| have_library(z, 'deflateReset')} and
 
   $defs.concat(defines.collect{|d|' -D'+d})
 
+  have_func('crc32_combine', 'zlib.h')
+  have_func('adler32_combine', 'zlib.h')
+
   create_makefile('zlib')
 
 end
