@@ -11,7 +11,7 @@
 
 #include "ruby/ruby.h"
 
-/* #define MARK_FREE_DEBUG 1 */
+/* #define RUBY_MARK_FREE_DEBUG 1 */
 #include "gc.h"
 #include "vm_core.h"
 #include "iseq.h"
@@ -23,6 +23,22 @@ VALUE rb_cISeq;
 
 #define hidden_obj_p(obj) (!SPECIAL_CONST_P(obj) && !RBASIC(obj)->klass)
 
+static inline VALUE
+obj_resurrect(VALUE obj)
+{
+    if (hidden_obj_p(obj)) {
+	switch (BUILTIN_TYPE(obj)) {
+	  case T_STRING:
+	    obj = rb_str_resurrect(obj);
+	    break;
+	  case T_ARRAY:
+	    obj = rb_ary_resurrect(obj);
+	    break;
+	}
+    }
+    return obj;
+}
+
 static void
 compile_data_free(struct iseq_compile_data *compile_data)
 {
@@ -47,10 +63,12 @@ iseq_free(void *ptr)
     if (ptr) {
 	iseq = ptr;
 	if (!iseq->orig) {
-	    /* It's possible that strings are freed
-	     * GC_INFO("%s @ %s\n", RSTRING_PTR(iseq->name),
-	     *                      RSTRING_PTR(iseq->filename));
-	     */
+	    /* It's possible that strings are freed */
+	    if (0) {
+		RUBY_GC_INFO("%s @ %s\n", RSTRING_PTR(iseq->name),
+					  RSTRING_PTR(iseq->filename));
+	    }
+
 	    if (iseq->iseq != iseq->iseq_encoded) {
 		RUBY_FREE_UNLESS_NULL(iseq->iseq_encoded);
 	    }
@@ -58,6 +76,7 @@ iseq_free(void *ptr)
 	    RUBY_FREE_UNLESS_NULL(iseq->iseq);
 	    RUBY_FREE_UNLESS_NULL(iseq->insn_info_table);
 	    RUBY_FREE_UNLESS_NULL(iseq->local_table);
+	    RUBY_FREE_UNLESS_NULL(iseq->ic_entries);
 	    RUBY_FREE_UNLESS_NULL(iseq->catch_table);
 	    RUBY_FREE_UNLESS_NULL(iseq->arg_opt_table);
 	    compile_data_free(iseq->compile_data);
@@ -70,54 +89,104 @@ iseq_free(void *ptr)
 static void
 iseq_mark(void *ptr)
 {
-    rb_iseq_t *iseq;
     RUBY_MARK_ENTER("iseq");
 
     if (ptr) {
-	iseq = ptr;
+	rb_iseq_t *iseq = ptr;
+
 	RUBY_GC_INFO("%s @ %s\n", RSTRING_PTR(iseq->name), RSTRING_PTR(iseq->filename));
 	RUBY_MARK_UNLESS_NULL(iseq->mark_ary);
 	RUBY_MARK_UNLESS_NULL(iseq->name);
 	RUBY_MARK_UNLESS_NULL(iseq->filename);
+	RUBY_MARK_UNLESS_NULL(iseq->filepath);
 	RUBY_MARK_UNLESS_NULL((VALUE)iseq->cref_stack);
 	RUBY_MARK_UNLESS_NULL(iseq->klass);
 	RUBY_MARK_UNLESS_NULL(iseq->coverage);
-/* 	RUBY_MARK_UNLESS_NULL((VALUE)iseq->node); */
-/*	RUBY_MARK_UNLESS_NULL(iseq->cached_special_block); */
+#if 0
+	RUBY_MARK_UNLESS_NULL((VALUE)iseq->node);
+	RUBY_MARK_UNLESS_NULL(iseq->cached_special_block);
+#endif
 	RUBY_MARK_UNLESS_NULL(iseq->orig);
 
 	if (iseq->compile_data != 0) {
-	    RUBY_MARK_UNLESS_NULL(iseq->compile_data->mark_ary);
-	    RUBY_MARK_UNLESS_NULL(iseq->compile_data->err_info);
-	    RUBY_MARK_UNLESS_NULL(iseq->compile_data->catch_table_ary);
+	    struct iseq_compile_data *const compile_data = iseq->compile_data;
+	    RUBY_MARK_UNLESS_NULL(compile_data->mark_ary);
+	    RUBY_MARK_UNLESS_NULL(compile_data->err_info);
+	    RUBY_MARK_UNLESS_NULL(compile_data->catch_table_ary);
 	}
     }
     RUBY_MARK_LEAVE("iseq");
 }
 
+static size_t
+iseq_memsize(const void *ptr)
+{
+    size_t size = sizeof(rb_iseq_t);
+    const rb_iseq_t *iseq;
+
+    if (ptr) {
+	iseq = ptr;
+	if (!iseq->orig) {
+	    if (iseq->iseq != iseq->iseq_encoded) {
+		size += iseq->iseq_size * sizeof(VALUE);
+	    }
+
+	    size += iseq->iseq_size * sizeof(VALUE);
+	    size += iseq->insn_info_size * sizeof(struct iseq_insn_info_entry);
+	    size += iseq->local_table_size * sizeof(ID);
+	    size += iseq->catch_table_size * sizeof(struct iseq_catch_table_entry);
+	    size += iseq->arg_opts * sizeof(VALUE);
+	    size += iseq->ic_size * sizeof(struct iseq_inline_cache_entry);
+
+	    if (iseq->compile_data) {
+		struct iseq_compile_data_storage *cur;
+
+		cur = iseq->compile_data->storage_head;
+		while (cur) {
+		    size += cur->size + sizeof(struct iseq_compile_data_storage);
+		    cur = cur->next;
+		}
+		size += sizeof(struct iseq_compile_data);
+	    }
+	}
+    }
+
+    return size;
+}
+
+static const rb_data_type_t iseq_data_type = {
+    "iseq",
+    iseq_mark,
+    iseq_free,
+    iseq_memsize,
+};
+
 static VALUE
 iseq_alloc(VALUE klass)
 {
-    VALUE volatile obj;
     rb_iseq_t *iseq;
-
-    obj = Data_Make_Struct(klass, rb_iseq_t, iseq_mark, iseq_free, iseq);
-    MEMZERO(iseq, rb_iseq_t, 1);
-    return obj;
+    return TypedData_Make_Struct(klass, rb_iseq_t, &iseq_data_type, iseq);
 }
 
 static void
 set_relation(rb_iseq_t *iseq, const VALUE parent)
 {
-    const int type = iseq->type;
+    const VALUE type = iseq->type;
     rb_thread_t *th = GET_THREAD();
 
     /* set class nest stack */
     if (type == ISEQ_TYPE_TOP) {
 	/* toplevel is private */
-	iseq->cref_stack = NEW_BLOCK(th->top_wrapper ? th->top_wrapper : rb_cObject);
+	iseq->cref_stack = NEW_BLOCK(rb_cObject);
 	iseq->cref_stack->nd_file = 0;
 	iseq->cref_stack->nd_visi = NOEX_PRIVATE;
+	if (th->top_wrapper) {
+	    NODE *cref = NEW_BLOCK(th->top_wrapper);
+	    cref->nd_file = 0;
+	    cref->nd_visi = NOEX_PRIVATE;
+	    cref->nd_next = iseq->cref_stack;
+	    iseq->cref_stack = cref;
+	}
     }
     else if (type == ISEQ_TYPE_METHOD || type == ISEQ_TYPE_CLASS) {
 	iseq->cref_stack = NEW_BLOCK(0); /* place holder */
@@ -146,9 +215,11 @@ set_relation(rb_iseq_t *iseq, const VALUE parent)
     }
 }
 
+VALUE rb_realpath_internal(VALUE basedir, VALUE path, int strict);
+
 static VALUE
 prepare_iseq_build(rb_iseq_t *iseq,
-		   VALUE name, VALUE filename,
+		   VALUE name, VALUE filename, VALUE filepath, VALUE line_no,
 		   VALUE parent, VALUE type, VALUE block_opt,
 		   const rb_compile_option_t *option)
 {
@@ -157,8 +228,11 @@ prepare_iseq_build(rb_iseq_t *iseq,
 
     iseq->name = name;
     iseq->filename = filename;
+    iseq->filepath = filepath == Qnil ? Qnil : rb_realpath_internal(Qnil, filepath, 1);
+    iseq->line_no = line_no;
     iseq->defined_method_id = 0;
-    iseq->mark_ary = rb_ary_new();
+    iseq->mark_ary = rb_ary_tmp_new(3);
+    OBJ_UNTRUST(iseq->mark_ary);
     RBASIC(iseq->mark_ary)->klass = 0;
 
     iseq->type = type;
@@ -174,8 +248,7 @@ prepare_iseq_build(rb_iseq_t *iseq,
 
     iseq->compile_data = ALLOC(struct iseq_compile_data);
     MEMZERO(iseq->compile_data, struct iseq_compile_data, 1);
-    iseq->compile_data->mark_ary = rb_ary_new();
-    RBASIC(iseq->compile_data->mark_ary)->klass = 0;
+    iseq->compile_data->mark_ary = rb_ary_tmp_new(3);
 
     iseq->compile_data->storage_head = iseq->compile_data->storage_current =
       (struct iseq_compile_data_storage *)
@@ -190,6 +263,7 @@ prepare_iseq_build(rb_iseq_t *iseq,
     iseq->compile_data->storage_head->buff =
       (char *)(&iseq->compile_data->storage_head->buff + 1);
     iseq->compile_data->option = option;
+    iseq->compile_data->last_coverable_line = -1;
 
     set_relation(iseq, parent);
 
@@ -298,31 +372,31 @@ make_compile_option_value(rb_compile_option_t *option)
 }
 
 VALUE
-rb_iseq_new(NODE *node, VALUE name, VALUE filename,
+rb_iseq_new(NODE *node, VALUE name, VALUE filename, VALUE filepath,
 	      VALUE parent, VALUE type)
 {
-    return rb_iseq_new_with_opt(node, name, filename, parent, type,
+    return rb_iseq_new_with_opt(node, name, filename, filepath, INT2FIX(0), parent, type,
 				&COMPILE_OPTION_DEFAULT);
 }
 
 VALUE
-rb_iseq_new_top(NODE *node, VALUE name, VALUE filename, VALUE parent)
+rb_iseq_new_top(NODE *node, VALUE name, VALUE filename, VALUE filepath, VALUE parent)
 {
-    return rb_iseq_new_with_opt(node, name, filename, parent, ISEQ_TYPE_TOP,
+    return rb_iseq_new_with_opt(node, name, filename, filepath, INT2FIX(0), parent, ISEQ_TYPE_TOP,
 				&COMPILE_OPTION_DEFAULT);
 }
 
 VALUE
-rb_iseq_new_main(NODE *node, VALUE filename)
+rb_iseq_new_main(NODE *node, VALUE filename, VALUE filepath)
 {
     rb_thread_t *th = GET_THREAD();
     VALUE parent = th->base_block->iseq->self;
-    return rb_iseq_new_with_opt(node, rb_str_new2("<main>"), filename,
+    return rb_iseq_new_with_opt(node, rb_str_new2("<main>"), filename, filepath, INT2FIX(0),
 				parent, ISEQ_TYPE_MAIN, &COMPILE_OPTION_DEFAULT);
 }
 
 static VALUE
-rb_iseq_new_with_bopt_and_opt(NODE *node, VALUE name, VALUE filename,
+rb_iseq_new_with_bopt_and_opt(NODE *node, VALUE name, VALUE filename, VALUE filepath, VALUE line_no,
 				VALUE parent, VALUE type, VALUE bopt,
 				const rb_compile_option_t *option)
 {
@@ -332,26 +406,28 @@ rb_iseq_new_with_bopt_and_opt(NODE *node, VALUE name, VALUE filename,
     GetISeqPtr(self, iseq);
     iseq->self = self;
 
-    prepare_iseq_build(iseq, name, filename, parent, type, bopt, option);
+    prepare_iseq_build(iseq, name, filename, filepath, line_no, parent, type, bopt, option);
     rb_iseq_compile_node(self, node);
     cleanup_iseq_build(iseq);
     return self;
 }
 
 VALUE
-rb_iseq_new_with_opt(NODE *node, VALUE name, VALUE filename,
+rb_iseq_new_with_opt(NODE *node, VALUE name, VALUE filename, VALUE filepath, VALUE line_no,
 		     VALUE parent, VALUE type,
 		     const rb_compile_option_t *option)
 {
-    return rb_iseq_new_with_bopt_and_opt(node, name, filename, parent, type,
+    /* TODO: argument check */
+    return rb_iseq_new_with_bopt_and_opt(node, name, filename, filepath, line_no, parent, type,
 					   Qfalse, option);
 }
 
 VALUE
-rb_iseq_new_with_bopt(NODE *node, VALUE name, VALUE filename,
+rb_iseq_new_with_bopt(NODE *node, VALUE name, VALUE filename, VALUE filepath, VALUE line_no,
 		       VALUE parent, VALUE type, VALUE bopt)
 {
-    return rb_iseq_new_with_bopt_and_opt(node, name, filename, parent, type,
+    /* TODO: argument check */
+    return rb_iseq_new_with_bopt_and_opt(node, name, filename, filepath, line_no, parent, type,
 					   bopt, &COMPILE_OPTION_DEFAULT);
 }
 
@@ -365,7 +441,7 @@ iseq_load(VALUE self, VALUE data, VALUE parent, VALUE opt)
     VALUE iseqval = iseq_alloc(self);
 
     VALUE magic, version1, version2, format_type, misc;
-    VALUE name, filename;
+    VALUE name, filename, filepath, line_no;
     VALUE type, body, locals, args, exception;
 
     VALUE iseq_type;
@@ -375,7 +451,7 @@ iseq_load(VALUE self, VALUE data, VALUE parent, VALUE opt)
     int i = 0;
 
     /* [magic, major_version, minor_version, format_type, misc,
-     *  name, filename,
+     *  name, filename, line_no,
      *  type, locals, args, exception_table, body]
      */
 
@@ -389,6 +465,9 @@ iseq_load(VALUE self, VALUE data, VALUE parent, VALUE opt)
 
     name        = CHECK_STRING(rb_ary_entry(data, i++));
     filename    = CHECK_STRING(rb_ary_entry(data, i++));
+    filepath    = rb_ary_entry(data, i++);
+    filepath    = NIL_P(filepath) ? Qnil : CHECK_STRING(filepath);
+    line_no     = CHECK_INTEGER(rb_ary_entry(data, i++));
 
     type        = CHECK_SYMBOL(rb_ary_entry(data, i++));
     locals      = CHECK_ARRAY(rb_ary_entry(data, i++));
@@ -430,7 +509,7 @@ iseq_load(VALUE self, VALUE data, VALUE parent, VALUE opt)
     }
 
     make_compile_option(&option, opt);
-    prepare_iseq_build(iseq, name, filename,
+    prepare_iseq_build(iseq, name, filename, filepath, line_no,
 		       parent, iseq_type, 0, &option);
 
     rb_iseq_build_from_ary(iseq, locals, args, exception, body);
@@ -449,17 +528,16 @@ iseq_s_load(int argc, VALUE *argv, VALUE self)
 }
 
 VALUE
-ruby_iseq_load(VALUE data, VALUE parent, VALUE opt)
+rb_iseq_load(VALUE data, VALUE parent, VALUE opt)
 {
     return iseq_load(rb_cISeq, data, parent, opt);
 }
 
 static NODE *
-compile_string(VALUE str, VALUE file, VALUE line)
+parse_string(VALUE str, const char *file, int line)
 {
     VALUE parser = rb_parser_new();
-    NODE *node = rb_parser_compile_string(parser, StringValueCStr(file),
-					  str, NUM2INT(line));
+    NODE *node = rb_parser_compile_string(parser, file, str, line);
 
     if (!node) {
 	rb_exc_raise(GET_THREAD()->errinfo);	/* TODO: check err */
@@ -468,20 +546,22 @@ compile_string(VALUE str, VALUE file, VALUE line)
 }
 
 VALUE
-rb_iseq_compile_with_option(VALUE src, VALUE file, VALUE line, VALUE opt)
+rb_iseq_compile_with_option(VALUE src, VALUE file, VALUE filepath, VALUE line, VALUE opt)
 {
     rb_compile_option_t option;
-    NODE *node = compile_string(StringValue(src), file, line);
+    const char *fn = StringValueCStr(file);
+    int ln = NUM2INT(line);
+    NODE *node = parse_string(StringValue(src), fn, ln);
     rb_thread_t *th = GET_THREAD();
     make_compile_option(&option, opt);
 
     if (th->base_block && th->base_block->iseq) {
 	return rb_iseq_new_with_opt(node, th->base_block->iseq->name,
-				    file, th->base_block->iseq->self,
+				    file, filepath, line, th->base_block->iseq->self,
 				    ISEQ_TYPE_EVAL, &option);
     }
     else {
-	return rb_iseq_new_with_opt(node, rb_str_new2("<compiled>"), file, Qfalse,
+	return rb_iseq_new_with_opt(node, rb_str_new2("<compiled>"), file, filepath, line, Qfalse,
 				    ISEQ_TYPE_TOP, &option);
     }
 }
@@ -489,21 +569,21 @@ rb_iseq_compile_with_option(VALUE src, VALUE file, VALUE line, VALUE opt)
 VALUE
 rb_iseq_compile(VALUE src, VALUE file, VALUE line)
 {
-    return rb_iseq_compile_with_option(src, file, line, Qnil);
+    return rb_iseq_compile_with_option(src, file, Qnil, line, Qnil);
 }
 
 static VALUE
 iseq_s_compile(int argc, VALUE *argv, VALUE self)
 {
-    VALUE src, file = Qnil, line = INT2FIX(1), opt = Qnil;
+    VALUE src, file = Qnil, path = Qnil, line = INT2FIX(1), opt = Qnil;
 
     rb_secure(1);
 
-    rb_scan_args(argc, argv, "13", &src, &file, &line, &opt);
-    file = file == Qnil ? rb_str_new2("<compiled>") : file;
-    line = line == Qnil ? INT2FIX(1) : line;
+    rb_scan_args(argc, argv, "14", &src, &file, &path, &line, &opt);
+    if (NIL_P(file)) file = rb_str_new2("<compiled>");
+    if (NIL_P(line)) line = INT2FIX(1);
 
-    return rb_iseq_compile_with_option(src, file, line, opt);
+    return rb_iseq_compile_with_option(src, file, path, line, opt);
 }
 
 static VALUE
@@ -526,7 +606,7 @@ iseq_s_compile_file(int argc, VALUE *argv, VALUE self)
     parser = rb_parser_new();
     node = rb_parser_compile_file(parser, fname, f, NUM2INT(line));
     make_compile_option(&option, opt);
-    return rb_iseq_new_with_opt(node, rb_str_new2("<main>"), file, Qfalse,
+    return rb_iseq_new_with_opt(node, rb_str_new2("<main>"), file, file, line, Qfalse,
 				ISEQ_TYPE_TOP, &option);
 }
 
@@ -592,7 +672,7 @@ iseq_to_a(VALUE self)
 int
 rb_iseq_first_lineno(rb_iseq_t *iseq)
 {
-    return iseq->insn_info_table[0].line_no;
+    return FIX2INT(iseq->line_no);
 }
 
 /* TODO: search algorithm is brute force.
@@ -647,8 +727,8 @@ find_prev_line_no(rb_iseq_t *iseqdat, unsigned long pos)
 
 static VALUE
 insn_operand_intern(rb_iseq_t *iseq,
-		    int insn, int op_no, VALUE op,
-		    int len, int pos, VALUE *pnop, VALUE child)
+		    VALUE insn, int op_no, VALUE op,
+		    int len, size_t pos, VALUE *pnop, VALUE child)
 {
     const char *types = insn_op_types(insn);
     char type = types[op_no];
@@ -666,7 +746,7 @@ insn_operand_intern(rb_iseq_t *iseq,
       case TS_LINDEX:
 	{
 	    rb_iseq_t *ip = iseq->local_iseq;
-	    int lidx = ip->local_size - op;
+	    int lidx = ip->local_size - (int)op;
 	    const char *name = rb_id2name(ip->local_table[lidx]);
 
 	    if (name) {
@@ -680,7 +760,7 @@ insn_operand_intern(rb_iseq_t *iseq,
       case TS_DINDEX:{
 	if (insn == BIN(getdynamic) || insn == BIN(setdynamic)) {
 	    rb_iseq_t *ip = iseq;
-	    int level = *pnop, i;
+	    VALUE level = *pnop, i;
 	    const char *name;
 	    for (i = 0; i < level; i++) {
 		ip = ip->parent_iseq;
@@ -701,16 +781,7 @@ insn_operand_intern(rb_iseq_t *iseq,
 	op = ID2SYM(op);
 
       case TS_VALUE:		/* VALUE */
-	if (hidden_obj_p(op)) {
-	    switch (BUILTIN_TYPE(op)) {
-	      case T_STRING:
-		op = rb_str_replace(rb_str_new(0, 0), op);
-		break;
-	      case T_ARRAY:
-		op = rb_ary_replace(rb_ary_new2(0), op);
-		break;
-	    }
-	}
+	op = obj_resurrect(op);
 	ret = rb_inspect(op);
 	if (CLASS_OF(op) == rb_cISeq) {
 	    rb_ary_push(child, op);
@@ -733,13 +804,13 @@ insn_operand_intern(rb_iseq_t *iseq,
 	}
       case TS_GENTRY:
 	{
-	    struct global_entry *entry = (struct global_entry *)op;
+	    struct rb_global_entry *entry = (struct rb_global_entry *)op;
 	    ret = rb_str_dup(rb_id2str(entry->id));
 	}
 	break;
 
       case TS_IC:
-	ret = rb_str_new2("<ic>");
+	ret = rb_sprintf("<ic:%"PRIdPTRDIFF">", (struct iseq_inline_cache_entry *)op - iseq->ic_entries);
 	break;
 
       case TS_CDHASH:
@@ -760,11 +831,11 @@ insn_operand_intern(rb_iseq_t *iseq,
  * Disassemble a instruction
  * Iseq -> Iseq inspect object
  */
-VALUE
-rb_iseq_disasm_insn(VALUE ret, VALUE *iseq, int pos,
+int
+rb_iseq_disasm_insn(VALUE ret, VALUE *iseq, size_t pos,
 		    rb_iseq_t *iseqdat, VALUE child)
 {
-    int insn = iseq[pos];
+    VALUE insn = iseq[pos];
     int len = insn_len(insn);
     int j;
     const char *types = insn_op_types(insn);
@@ -773,10 +844,10 @@ rb_iseq_disasm_insn(VALUE ret, VALUE *iseq, int pos,
 
     insn_name_buff = insn_name(insn);
     if (1) {
-	rb_str_catf(str, "%04d %-16s ", pos, insn_name_buff);
+	rb_str_catf(str, "%04"PRIdSIZE" %-16s ", pos, insn_name_buff);
     }
     else {
-	rb_str_catf(str, "%04d %-16.*s ", pos,
+	rb_str_catf(str, "%04"PRIdSIZE" %-16.*s ", pos,
 		    (int)strcspn(insn_name_buff, "_"), insn_name_buff);
     }
 
@@ -853,6 +924,7 @@ rb_iseq_disasm(VALUE self)
     int i;
     long l;
     ID *tbl;
+    size_t n;
     enum {header_minlen = 72};
 
     rb_secure(1);
@@ -932,8 +1004,8 @@ rb_iseq_disasm(VALUE self)
     }
 
     /* show each line */
-    for (i = 0; i < size;) {
-	i += rb_iseq_disasm_insn(str, iseq, i, iseqdat, child);
+    for (n = 0; n < size;) {
+	n += rb_iseq_disasm_insn(str, iseq, n, iseqdat, child);
     }
 
     for (i = 0; i < RARRAY_LEN(child); i++) {
@@ -947,18 +1019,24 @@ rb_iseq_disasm(VALUE self)
 static VALUE
 iseq_s_disasm(VALUE klass, VALUE body)
 {
-    extern NODE *rb_method_body(VALUE body);
-    NODE *node;
     VALUE ret = Qnil;
+    rb_iseq_t *iseq;
+    extern rb_iseq_t *rb_method_get_iseq(VALUE body);
 
     rb_secure(1);
 
-    if ((node = rb_method_body(body)) != 0) {
-	if (nd_type(node) == RUBY_VM_METHOD_NODE) {
- 	    VALUE iseqval = (VALUE)node->nd_body;
+    if (rb_obj_is_proc(body)) {
+	rb_proc_t *proc;
+	VALUE iseqval;
+	GetProcPtr(body, proc);
+	iseqval = proc->block.iseq->self;
+	if (RUBY_VM_NORMAL_ISEQ_P(iseqval)) {
 	    ret = rb_iseq_disasm(iseqval);
 	}
     }
+    else if ((iseq = rb_method_get_iseq(body)) != 0) {
+	ret = rb_iseq_disasm(iseq->self);
+    }
 
     return ret;
 }
@@ -981,12 +1059,12 @@ ruby_node_name(int node)
   sym_##name = ID2SYM(rb_intern(#name))
 
 static VALUE
-register_label(struct st_table *table, int idx)
+register_label(struct st_table *table, unsigned long idx)
 {
     VALUE sym;
     char buff[8 + (sizeof(idx) * CHAR_BIT * 32 / 100)];
 
-    snprintf(buff, sizeof(buff), "label_%u", idx);
+    snprintf(buff, sizeof(buff), "label_%lu", idx);
     sym = ID2SYM(rb_intern(buff));
     st_insert(table, idx, sym);
     return sym;
@@ -1012,7 +1090,7 @@ exception_type2symbol(VALUE type)
 static int
 cdhash_each(VALUE key, VALUE value, VALUE ary)
 {
-    rb_ary_push(ary, key);
+    rb_ary_push(ary, obj_resurrect(key));
     rb_ary_push(ary, value);
     return ST_CONTINUE;
 }
@@ -1020,7 +1098,8 @@ cdhash_each(VALUE key, VALUE value, VALUE ary)
 static VALUE
 iseq_data_to_ary(rb_iseq_t *iseq)
 {
-    int i, pos, line = 0;
+    long i, pos;
+    int line = 0;
     VALUE *seq;
 
     VALUE val = rb_ary_new();
@@ -1132,7 +1211,7 @@ iseq_data_to_ary(rb_iseq_t *iseq)
 	for (j=0; j<len-1; j++, seq++) {
 	    switch (insn_op_type(insn, j)) {
 	      case TS_OFFSET: {
-		unsigned int idx = nseq - iseq->iseq + *seq;
+		unsigned long idx = nseq - iseq->iseq + *seq;
 		rb_ary_push(ary, register_label(labels_table, idx));
 		break;
 	      }
@@ -1142,7 +1221,7 @@ iseq_data_to_ary(rb_iseq_t *iseq)
 		rb_ary_push(ary, INT2FIX(*seq));
 		break;
 	      case TS_VALUE:
-		rb_ary_push(ary, *seq);
+		rb_ary_push(ary, obj_resurrect(*seq));
 		break;
 	      case TS_ISEQ:
 		{
@@ -1158,12 +1237,14 @@ iseq_data_to_ary(rb_iseq_t *iseq)
 		break;
 	      case TS_GENTRY:
 		{
-		    struct global_entry *entry = (struct global_entry *)*seq;
+		    struct rb_global_entry *entry = (struct rb_global_entry *)*seq;
 		    rb_ary_push(ary, ID2SYM(entry->id));
 		}
 		break;
-	      case TS_IC:
-		rb_ary_push(ary, Qnil);
+	      case TS_IC: {
+		  struct iseq_inline_cache_entry *ic = (struct iseq_inline_cache_entry *)*seq;
+		    rb_ary_push(ary, INT2FIX(ic - iseq->ic_entries));
+	        }
 		break;
 	      case TS_ID:
 		rb_ary_push(ary, ID2SYM(*seq));
@@ -1178,7 +1259,7 @@ iseq_data_to_ary(rb_iseq_t *iseq)
 
 		    for (i=0; i<RARRAY_LEN(val); i+=2) {
 			VALUE pos = FIX2INT(rb_ary_entry(val, i+1));
-			unsigned int idx = nseq - iseq->iseq + pos;
+			unsigned long idx = nseq - iseq->iseq + pos;
 
 			rb_ary_store(val, i+1,
 				     register_label(labels_table, idx));
@@ -1241,18 +1322,20 @@ iseq_data_to_ary(rb_iseq_t *iseq)
     rb_hash_aset(misc, ID2SYM(rb_intern("local_size")), INT2FIX(iseq->local_size));
     rb_hash_aset(misc, ID2SYM(rb_intern("stack_max")), INT2FIX(iseq->stack_max));
 
-    /* 
+    /*
      * [:magic, :major_version, :minor_version, :format_type, :misc,
-     *  :name, :filename, :type, :locals, :args,
+     *  :name, :filename, :filepath, :line_no, :type, :locals, :args,
      *  :catch_table, :bytecode]
      */
     rb_ary_push(val, rb_str_new2("YARVInstructionSequence/SimpleDataFormat"));
-    rb_ary_push(val, INT2FIX(1));
-    rb_ary_push(val, INT2FIX(1));
+    rb_ary_push(val, INT2FIX(1)); /* major */
+    rb_ary_push(val, INT2FIX(2)); /* minor */
     rb_ary_push(val, INT2FIX(1));
     rb_ary_push(val, misc);
     rb_ary_push(val, iseq->name);
     rb_ary_push(val, iseq->filename);
+    rb_ary_push(val, iseq->filepath);
+    rb_ary_push(val, iseq->line_no);
     rb_ary_push(val, type);
     rb_ary_push(val, locals);
     rb_ary_push(val, args);
@@ -1275,16 +1358,83 @@ rb_iseq_clone(VALUE iseqval, VALUE newcbase)
     if (!iseq1->orig) {
 	iseq1->orig = iseqval;
     }
+    if (iseq0->local_iseq == iseq0) {
+	iseq1->local_iseq = iseq1;
+    }
     if (newcbase) {
 	iseq1->cref_stack = NEW_BLOCK(newcbase);
 	if (iseq0->cref_stack->nd_next) {
 	    iseq1->cref_stack->nd_next = iseq0->cref_stack->nd_next;
 	}
+	iseq1->klass = newcbase;
     }
 
     return newiseq;
 }
 
+VALUE
+rb_iseq_parameters(const rb_iseq_t *iseq, int is_proc)
+{
+    int i, r, s;
+    VALUE a, args = rb_ary_new2(iseq->arg_size);
+    ID req, opt, rest, block;
+#define PARAM_TYPE(type) rb_ary_push(a = rb_ary_new2(2), ID2SYM(type))
+#define PARAM_ID(i) iseq->local_table[i]
+#define PARAM(i, type) (		      \
+	PARAM_TYPE(type),		      \
+	rb_id2name(PARAM_ID(i)) ?	      \
+	rb_ary_push(a, ID2SYM(PARAM_ID(i))) : \
+	a)
+
+    CONST_ID(req, "req");
+    CONST_ID(opt, "opt");
+    if (is_proc) {
+	for (i = 0; i < iseq->argc; i++) {
+	    PARAM_TYPE(opt);
+	    rb_ary_push(a, rb_id2name(PARAM_ID(i)) ? ID2SYM(PARAM_ID(i)) : Qnil);
+	    rb_ary_push(args, a);
+	}
+    }
+    else {
+	for (i = 0; i < iseq->argc; i++) {
+	    rb_ary_push(args, PARAM(i, req));
+	}
+    }
+    r = iseq->arg_rest != -1 ? iseq->arg_rest :
+	iseq->arg_post_len > 0 ? iseq->arg_post_start :
+	iseq->arg_block != -1 ? iseq->arg_block :
+	iseq->arg_size;
+    for (s = i; i < r; i++) {
+	PARAM_TYPE(opt);
+	if (rb_id2name(PARAM_ID(i))) {
+	    rb_ary_push(a, ID2SYM(PARAM_ID(i)));
+	}
+	rb_ary_push(args, a);
+    }
+    if (iseq->arg_rest != -1) {
+	CONST_ID(rest, "rest");
+	rb_ary_push(args, PARAM(iseq->arg_rest, rest));
+    }
+    r = iseq->arg_post_start + iseq->arg_post_len;
+    if (is_proc) {
+	for (i = iseq->arg_post_start; i < r; i++) {
+	    PARAM_TYPE(opt);
+	    rb_ary_push(a, rb_id2name(PARAM_ID(i)) ? ID2SYM(PARAM_ID(i)) : Qnil);
+	    rb_ary_push(args, a);
+	}
+    }
+    else {
+	for (i = iseq->arg_post_start; i < r; i++) {
+	    rb_ary_push(args, PARAM(i, req));
+	}
+    }
+    if (iseq->arg_block != -1) {
+	CONST_ID(block, "block");
+	rb_ary_push(args, PARAM(iseq->arg_block, block));
+    }
+    return args;
+}
+
 /* ruby2cext */
 
 VALUE
@@ -1296,9 +1446,10 @@ rb_iseq_build_for_ruby2cext(
     const VALUE *arg_opt_table,
     const struct iseq_catch_table_entry *catch_table,
     const char *name,
-    const char *filename)
+    const char *filename,
+    const unsigned short line_no)
 {
-    int i;
+    unsigned long i;
     VALUE iseqval = iseq_alloc(rb_cISeq);
     rb_iseq_t *iseq;
     GetISeqPtr(iseqval, iseq);
@@ -1307,7 +1458,9 @@ rb_iseq_build_for_ruby2cext(
     *iseq = *iseq_template;
     iseq->name = rb_str_new2(name);
     iseq->filename = rb_str_new2(filename);
-    iseq->mark_ary = rb_ary_new();
+    iseq->line_no = line_no;
+    iseq->mark_ary = rb_ary_tmp_new(3);
+    OBJ_UNTRUST(iseq->mark_ary);
     iseq->self = iseqval;
 
     iseq->iseq = ALLOC_N(VALUE, iseq->iseq_size);
