@@ -1,7 +1,7 @@
 #
-#   irb/completor.rb - 
+#   irb/completor.rb -
 #   	$Release Version: 0.9$
-#   	$Revision: 23233 $
+#   	$Revision: 32919 $
 #   	by Keiju ISHITSUKA(keiju@ishitsuka.com)
 #       From Original Idea of shugo@ruby-lang.org
 #
@@ -11,20 +11,20 @@ require "readline"
 module IRB
   module InputCompletor
 
-    @RCS_ID='-$Id: completion.rb 23233 2009-04-19 13:35:47Z yugui $-'
+    @RCS_ID='-$Id: completion.rb 32919 2011-08-11 00:38:36Z yugui $-'
 
     ReservedWords = [
       "BEGIN", "END",
-      "alias", "and", 
-      "begin", "break", 
+      "alias", "and",
+      "begin", "break",
       "case", "class",
       "def", "defined", "do",
       "else", "elsif", "end", "ensure",
-      "false", "for", 
-      "if", "in", 
-      "module", 
+      "false", "for",
+      "if", "in",
+      "module",
       "next", "nil", "not",
-      "or", 
+      "or",
       "redo", "rescue", "retry", "return",
       "self", "super",
       "then", "true",
@@ -32,13 +32,21 @@ module IRB
       "when", "while",
       "yield",
     ]
-      
+
     CompletionProc = proc { |input|
       bind = IRB.conf[:MAIN_CONTEXT].workspace.binding
-      
+
 #      puts "input: #{input}"
 
       case input
+      when /^((["'`]).*\2)\.([^.]*)$/
+	# String
+	receiver = $1
+	message = $3
+
+	candidates = String.instance_methods.collect{|m| m.to_s}
+	select_message(receiver, message, candidates)
+
       when /^(\/[^\/]*\/)\.([^.]*)$/
 	# Regexp
 	receiver = $1
@@ -63,7 +71,7 @@ module IRB
 	candidates = Proc.instance_methods.collect{|m| m.to_s}
 	candidates |= Hash.instance_methods.collect{|m| m.to_s}
 	select_message(receiver, message, candidates)
-	
+
       when /^(:[^:.]*)$/
  	# Symbol
 	if Symbol.respond_to?(:all_symbols)
@@ -80,10 +88,11 @@ module IRB
 	candidates = Object.constants.collect{|m| m.to_s}
 	candidates.grep(/^#{receiver}/).collect{|e| "::" + e}
 
-      when /^(((::)?[A-Z][^:.\(]*)+)::?([^:.]*)$/
+#      when /^(((::)?[A-Z][^:.\(]*)+)::?([^:.]*)$/
+      when /^([A-Z].*)::([^:.]*)$/
 	# Constant or class methods
 	receiver = $1
-	message = Regexp.quote($4)
+	message = Regexp.quote($2)
 	begin
 	  candidates = eval("#{receiver}.constants.collect{|m| m.to_s}", bind)
 	  candidates |= eval("#{receiver}.methods.collect{|m| m.to_s}", bind)
@@ -129,19 +138,18 @@ module IRB
 	candidates = global_variables.collect{|m| m.to_s}.grep(regmessage)
 
 #      when /^(\$?(\.?[^.]+)+)\.([^.]*)$/
-      when /^((\.?[^.]+)+)\.([^.]*)$/
+#      when /^((\.?[^.]+)+)\.([^.]*)$/
+      when /^([^."].*)\.([^.]*)$/
 	# variable
 	receiver = $1
-	message = Regexp.quote($3)
+	message = Regexp.quote($2)
 
 	gv = eval("global_variables", bind).collect{|m| m.to_s}
 	lv = eval("local_variables", bind).collect{|m| m.to_s}
 	cv = eval("self.class.constants", bind).collect{|m| m.to_s}
-	
-	if (gv | lv | cv).include?(receiver)
-	  # foo.func and foo is local var.
-	  candidates = eval("#{receiver}.methods", bind).collect{|m| m.to_s}
-	elsif /^[A-Z]/ =~ receiver and /\./ !~ receiver
+
+	if (gv | lv | cv).include?(receiver) or /^[A-Z]/ =~ receiver && /\./ !~ receiver
+	  # foo.func and foo is local var. OR
 	  # Foo::Bar.func
 	  begin
 	    candidates = eval("#{receiver}.methods", bind).collect{|m| m.to_s}
@@ -157,7 +165,7 @@ module IRB
 	    rescue Exception
 	      name = ""
 	    end
-	    next if name != "IRB::Context" and 
+	    next if name != "IRB::Context" and
 	      /^(IRB|SLex|RubyLex|RubyToken)/ =~ name
 	    candidates.concat m.instance_methods(false).collect{|x| x.to_s}
 	  }
@@ -177,7 +185,7 @@ module IRB
 
       else
 	candidates = eval("methods | private_methods | local_variables | self.class.constants", bind).collect{|m| m.to_s}
-			  
+
 	(candidates|ReservedWords).grep(/^#{Regexp.quote(input)}/)
       end
     }
@@ -201,7 +209,8 @@ module IRB
 end
 
 if Readline.respond_to?("basic_word_break_characters=")
-  Readline.basic_word_break_characters= " \t\n\"\\'`><=;|&{("
+#  Readline.basic_word_break_characters= " \t\n\"\\'`><=;|&{("
+  Readline.basic_word_break_characters= " \t\n`><=;|&{("
 end
 Readline.completion_append_character = nil
 Readline.completion_proc = IRB::InputCompletor::CompletionProc
