@@ -23,7 +23,7 @@ module WEBrick
     attr_reader :request_method, :unparsed_uri, :http_version
 
     # Request-URI
-    attr_reader :request_uri, :host, :port, :path
+    attr_reader :request_uri, :path
     attr_accessor :script_name, :path_info, :query_string
 
     # Header and entity body
@@ -69,6 +69,9 @@ module WEBrick
 
       @remaining_size = nil
       @socket = nil
+
+      @forwarded_proto = @forwarded_host = @forwarded_port =
+        @forwarded_server = @forwarded_for = nil
     end
 
     def parse(socket=nil)
@@ -95,6 +98,7 @@ module WEBrick
       return if @unparsed_uri == "*"
 
       begin
+        setup_forwarded_info
         @request_uri = parse_uri(@unparsed_uri)
         @path = HTTPUtils::unescape(@request_uri.path)
         @path = HTTPUtils::normalize_path(@path)
@@ -153,6 +157,26 @@ module WEBrick
       }
     end
 
+    def host
+      return @forwarded_host || @host
+    end
+
+    def port
+      return @forwarded_port || @port
+    end
+
+    def server_name
+      return @forwarded_server || @config[:ServerName]
+    end
+
+    def remote_ip
+      return self["client-ip"] || @forwarded_for || @peeraddr[3]
+    end
+
+    def ssl?
+      return @request_uri.scheme == "https"
+    end
+
     def keep_alive?
       @keep_alive
     end
@@ -220,12 +244,12 @@ module WEBrick
 
     def read_request_line(socket)
       @request_line = read_line(socket, 1024) if socket
-      if @request_line.size >= 1024 and @request_line[-1, 1] != LF
+      if @request_line.bytesize >= 1024 and @request_line[-1, 1] != LF
         raise HTTPStatus::RequestURITooLarge
       end
       @request_time = Time.now
       raise HTTPStatus::EOFError unless @request_line
-      if /^(\S+)\s+(\S+)(?:\s+HTTP\/(\d+\.\d+))?\r?\n/mo =~ @request_line
+      if /^(\S+)\s+(\S++)(?:\s+HTTP\/(\d+\.\d+))?\r?\n/mo =~ @request_line
         @request_method = $1
         @unparsed_uri   = $2
         @http_version   = HTTPVersion.new($3 ? $3 : "0.9")
@@ -242,11 +266,7 @@ module WEBrick
           @raw_header << line
         end
       end
-      begin
-        @header = HTTPUtils::parse_header(@raw_header.join)
-      rescue => ex
-        raise  HTTPStatus::BadRequest, ex.message
-      end
+      @header = HTTPUtils::parse_header(@raw_header.join)
     end
 
     def parse_uri(str, scheme="http")
@@ -255,7 +275,9 @@ module WEBrick
       end
       uri = URI::parse(str)
       return uri if uri.absolute?
-      if self["host"]
+      if @forwarded_host
+        host, port = @forwarded_host, @forwarded_port
+      elsif self["host"]
         pattern = /\A(#{URI::REGEXP::PATTERN::HOST})(?::(\d+))?\z/n
         host, port = *self['host'].scan(pattern)[0]
       elsif @addr.size > 0
@@ -263,7 +285,7 @@ module WEBrick
       else
         host, port = @config[:ServerName], @config[:Port]
       end
-      uri.scheme = scheme
+      uri.scheme = @forwarded_proto || scheme
       uri.host = host
       uri.port = port ? port.to_i : nil
       return URI::parse(uri.to_s)
@@ -278,10 +300,10 @@ module WEBrick
         end
       elsif self['content-length'] || @remaining_size
         @remaining_size ||= self['content-length'].to_i
-        while @remaining_size > 0 
+        while @remaining_size > 0
           sz = [@buffer_size, @remaining_size].min
           break unless buf = read_data(socket, sz)
-          @remaining_size -= buf.size
+          @remaining_size -= buf.bytesize
           block.call(buf)
         end
         if @remaining_size > 0 && @socket.eof?
@@ -308,7 +330,7 @@ module WEBrick
       chunk_size, = read_chunk_size(socket)
       while chunk_size > 0
         data = read_data(socket, chunk_size) # read chunk-data
-        if data.nil? || data.size != chunk_size
+        if data.nil? || data.bytesize != chunk_size
           raise BadRequest, "bad chunk data size."
         end
         read_line(socket)                    # skip CRLF
@@ -356,5 +378,25 @@ module WEBrick
         raise HTTPStatus::BadRequest, ex.message
       end
     end
+
+    PrivateNetworkRegexp = /
+      ^unknown$|
+      ^((::ffff:)?127.0.0.1|::1)$|
+      ^(::ffff:)?(10|172\.(1[6-9]|2[0-9]|3[01])|192\.168)\.
+    /ixo
+
+    def setup_forwarded_info
+      @forwarded_server = self["x-forwarded-server"]
+      @forwarded_proto = self["x-forwarded-proto"]
+      if host_port = self["x-forwarded-host"]
+        @forwarded_host, tmp = host_port.split(":", 2)
+        @forwarded_port = (tmp || (@forwarded_proto == "https" ? 443 : 80)).to_i
+      end
+      if addrs = self["x-forwarded-for"]
+        addrs = addrs.split(",").collect(&:strip)
+        addrs.reject!{|ip| PrivateNetworkRegexp =~ ip }
+        @forwarded_for = addrs.first
+      end
+    end
   end
 end
