@@ -2,8 +2,7 @@
 
 #include <math.h>
 
-double hypot(x,y)
-    double x, y;
+double hypot(double x, double y)
 {
     if (x < 0) x = -x;
     if (y < 0) y = -y;
