@@ -2,7 +2,7 @@
 
   sprintf.c -
 
-  $Author: nobu $
+  $Author: yugui $
   created at: Fri Oct 15 10:39:26 JST 1993
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -25,13 +25,13 @@
 #define BITSPERDIG (SIZEOF_BDIGITS*CHAR_BIT)
 #define EXTENDSIGN(n, l) (((~0 << (n)) >> (((n)*(l)) % BITSPERDIG)) & ~(~0 << (n)))
 
-static void fmt_setup(char*,int,int,int,int);
+static void fmt_setup(char*,size_t,int,int,int,int);
 
 static char*
 remove_sign_bits(char *str, int base)
 {
     char *s, *t;
-    
+
     s = t = str;
 
     if (base == 16) {
@@ -83,10 +83,12 @@ sign_bits(int base, const char *p)
 #define FPREC0 128
 
 #define CHECK(l) do {\
+    int cr = ENC_CODERANGE(result);\
     while (blen + (l) >= bsiz) {\
 	bsiz*=2;\
     }\
     rb_str_resize(result, bsiz);\
+    ENC_CODERANGE_SET(result, cr);\
     buf = RSTRING_PTR(result);\
 } while (0)
 
@@ -103,18 +105,29 @@ sign_bits(int base, const char *p)
 } while (0)
 
 #define GETARG() (nextvalue != Qundef ? nextvalue : \
-    posarg < 0 ? \
+    posarg == -1 ? \
     (rb_raise(rb_eArgError, "unnumbered(%d) mixed with numbered", nextarg), 0) : \
+    posarg == -2 ? \
+    (rb_raise(rb_eArgError, "unnumbered(%d) mixed with named", nextarg), 0) : \
     (posarg = nextarg++, GETNTHARG(posarg)))
 
 #define GETPOSARG(n) (posarg > 0 ? \
     (rb_raise(rb_eArgError, "numbered(%d) after unnumbered(%d)", n, posarg), 0) : \
+    posarg == -2 ? \
+    (rb_raise(rb_eArgError, "numbered(%d) after named", n), 0) : \
     ((n < 1) ? (rb_raise(rb_eArgError, "invalid index - %d$", n), 0) : \
 	       (posarg = -1, GETNTHARG(n))))
 
 #define GETNTHARG(nth) \
     ((nth >= argc) ? (rb_raise(rb_eArgError, "too few arguments"), 0) : argv[nth])
 
+#define GETNAMEARG(id, name, len) ( \
+    posarg > 0 ? \
+    (rb_raise(rb_eArgError, "named%.*s after unnumbered(%d)", (len), (name), posarg), 0) : \
+    posarg == -1 ? \
+    (rb_raise(rb_eArgError, "named%.*s after numbered", (len), (name)), 0) :	\
+    (posarg = -2, rb_hash_lookup2(get_hash(&hash, argc, argv), id, Qundef)))
+
 #define GETNUM(n, val) \
     for (; p < end && rb_enc_isdigit(*p, enc); p++) {	\
 	int next_n = 10 * n + (*p - '0'); \
@@ -141,15 +154,30 @@ sign_bits(int base, const char *p)
     val = NUM2INT(tmp); \
 } while (0)
 
+static VALUE
+get_hash(volatile VALUE *hash, int argc, const VALUE *argv)
+{
+    VALUE tmp;
+
+    if (*hash != Qundef) return *hash;
+    if (argc != 2) {
+	rb_raise(rb_eArgError, "one hash required");
+    }
+    tmp = rb_check_convert_type(argv[1], T_HASH, "Hash", "to_hash");
+    if (NIL_P(tmp)) {
+	rb_raise(rb_eArgError, "one hash required");
+    }
+    return (*hash = tmp);
+}
 
 /*
  *  call-seq:
- *     format(format_string [, arguments...] )   => string
- *     sprintf(format_string [, arguments...] )  => string
- *  
+ *     format(format_string [, arguments...] )   -> string
+ *     sprintf(format_string [, arguments...] )  -> string
+ *
  *  Returns the string resulting from applying <i>format_string</i> to
  *  any additional arguments.  Within the format string, any characters
- *  other than format sequences are copied to the result. 
+ *  other than format sequences are copied to the result.
  *
  *  The syntax of a format sequence is follows.
  *
@@ -185,13 +213,13 @@ sign_bits(int base, const char *p)
  *
  *      Field |  Float Format
  *      ------+--------------------------------------------------------------
- *        e   | Convert floating point argument into exponential notation 
+ *        e   | Convert floating point argument into exponential notation
  *            | with one digit before the decimal point as [-]d.dddddde[+-]dd.
  *            | The precision specifies the number of digits after the decimal
  *            | point (defaulting to six).
  *        E   | Equivalent to `e', but uses an uppercase E to indicate
  *            | the exponent.
- *        f   | Convert floating point argument as [-]ddd.dddddd, 
+ *        f   | Convert floating point argument as [-]ddd.dddddd,
  *            | where the precision specifies the number of digits after
  *            | the decimal point.
  *        g   | Convert a floating point number using exponential form
@@ -199,6 +227,10 @@ sign_bits(int base, const char *p)
  *            | equal to the precision, or in dd.dddd form otherwise.
  *            | The precision specifies the number of significant digits.
  *        G   | Equivalent to `g', but use an uppercase `E' in exponent form.
+ *        a   | Convert floating point argument as [-]0xh.hhhhp[+-]dd,
+ *            | which is consisted from optional sign, "0x", fraction part
+ *            | as hexadecimal, "p", and exponential part as decimal.
+ *        A   | Equivalent to `a', but use uppercase `X' and `P'.
  *
  *      Field |  Other Format
  *      ------+--------------------------------------------------------------
@@ -209,14 +241,14 @@ sign_bits(int base, const char *p)
  *            | sequence contains a precision, at most that many characters
  *            | will be copied.
  *        %   | A percent sign itself will be displayed.  No argument taken.
- *     
+ *
  *  The flags modifies the behavior of the formats.
  *  The flag characters are:
  *
  *    Flag     | Applies to    | Meaning
  *    ---------+---------------+-----------------------------------------
- *    space    | bBdiouxX      | Leave a space at the start of 
- *             | eEfgG         | non-negative numbers.
+ *    space    | bBdiouxX      | Leave a space at the start of
+ *             | aAeEfgG       | non-negative numbers.
  *             | (numeric fmt) | For `o', `x', `X', `b' and `B', use
  *             |               | a minus sign with absolute value for
  *             |               | negative values.
@@ -227,19 +259,19 @@ sign_bits(int base, const char *p)
  *             |               | sprintf string.
  *    ---------+---------------+-----------------------------------------
  *     #       | bBoxX         | Use an alternative format.
- *             | eEfgG         | For the conversions `o', increase the precision
+ *             | aAeEfgG       | For the conversions `o', increase the precision
  *             |               | until the first digit will be `0' if
  *             |               | it is not formatted as complements.
  *             |               | For the conversions `x', `X', `b' and `B'
  *             |               | on non-zero, prefix the result with ``0x'',
  *             |               | ``0X'', ``0b'' and ``0B'', respectively.
- *             |               | For `e', `E', `f', `g', and 'G',
+ *             |               | For `a', `A', `e', `E', `f', `g', and 'G',
  *             |               | force a decimal point to be added,
  *             |               | even if no digits follow.
  *             |               | For `g' and 'G', do not remove trailing zeros.
  *    ---------+---------------+-----------------------------------------
  *    +        | bBdiouxX      | Add a leading plus sign to non-negative
- *             | eEfgG         | numbers.
+ *             | aAeEfgG       | numbers.
  *             | (numeric fmt) | For `o', `x', `X', `b' and `B', use
  *             |               | a minus sign with absolute value for
  *             |               | negative values.
@@ -247,13 +279,13 @@ sign_bits(int base, const char *p)
  *    -        | all           | Left-justify the result of this conversion.
  *    ---------+---------------+-----------------------------------------
  *    0 (zero) | bBdiouxX      | Pad with zeros, not spaces.
- *             | eEfgG         | For `o', `x', `X', `b' and `B', radix-1
+ *             | aAeEfgG       | For `o', `x', `X', `b' and `B', radix-1
  *             | (numeric fmt) | is used for negative numbers formatted as
  *             |               | complements.
  *    ---------+---------------+-----------------------------------------
- *    *        | all           | Use the next argument as the field width. 
+ *    *        | all           | Use the next argument as the field width.
  *             |               | If negative, left-justify the result. If the
- *             |               | asterisk is followed by a number and a dollar 
+ *             |               | asterisk is followed by a number and a dollar
  *             |               | sign, use the indicated argument as the width.
  *
  *  Examples of flags:
@@ -311,7 +343,7 @@ sign_bits(int base, const char *p)
  *   sprintf("%#g", 123.4)  #=> "123.400"
  *   sprintf("%g", 123456)  #=> "123456"
  *   sprintf("%#g", 123456) #=> "123456."
- *     
+ *
  *  The field width is an optional integer, followed optionally by a
  *  period and a precision.  The width specifies the minimum number of
  *  characters that will be written to the result for this field.
@@ -364,7 +396,7 @@ sign_bits(int base, const char *p)
  *   # precision for `e' is number of
  *   # digits after the decimal point           <------>
  *   sprintf("%20.8e", 1234.56789) #=> "      1.23456789e+03"
- *                                    
+ *
  *   # precision for `f' is number of
  *   # digits after the decimal point               <------>
  *   sprintf("%20.8f", 1234.56789) #=> "       1234.56789000"
@@ -388,6 +420,15 @@ sign_bits(int base, const char *p)
  *     sprintf("%1$*2$s %2$d", "hello", -8)       #=> "hello    -8"
  *     sprintf("%+g:% g:%-g", 1.23, 1.23, 1.23)   #=> "+1.23: 1.23:1.23"
  *     sprintf("%u", -123)                        #=> "-123"
+ *
+ *  For more complex formatting, Ruby supports a reference by name.
+ *  %<name>s style uses format style, but %{name} style doesn't.
+ *
+ *  Exapmles:
+ *    sprintf("%<foo>d : %<bar>f", { :foo => 1, :bar => 2 })
+ *      #=> 1 : 2.000000
+ *    sprintf("%{foo}f", { :foo => 1 })
+ *      # => "1f"
  */
 
 VALUE
@@ -402,9 +443,11 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
     rb_encoding *enc;
     const char *p, *end;
     char *buf;
-    int blen, bsiz;
+    long blen, bsiz;
     VALUE result;
 
+    long scanned = 0;
+    int coderange = ENC_CODERANGE_7BIT;
     int width, prec, flags = FNONE;
     int nextarg = 1;
     int posarg = 0;
@@ -412,6 +455,7 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
     VALUE nextvalue;
     VALUE tmp;
     VALUE str;
+    volatile VALUE hash = Qundef;
 
 #define CHECK_FOR_WIDTH(f)				 \
     if ((f) & FWIDTH) {					 \
@@ -442,13 +486,19 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
     rb_enc_copy(result, fmt);
     buf = RSTRING_PTR(result);
     memset(buf, 0, bsiz);
+    ENC_CODERANGE_SET(result, coderange);
 
     for (; p < end; p++) {
 	const char *t;
 	int n;
+	ID id = 0;
 
 	for (t = p; t < end && *t != '%'; t++) ;
 	PUSH(p, t - p);
+	if (coderange != ENC_CODERANGE_BROKEN && scanned < blen) {
+	    scanned += rb_str_coderange_scan_restartable(buf+scanned, buf+blen, enc, &coderange);
+	    ENC_CODERANGE_SET(result, coderange);
+	}
 	if (t >= end) {
 	    /* end of fmt string */
 	    goto sprint_exit;
@@ -513,6 +563,32 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 	    flags |= FWIDTH;
 	    goto retry;
 
+	  case '<':
+	  case '{':
+	    {
+		const char *start = p;
+		char term = (*p == '<') ? '>' : '}';
+
+		for (; p < end && *p != term; ) {
+		    p += rb_enc_mbclen(p, end, enc);
+		}
+		if (p >= end) {
+		    rb_raise(rb_eArgError, "malformed name - unmatched parenthesis");
+		}
+		if (id) {
+		    rb_raise(rb_eArgError, "name%.*s after <%s>",
+			     (int)(p - start + 1), start, rb_id2name(id));
+		}
+		id = rb_intern3(start + 1, p - start - 1, enc);
+		nextvalue = GETNAMEARG(ID2SYM(id), start, (int)(p - start + 1));
+		if (nextvalue == Qundef) {
+		    rb_raise(rb_eKeyError, "key%.*s not found", (int)(p - start + 1), start);
+		}
+		if (term == '}') goto format_s;
+		p++;
+		goto retry;
+	    }
+
 	  case '*':
 	    CHECK_FOR_WIDTH(flags);
 	    flags |= FWIDTH;
@@ -558,20 +634,21 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 	    {
 		VALUE val = GETARG();
 		VALUE tmp;
-		int c, n;
+		unsigned int c;
+		int n;
 
 		tmp = rb_check_string_type(val);
 		if (!NIL_P(tmp)) {
 		    if (rb_enc_strlen(RSTRING_PTR(tmp),RSTRING_END(tmp),enc) != 1) {
 			rb_raise(rb_eArgError, "%%c requires a character");
 		    }
-		    c = rb_enc_codepoint(RSTRING_PTR(tmp), RSTRING_END(tmp), enc);
+		    c = rb_enc_codepoint_len(RSTRING_PTR(tmp), RSTRING_END(tmp), &n, enc);
 		}
 		else {
 		    c = NUM2INT(val);
+		    n = rb_enc_codelen(c, enc);
 		}
-		n = rb_enc_codelen(c, enc);
-		if (n == 0) {
+		if (n <= 0) {
 		    rb_raise(rb_eArgError, "invalid character");
 		}
 		if (!(flags & FWIDTH)) {
@@ -596,6 +673,7 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 
 	  case 's':
 	  case 'p':
+	  format_s:
 	    {
 		VALUE arg = GETARG();
 		long len, slen;
@@ -604,6 +682,14 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 		str = rb_obj_as_string(arg);
 		if (OBJ_TAINTED(str)) tainted = 1;
 		len = RSTRING_LEN(str);
+		rb_str_set_len(result, blen);
+		if (coderange != ENC_CODERANGE_BROKEN && scanned < blen) {
+		    int cr = coderange;
+		    scanned += rb_str_coderange_scan_restartable(buf+scanned, buf+blen, enc, &cr);
+		    ENC_CODERANGE_SET(result,
+				      (cr == ENC_CODERANGE_UNKNOWN ?
+				       ENC_CODERANGE_BROKEN : (coderange = cr)));
+		}
 		enc = rb_enc_check(result, str);
 		if (flags&(FPREC|FWIDTH)) {
 		    slen = rb_enc_strlen(RSTRING_PTR(str),RSTRING_END(str),enc);
@@ -618,7 +704,7 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 		    }
 		    /* need to adjust multi-byte string pos */
 		    if ((flags&FWIDTH) && (width > slen)) {
-			width -= slen;
+			width -= (int)slen;
 			if (!(flags&FMINUS)) {
 			    CHECK(width);
 			    while (width--) {
@@ -702,7 +788,7 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 		    bignum = 1;
 		    break;
 		  case T_STRING:
-		    val = rb_str_to_inum(val, 0, Qtrue);
+		    val = rb_str_to_inum(val, 0, TRUE);
 		    goto bin_retry;
 		  case T_BIGNUM:
 		    bignum = 1;
@@ -752,8 +838,8 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 			    sc = ' ';
 			    width--;
 			}
-			sprintf(fbuf, "%%l%c", c);
-			sprintf(nbuf, fbuf, v);
+			snprintf(fbuf, sizeof(fbuf), "%%l%c", c);
+			snprintf(nbuf, sizeof(nbuf), fbuf, v);
 			s = nbuf;
 		    }
 		    else {
@@ -761,8 +847,8 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 			if (v < 0) {
 			    dots = 1;
 			}
-			sprintf(fbuf, "%%l%c", *p == 'X' ? 'x' : *p);
-			sprintf(++s, fbuf, v);
+			snprintf(fbuf, sizeof(fbuf), "%%l%c", *p == 'X' ? 'x' : *p);
+			snprintf(++s, sizeof(nbuf) - 1, fbuf, v);
 			if (v < 0) {
 			    char d = 0;
 
@@ -778,6 +864,7 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 			    }
 			}
 		    }
+		    len = (int)strlen(s);
 		}
 		else {
 		    if (sign) {
@@ -820,10 +907,10 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 			    }
 			}
 		    }
+		    len = rb_long2int(RSTRING_END(tmp) - s);
 		}
 
 		pos = -1;
-		len = strlen(s);
 		if (dots) {
 		    prec -= 2;
 		    width -= 2;
@@ -853,7 +940,7 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 		    prefix = 0;
 		}
 		if (prefix) {
-		    width -= strlen(prefix);
+		    width -= (int)strlen(prefix);
 		}
 		if ((flags & (FZERO|FMINUS|FPREC)) == FZERO) {
 		    prec = width;
@@ -874,7 +961,7 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 		}
 		if (sc) PUSH(&sc, 1);
 		if (prefix) {
-		    int plen = strlen(prefix);
+		    int plen = (int)strlen(prefix);
 		    PUSH(prefix, plen);
 		}
 		CHECK(prec - len);
@@ -909,6 +996,8 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 	  case 'G':
 	  case 'e':
 	  case 'E':
+	  case 'a':
+	  case 'A':
 	    {
 		VALUE val = GETARG();
 		double fval;
@@ -925,14 +1014,14 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 		    else {
 			expr = "Inf";
 		    }
-		    need = strlen(expr);
+		    need = (int)strlen(expr);
 		    if ((!isnan(fval) && fval < 0.0) || (flags & FPLUS))
 			need++;
 		    if ((flags & FWIDTH) && need < width)
 			need = width;
 
-		    CHECK(need);
-		    sprintf(&buf[blen], "%*s", need, "");
+		    CHECK(need + 1);
+		    snprintf(&buf[blen], need + 1, "%*s", need, "");
 		    if (flags & FMINUS) {
 			if (!isnan(fval) && fval < 0.0)
 			    buf[blen++] = '-';
@@ -940,7 +1029,7 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 			    buf[blen++] = '+';
 			else if (flags & FSPACE)
 			    blen++;
-			strncpy(&buf[blen], expr, strlen(expr));
+			memcpy(&buf[blen], expr, strlen(expr));
 		    }
 		    else {
 			if (!isnan(fval) && fval < 0.0)
@@ -949,14 +1038,14 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 			    buf[blen + need - strlen(expr) - 1] = '+';
 			else if ((flags & FSPACE) && need > width)
 			    blen++;
-			strncpy(&buf[blen + need - strlen(expr)], expr,
-				strlen(expr));
+			memcpy(&buf[blen + need - strlen(expr)], expr,
+			       strlen(expr));
 		    }
 		    blen += strlen(&buf[blen]);
 		    break;
 		}
 
-		fmt_setup(fbuf, *p, flags, width, prec);
+		fmt_setup(fbuf, sizeof(fbuf), *p, flags, width, prec);
 		need = 0;
 		if (*p != 'e' && *p != 'E') {
 		    i = INT_MIN;
@@ -970,7 +1059,7 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 		need += 20;
 
 		CHECK(need);
-		sprintf(&buf[blen], fbuf, fval);
+		snprintf(&buf[blen], need, fbuf, fval);
 		blen += strlen(&buf[blen]);
 	    }
 	    break;
@@ -993,8 +1082,9 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 }
 
 static void
-fmt_setup(char *buf, int c, int flags, int width, int prec)
+fmt_setup(char *buf, size_t size, int c, int flags, int width, int prec)
 {
+    char *end = buf + size;
     *buf++ = '%';
     if (flags & FSHARP) *buf++ = '#';
     if (flags & FPLUS)  *buf++ = '+';
@@ -1003,12 +1093,12 @@ fmt_setup(char *buf, int c, int flags, int width, int prec)
     if (flags & FSPACE) *buf++ = ' ';
 
     if (flags & FWIDTH) {
-	sprintf(buf, "%d", width);
+	snprintf(buf, end - buf, "%d", width);
 	buf += strlen(buf);
     }
 
     if (flags & FPREC) {
-	sprintf(buf, ".%d", prec);
+	snprintf(buf, end - buf, ".%d", prec);
 	buf += strlen(buf);
     }
 
@@ -1032,11 +1122,9 @@ fmt_setup(char *buf, int c, int flags, int width, int prec)
 #  define u_quad_t unsigned LONG_LONG
 # endif
 #endif
-#undef vsnprintf
-#undef snprintf
 #define FLOATING_POINT 1
-#define BSD__dtoa dtoa
-#include "missing/vsnprintf.c"
+#define BSD__dtoa ruby_dtoa
+#include "vsnprintf.c"
 
 static int
 ruby__sfvwrite(register rb_printf_buffer *fp, register struct __suio *uio)
