@@ -1,4 +1,4 @@
-# $Id: test_http.rb 17437 2008-06-19 14:49:39Z mame $
+# $Id: test_http.rb 30571 2011-01-16 12:35:11Z yugui $
 
 require 'test/unit'
 require 'net/http'
@@ -17,7 +17,9 @@ module TestNetHTTP_version_1_1_methods
       res = http.head('/')
       assert_kind_of Net::HTTPResponse, res
       assert_equal $test_net_http_data_type, res['Content-Type']
-      assert_equal $test_net_http_data.size, res['Content-Length'].to_i
+      unless self.is_a?(TestNetHTTP_v1_2_chunked)
+        assert_equal $test_net_http_data.size, res['Content-Length'].to_i
+      end
     }
   end
 
@@ -34,13 +36,19 @@ module TestNetHTTP_version_1_1_methods
     assert_kind_of Net::HTTPResponse, res
     assert_kind_of String, res.body
     assert_kind_of String, body
-    assert_not_nil res['content-length']
-    assert_equal $test_net_http_data.size, res['content-length'].to_i
+    unless self.is_a?(TestNetHTTP_v1_2_chunked)
+      assert_not_nil res['content-length']
+      assert_equal $test_net_http_data.size, res['content-length'].to_i
+    end
     assert_equal $test_net_http_data_type, res['Content-Type']
     assert_equal $test_net_http_data.size, body.size
     assert_equal $test_net_http_data, body
     assert_equal $test_net_http_data.size, res.body.size
     assert_equal $test_net_http_data, res.body
+
+    assert_nothing_raised {
+      res, body = http.get('/', { 'User-Agent' => 'test' }.freeze)
+    }
   end
 
   def _test_get__iter(http)
@@ -49,8 +57,10 @@ module TestNetHTTP_version_1_1_methods
     assert_kind_of Net::HTTPResponse, res
     # assert_kind_of String, res.body
     # assert_kind_of String, body
-    assert_not_nil res['content-length']
-    assert_equal $test_net_http_data.size, res['content-length'].to_i
+    unless self.is_a?(TestNetHTTP_v1_2_chunked)
+      assert_not_nil res['content-length']
+      assert_equal $test_net_http_data.size, res['content-length'].to_i
+    end
     assert_equal $test_net_http_data_type, res['Content-Type']
     assert_equal $test_net_http_data.size, buf.size
     assert_equal $test_net_http_data, buf
@@ -64,8 +74,10 @@ module TestNetHTTP_version_1_1_methods
     assert_kind_of Net::HTTPResponse, res
     # assert_kind_of String, res.body
     # assert_kind_of String, body
-    assert_not_nil res['content-length']
-    assert_equal $test_net_http_data.size, res['content-length'].to_i
+    unless self.is_a?(TestNetHTTP_v1_2_chunked)
+      assert_not_nil res['content-length']
+      assert_equal $test_net_http_data.size, res['content-length'].to_i
+    end
     assert_equal $test_net_http_data_type, res['Content-Type']
     assert_equal $test_net_http_data.size, buf.size
     assert_equal $test_net_http_data, buf
@@ -89,7 +101,9 @@ module TestNetHTTP_version_1_1_methods
     assert_kind_of Net::HTTPResponse, res
     assert_kind_of String, body
     assert_kind_of String, res.body
-    assert_not_nil res['content-length']
+    unless self.is_a?(TestNetHTTP_v1_2_chunked)
+      assert_not_nil res['content-length']
+    end
     assert_equal $test_net_http_data_type, res['Content-Type']
     assert_equal $test_net_http_data.size, res.body.size
     assert_equal $test_net_http_data, res.body
@@ -100,7 +114,9 @@ module TestNetHTTP_version_1_1_methods
       http.get2('/') {|res|
         assert_kind_of Net::HTTPResponse, res
         assert_kind_of Net::HTTPResponse, res.header
-        assert_not_nil res['content-length']
+        unless self.is_a?(TestNetHTTP_v1_2_chunked)
+          assert_not_nil res['content-length']
+        end
         assert_equal $test_net_http_data_type, res['Content-Type']
         assert_kind_of String, res.body
         assert_kind_of String, res.entity
@@ -157,6 +173,25 @@ module TestNetHTTP_version_1_1_methods
     assert_equal ["a=x1", "a=x2", "b=y"], res.body.split(/[;&]/).sort
   end
 
+  def test_timeout_during_HTTP_session
+    bug4246 = "expected the HTTP session to have timed out but have not. c.f. [ruby-core:34203]"
+
+    # listen for connections... but deliberately do not complete SSL handshake
+    TCPServer.open(0) {|server|
+      port = server.addr[1]
+
+      conn = Net::HTTP.new('localhost', port)
+      conn.read_timeout = 1
+      conn.open_timeout = 1
+
+      th = Thread.new do
+        assert_raise(Timeout::Error) {
+          conn.get('/')
+        }
+      end
+      assert th.join(10), bug4246
+    }
+  end
 end
 
 
@@ -178,8 +213,10 @@ module TestNetHTTP_version_1_2_methods
     http.request(req) {|res|
       assert_kind_of Net::HTTPResponse, res
       assert_kind_of String, res.body
-      assert_not_nil res['content-length']
-      assert_equal $test_net_http_data.size, res['content-length'].to_i
+      unless self.is_a?(TestNetHTTP_v1_2_chunked)
+        assert_not_nil res['content-length']
+        assert_equal $test_net_http_data.size, res['content-length'].to_i
+      end
       assert_equal $test_net_http_data.size, res.body.size
       assert_equal $test_net_http_data, res.body
     }
@@ -189,11 +226,13 @@ module TestNetHTTP_version_1_2_methods
     req = Net::HTTP::Get.new('/')
     http.request(req) {|res|
       assert_kind_of Net::HTTPResponse, res
-      assert_not_nil res['content-length']
-      assert_equal $test_net_http_data.size, res['content-length'].to_i
-      f = StringIO.new
+      unless self.is_a?(TestNetHTTP_v1_2_chunked)
+        assert_not_nil res['content-length']
+        assert_equal $test_net_http_data.size, res['content-length'].to_i
+      end
+      f = StringIO.new("".force_encoding("ASCII-8BIT"))
       res.read_body f
-      assert_equal $test_net_http_data.size, f.string.size
+      assert_equal $test_net_http_data.bytesize, f.string.bytesize
       assert_equal $test_net_http_data.encoding, f.string.encoding
       assert_equal $test_net_http_data, f.string
     }
@@ -209,8 +248,10 @@ module TestNetHTTP_version_1_2_methods
     req = Net::HTTP::Head.new('/')
     http.request(req) {|res|
       assert_kind_of Net::HTTPResponse, res
-      assert_not_nil res['content-length']
-      assert_equal $test_net_http_data.size, res['content-length'].to_i
+      unless self.is_a?(TestNetHTTP_v1_2_chunked)
+        assert_not_nil res['content-length']
+        assert_equal $test_net_http_data.size, res['content-length'].to_i
+      end
       assert_nil res.body
     }
   end
@@ -221,7 +262,9 @@ module TestNetHTTP_version_1_2_methods
     req['Accept'] = $test_net_http_data_type
     http.request(req, data) {|res|
       assert_kind_of Net::HTTPResponse, res
-      assert_equal data.size, res['content-length'].to_i
+      unless self.is_a?(TestNetHTTP_v1_2_chunked)
+        assert_equal data.size, res['content-length'].to_i
+      end
       assert_kind_of String, res.body
       assert_equal data, res.body
     }
@@ -249,11 +292,13 @@ module TestNetHTTP_version_1_2_methods
   def _test_send_request__GET(http)
     res = http.send_request('GET', '/')
     assert_kind_of Net::HTTPResponse, res
-    assert_equal $test_net_http_data.size, res['content-length'].to_i
+    unless self.is_a?(TestNetHTTP_v1_2_chunked)
+      assert_equal $test_net_http_data.size, res['content-length'].to_i
+    end
     assert_kind_of String, res.body
     assert_equal $test_net_http_data, res.body
   end
-  
+
   def _test_send_request__POST(http)
     data = 'aaabbb cc ddddddddddd lkjoiu4j3qlkuoa'
     res = http.send_request('POST', '/', data)
@@ -299,6 +344,38 @@ class TestNetHTTP_v1_2 < Test::Unit::TestCase
   end
 end
 
+class TestNetHTTP_v1_2_chunked < Test::Unit::TestCase
+  CONFIG = {
+    'host' => '127.0.0.1',
+    'port' => 10081,
+    'proxy_host' => nil,
+    'proxy_port' => nil,
+    'chunked' => true,
+  }
+
+  include TestNetHTTPUtils
+  include TestNetHTTP_version_1_1_methods
+  include TestNetHTTP_version_1_2_methods
+
+  def new
+    Net::HTTP.version_1_2
+    super
+  end
+
+  def test_chunked_break
+    i = 0
+    assert_nothing_raised("[ruby-core:29229]") {
+      start {|http|
+        http.request_get('/') {|res|
+          res.read_body {|chunk|
+            break
+          }
+        }
+      }
+    }
+  end
+end
+
 =begin
 class TestNetHTTP_proxy < Test::Unit::TestCase
   CONFIG = {
