@@ -7,7 +7,7 @@ class Complex_Test < Test::Unit::TestCase
   def setup
     @rational = defined?(Rational)
     if @rational
-      @keiju = Rational.instance_variable_get('@RCS_ID')
+      @keiju = Rational.instance_variables.include?(:@RCS_ID)
     end
     seps = [File::SEPARATOR, File::ALT_SEPARATOR].compact.map{|x| Regexp.escape(x)}.join("|")
     @unify = $".grep(/(?:^|#{seps})mathn(?:\.(?:rb|so))?/).size != 0
@@ -37,6 +37,9 @@ class Complex_Test < Test::Unit::TestCase
       assert_instance_of(ComplexSub, c5)
     end
 
+    c1 = Complex(1)
+    assert_equal(c1.hash, c.hash, '[ruby-dev:38850]')
+    assert_equal([true, true], [c.eql?(c1), c1.eql?(c)])
   end
 
   def test_eql_p
@@ -127,8 +130,8 @@ class Complex_Test < Test::Unit::TestCase
     if @rational && !@keiju
       assert_equal(Complex(1,1),Complex('3/3','3/3'))
     end
-    assert_raise(ArgumentError){Complex(nil)}
-    assert_raise(ArgumentError){Complex(Object.new)}
+    assert_raise(TypeError){Complex(nil)}
+    assert_raise(TypeError){Complex(Object.new)}
     assert_raise(ArgumentError){Complex()}
     assert_raise(ArgumentError){Complex(1,2,3)}
 
@@ -489,7 +492,7 @@ class Complex_Test < Test::Unit::TestCase
     assert_raise(NoMethodError){Complex(1,1) <=> Complex(1,1)}
   end
 
-  def test_equal
+  def test_eqeq
     assert(Complex(1,0) == Complex(1))
     assert(Complex(-1,0) == Complex(-1))
 
@@ -630,6 +633,11 @@ class Complex_Test < Test::Unit::TestCase
       assert_equal(c, c2)
       assert_instance_of(Complex, c2)
     end
+
+    bug3656 = '[ruby-core:31622]'
+    assert_raise(TypeError, bug3656) {
+      Complex(1,2).marshal_load(0)
+    }
   end
 
   def test_parse
@@ -892,6 +900,13 @@ class Complex_Test < Test::Unit::TestCase
     assert_equal(0, 1.0.angle)
     assert_equal(0, 1.0.phase)
 
+    if (0.0/0).nan?
+      nan = 0.0/0
+      assert(nan.arg.equal?(nan))
+      assert(nan.angle.equal?(nan))
+      assert(nan.phase.equal?(nan))
+    end
+
     assert_equal(Math::PI, -1.arg)
     assert_equal(Math::PI, -1.angle)
     assert_equal(Math::PI, -1.phase)
