@@ -3,6 +3,13 @@ require "fileutils"
 require "tmpdir"
 
 class TestFileExhaustive < Test::Unit::TestCase
+  def assert_incompatible_encoding
+    d = "\u{3042}\u{3044}".encode("utf-16le")
+    assert_raise(Encoding::CompatibilityError) {yield d}
+    m = Class.new {define_method(:to_path) {d}}
+    assert_raise(Encoding::CompatibilityError) {yield m.new}
+  end
+
   def setup
     @dir = Dir.mktmpdir("rubytest-file")
     File.chown(-1, Process.gid, @dir)
@@ -34,7 +41,7 @@ class TestFileExhaustive < Test::Unit::TestCase
   def make_file(content, file = @file)
     open(file, "w") {|fh| fh << content }
   end
-  
+
   def make_tmp_filename(prefix)
     @hardlinkfile = @dir + "/" + prefix + File.basename(__FILE__) + ".#{$$}.test"
   end
@@ -43,6 +50,12 @@ class TestFileExhaustive < Test::Unit::TestCase
     file = @file
 
     assert_equal(file, File.open(file) {|f| f.path})
+    assert_equal(file, File.path(file))
+    o = Object.new
+    class << o; self; end.class_eval do
+      define_method(:to_path) { file }
+    end
+    assert_equal(file, File.path(o))
   end
 
   def assert_integer(n)
@@ -70,7 +83,9 @@ class TestFileExhaustive < Test::Unit::TestCase
       assert_integer_or_nil(fs1.rdev_minor)
       assert_integer(fs1.ino)
       assert_integer(fs1.mode)
-      unless /emx/ =~ RUBY_PLATFORM
+      unless /emx|mswin|mingw/ =~ RUBY_PLATFORM
+        # on Windows, nlink is always 1. but this behavior will be changed
+        # in the future.
         assert_equal(@hardlinkfile ? 2 : 1, fs1.nlink)
       end
       assert_integer(fs1.uid)
@@ -136,6 +151,7 @@ class TestFileExhaustive < Test::Unit::TestCase
 
   def test_readable_p
     return if /cygwin|mswin|bccwin|mingw|emx/ =~ RUBY_PLATFORM
+    return if Process.euid == 0
     File.chmod(0200, @file)
     assert(!(File.readable?(@file)))
     File.chmod(0600, @file)
@@ -145,6 +161,7 @@ class TestFileExhaustive < Test::Unit::TestCase
 
   def test_readable_real_p
     return if /cygwin|mswin|bccwin|mingw|emx/ =~ RUBY_PLATFORM
+    return if Process.euid == 0
     File.chmod(0200, @file)
     assert(!(File.readable_real?(@file)))
     File.chmod(0600, @file)
@@ -152,8 +169,20 @@ class TestFileExhaustive < Test::Unit::TestCase
     assert(!(File.readable_real?(@nofile)))
   end
 
+  def test_world_readable_p
+    return if /cygwin|mswin|bccwin|mingw|emx/ =~ RUBY_PLATFORM
+    File.chmod(0006, @file)
+    assert(File.world_readable?(@file))
+    File.chmod(0060, @file)
+    assert(!(File.world_readable?(@file)))
+    File.chmod(0600, @file)
+    assert(!(File.world_readable?(@file)))
+    assert(!(File.world_readable?(@nofile)))
+  end
+
   def test_writable_p
     return if /cygwin|mswin|bccwin|mingw|emx/ =~ RUBY_PLATFORM
+    return if Process.euid == 0
     File.chmod(0400, @file)
     assert(!(File.writable?(@file)))
     File.chmod(0600, @file)
@@ -163,6 +192,7 @@ class TestFileExhaustive < Test::Unit::TestCase
 
   def test_writable_real_p
     return if /cygwin|mswin|bccwin|mingw|emx/ =~ RUBY_PLATFORM
+    return if Process.euid == 0
     File.chmod(0400, @file)
     assert(!(File.writable_real?(@file)))
     File.chmod(0600, @file)
@@ -170,6 +200,17 @@ class TestFileExhaustive < Test::Unit::TestCase
     assert(!(File.writable_real?(@nofile)))
   end
 
+  def test_world_writable_p
+    return if /cygwin|mswin|bccwin|mingw|emx/ =~ RUBY_PLATFORM
+    File.chmod(0006, @file)
+    assert(File.world_writable?(@file))
+    File.chmod(0060, @file)
+    assert(!(File.world_writable?(@file)))
+    File.chmod(0600, @file)
+    assert(!(File.world_writable?(@file)))
+    assert(!(File.world_writable?(@nofile)))
+  end
+
   def test_executable_p
     return if /cygwin|mswin|bccwin|mingw|emx/ =~ RUBY_PLATFORM
     File.chmod(0100, @file)
@@ -227,7 +268,7 @@ class TestFileExhaustive < Test::Unit::TestCase
     assert(!(File.identical?(@nofile, @file)))
   end
 
-  def test_size
+  def test_s_size
     assert_integer(File.size(@dir))
     assert_equal(3, File.size(@file))
     assert_equal(0, File.size(@zerofile))
@@ -313,11 +354,14 @@ class TestFileExhaustive < Test::Unit::TestCase
     assert_raise(Errno::EEXIST) { File.link(@file, @file) }
   end
 
-  def test_symlink2
+  def test_readlink
     return unless @symlinkfile
     assert_equal(@file, File.readlink(@symlinkfile))
     assert_raise(Errno::EINVAL) { File.readlink(@file) }
     assert_raise(Errno::ENOENT) { File.readlink(@nofile) }
+    if fs = Encoding.find("filesystem")
+      assert_equal(fs, File.readlink(@symlinkfile).encoding)
+    end
   rescue NotImplementedError
   end
 
@@ -352,6 +396,7 @@ class TestFileExhaustive < Test::Unit::TestCase
       assert_equal(@file, File.expand_path(@file + " "))
       assert_equal(@file, File.expand_path(@file + "."))
       assert_equal(@file, File.expand_path(@file + "::$DATA"))
+      assert_match(/\Ac:\//i, File.expand_path('c:'), '[ruby-core:31591]')
     end
     assert_kind_of(String, File.expand_path("~"))
     assert_raise(ArgumentError) { File.expand_path("~foo_bar_baz_unknown_user_wahaha") }
@@ -368,12 +413,15 @@ class TestFileExhaustive < Test::Unit::TestCase
     ensure
       ENV["HOME"] = home
     end
+    assert_incompatible_encoding {|d| File.expand_path(d)}
   end
 
   def test_basename
     assert_equal(File.basename(@file).sub(/\.test$/, ""), File.basename(@file, ".test"))
-    assert_equal("", File.basename(""))
-    assert_equal("foo", File.basename("foo"))
+    assert_equal("", s = File.basename(""))
+    assert(!s.frozen?, '[ruby-core:24199]')
+    assert_equal("foo", s = File.basename("foo"))
+    assert(!s.frozen?, '[ruby-core:24199]')
     assert_equal("foo", File.basename("foo", ".ext"))
     assert_equal("foo", File.basename("foo.ext", ".ext"))
     assert_equal("foo", File.basename("foo.ext", ".*"))
@@ -390,15 +438,20 @@ class TestFileExhaustive < Test::Unit::TestCase
       assert_equal(basename, File.basename(@file + ".", ".*"))
       assert_equal(basename, File.basename(@file + "::$DATA", ".*"))
     end
+
+    assert_incompatible_encoding {|d| File.basename(d)}
+    assert_incompatible_encoding {|d| File.basename(d, ".*")}
+    assert_raise(Encoding::CompatibilityError) {File.basename("foo.ext", ".*".encode("utf-16le"))}
   end
 
   def test_dirname
     assert(@file.start_with?(File.dirname(@file)))
     assert_equal(".", File.dirname(""))
+    assert_incompatible_encoding {|d| File.dirname(d)}
   end
 
   def test_extname
-    assert(".test", File.extname(@file))
+    assert_equal(".test", File.extname(@file))
     prefixes = ["", "/", ".", "/.", "bar/.", "/bar/."]
     infixes = ["", " ", "."]
     infixes2 = infixes + [".ext "]
@@ -420,6 +473,8 @@ class TestFileExhaustive < Test::Unit::TestCase
     end
     bug3175 = '[ruby-core:29627]'
     assert_equal(".rb", File.extname("/tmp//bla.rb"), bug3175)
+
+    assert_incompatible_encoding {|d| File.extname(d)}
   end
 
   def test_split
@@ -432,6 +487,9 @@ class TestFileExhaustive < Test::Unit::TestCase
     s = "foo" + File::SEPARATOR + "bar" + File::SEPARATOR + "baz"
     assert_equal(s, File.join("foo", "bar", "baz"))
     assert_equal(s, File.join(["foo", "bar", "baz"]))
+    o = Object.new
+    def o.to_path; "foo"; end
+    assert_equal(s, File.join(o, "bar", "baz"))
     assert_equal(s, File.join("foo" + File::SEPARATOR, "bar", File::SEPARATOR + "baz"))
   end
 
@@ -455,7 +513,7 @@ class TestFileExhaustive < Test::Unit::TestCase
     f.close
     make_file("foo", @file)
 
-    assert_raise(IOError) { File.open(@file) {|f| f.truncate(0)} }
+    assert_raise(IOError) { File.open(@file) {|ff| ff.truncate(0)} }
   rescue NotImplementedError
   end
 
@@ -493,6 +551,7 @@ class TestFileExhaustive < Test::Unit::TestCase
       assert_equal(File.socket?(f), test(?S, f))
       assert_equal(File.setuid?(f), test(?u, f))
       assert_equal(File.writable?(f), test(?w, f))
+      assert_equal(File.writable_real?(f), test(?W, f))
       assert_equal(File.executable?(f), test(?x, f))
       assert_equal(File.executable_real?(f), test(?X, f))
       assert_equal(File.zero?(f), test(?z, f))
@@ -512,6 +571,7 @@ class TestFileExhaustive < Test::Unit::TestCase
     assert_raise(ArgumentError) { test }
     assert_raise(Errno::ENOENT) { test(?A, @nofile) }
     assert_raise(ArgumentError) { test(?a) }
+    assert_raise(ArgumentError) { test("\0".ord) }
   end
 
   def test_stat_init
@@ -531,7 +591,9 @@ class TestFileExhaustive < Test::Unit::TestCase
       assert_integer_or_nil(fs1.rdev_minor)
       assert_integer(fs1.ino)
       assert_integer(fs1.mode)
-      unless /emx/ =~ RUBY_PLATFORM
+      unless /emx|mswin|mingw/ =~ RUBY_PLATFORM
+        # on Windows, nlink is always 1. but this behavior will be changed
+        # in the future.
         assert_equal(@hardlinkfile ? 2 : 1, fs1.nlink)
       end
       assert_integer(fs1.uid)
@@ -594,6 +656,7 @@ class TestFileExhaustive < Test::Unit::TestCase
 
   def test_stat_readable_p
     return if /cygwin|mswin|bccwin|mingw|emx/ =~ RUBY_PLATFORM
+    return if Process.euid == 0
     File.chmod(0200, @file)
     assert(!(File::Stat.new(@file).readable?))
     File.chmod(0600, @file)
@@ -602,14 +665,26 @@ class TestFileExhaustive < Test::Unit::TestCase
 
   def test_stat_readable_real_p
     return if /cygwin|mswin|bccwin|mingw|emx/ =~ RUBY_PLATFORM
+    return if Process.euid == 0
     File.chmod(0200, @file)
     assert(!(File::Stat.new(@file).readable_real?))
     File.chmod(0600, @file)
     assert(File::Stat.new(@file).readable_real?)
   end
 
+  def test_stat_world_readable_p
+    return if /cygwin|mswin|bccwin|mingw|emx/ =~ RUBY_PLATFORM
+    File.chmod(0006, @file)
+    assert(File::Stat.new(@file).world_readable?)
+    File.chmod(0060, @file)
+    assert(!(File::Stat.new(@file).world_readable?))
+    File.chmod(0600, @file)
+    assert(!(File::Stat.new(@file).world_readable?))
+  end
+
   def test_stat_writable_p
     return if /cygwin|mswin|bccwin|mingw|emx/ =~ RUBY_PLATFORM
+    return if Process.euid == 0
     File.chmod(0400, @file)
     assert(!(File::Stat.new(@file).writable?))
     File.chmod(0600, @file)
@@ -618,12 +693,23 @@ class TestFileExhaustive < Test::Unit::TestCase
 
   def test_stat_writable_real_p
     return if /cygwin|mswin|bccwin|mingw|emx/ =~ RUBY_PLATFORM
+    return if Process.euid == 0
     File.chmod(0400, @file)
     assert(!(File::Stat.new(@file).writable_real?))
     File.chmod(0600, @file)
     assert(File::Stat.new(@file).writable_real?)
   end
 
+  def test_stat_world_writable_p
+    return if /cygwin|mswin|bccwin|mingw|emx/ =~ RUBY_PLATFORM
+    File.chmod(0006, @file)
+    assert(File::Stat.new(@file).world_writable?)
+    File.chmod(0060, @file)
+    assert(!(File::Stat.new(@file).world_writable?))
+    File.chmod(0600, @file)
+    assert(!(File::Stat.new(@file).world_writable?))
+  end
+
   def test_stat_executable_p
     return if /cygwin|mswin|bccwin|mingw|emx/ =~ RUBY_PLATFORM
     File.chmod(0100, @file)
@@ -687,4 +773,18 @@ class TestFileExhaustive < Test::Unit::TestCase
       end.join
     end
   end
+
+  def test_size
+    assert_equal(3, File.open(@file) {|f| f.size })
+    File.open(@file, "a") do |f|
+      f.write("bar")
+      assert_equal(6, f.size)
+    end
+  end
+
+  def test_absolute_path
+    assert_equal(File.join(Dir.pwd, "~foo"), File.absolute_path("~foo"))
+    dir = File.expand_path("/bar")
+    assert_equal(File.join(dir, "~foo"), File.absolute_path("~foo", dir))
+  end
 end
