@@ -7,6 +7,7 @@ class TestM17N < Test::Unit::TestCase
   end
 
   module AESU
+    def ua(str) str.dup.force_encoding("US-ASCII") end
     def a(str) str.dup.force_encoding("ASCII-8BIT") end
     def e(str) str.dup.force_encoding("EUC-JP") end
     def s(str) str.dup.force_encoding("Windows-31J") end
@@ -48,7 +49,7 @@ class TestM17N < Test::Unit::TestCase
       if enc == r.encoding
         assert_nothing_raised { r =~ "\xc2\xa1".force_encoding(enc) }
       else
-        assert_raise(ArgumentError) { r =~ "\xc2\xa1".force_encoding(enc) }
+        assert_raise(Encoding::CompatibilityError) { r =~ "\xc2\xa1".force_encoding(enc) }
       end
     }
   end
@@ -146,17 +147,17 @@ class TestM17N < Test::Unit::TestCase
   # tests start
 
   def test_string_ascii_literal
-    assert_encoding("US-ASCII", eval(a(%{""})).encoding)
-    assert_encoding("US-ASCII", eval(a(%{"a"})).encoding)
+    assert_encoding("ASCII-8BIT", eval(a(%{""})).encoding)
+    assert_encoding("ASCII-8BIT", eval(a(%{"a"})).encoding)
   end
 
   def test_string_eucjp_literal
-    assert_encoding("US-ASCII", eval(e(%{""})).encoding)
-    assert_encoding("US-ASCII", eval(e(%{"a"})).encoding)
+    assert_encoding("EUC-JP", eval(e(%{""})).encoding)
+    assert_encoding("EUC-JP", eval(e(%{"a"})).encoding)
     assert_encoding("EUC-JP", eval(e(%{"\xa1\xa1"})).encoding)
     assert_encoding("EUC-JP", eval(e(%{"\\xa1\\xa1"})).encoding)
-    assert_encoding("US-ASCII", eval(e(%{"\\x20"})).encoding)
-    assert_encoding("US-ASCII", eval(e(%{"\\n"})).encoding)
+    assert_encoding("EUC-JP", eval(e(%{"\\x20"})).encoding)
+    assert_encoding("EUC-JP", eval(e(%{"\\n"})).encoding)
     assert_encoding("EUC-JP", eval(e(%{"\\x80"})).encoding)
   end
 
@@ -176,7 +177,7 @@ class TestM17N < Test::Unit::TestCase
     assert_nothing_raised { eval(u(%{"\\u{6666}\xc2\xa1"})) }
   end
 
-  def test_string_inspect
+  def test_string_inspect_invalid
     assert_equal('"\xFE"', e("\xfe").inspect)
     assert_equal('"\x8E"', e("\x8e").inspect)
     assert_equal('"\x8F"', e("\x8f").inspect)
@@ -199,15 +200,38 @@ class TestM17N < Test::Unit::TestCase
     assert_equal('"\xF8\x80\x80\x80 "', u("\xf8\x80\x80\x80 ").inspect)
     assert_equal('"\xFC\x80\x80\x80\x80 "', u("\xfc\x80\x80\x80\x80 ").inspect)
 
-
-    assert_equal(e("\"\\xA1\x8f\xA1\xA1\""), e("\xa1\x8f\xa1\xa1").inspect)
-
     assert_equal('"\x81."', s("\x81.").inspect)
-    assert_equal(s("\"\x81@\""), s("\x81@").inspect)
-
     assert_equal('"\xFC"', u("\xfc").inspect)
   end
 
+  def test_string_inspect_encoding
+    orig_int = Encoding.default_internal
+    orig_ext = Encoding.default_external
+    Encoding.default_internal = nil
+    [Encoding::UTF_8, Encoding::EUC_JP, Encoding::Windows_31J, Encoding::GB18030].
+      each do |e|
+      Encoding.default_external = e
+      str = "\x81\x30\x81\x30".force_encoding('GB18030')
+      assert_equal(Encoding::GB18030 == e ? %{"#{str}"} : '"\x{81308130}"', str.inspect)
+      str = e("\xa1\x8f\xa1\xa1")
+      expected = "\"\\xA1\x8F\xA1\xA1\"".force_encoding("EUC-JP")
+      assert_equal(Encoding::EUC_JP == e ? expected : "\"\\xA1\\x{8FA1A1}\"", str.inspect)
+      str = s("\x81@")
+      assert_equal(Encoding::Windows_31J == e ? %{"#{str}"} : '"\x{8140}"', str.inspect)
+      str = "\u3042\u{10FFFD}"
+      assert_equal(Encoding::UTF_8 == e ? %{"#{str}"} : '"\u3042\u{10FFFD}"', str.inspect)
+    end
+    Encoding.default_external = Encoding::UTF_8
+    [Encoding::UTF_16BE, Encoding::UTF_16LE, Encoding::UTF_32BE, Encoding::UTF_32LE,
+      Encoding::UTF8_SOFTBANK].each do |e|
+      str = "abc".encode(e)
+      assert_equal('"abc"', str.inspect)
+    end
+  ensure
+    Encoding.default_internal = orig_int
+    Encoding.default_external = orig_ext
+  end
+
   def test_str_dump
     [
       e("\xfe"),
@@ -241,6 +265,9 @@ class TestM17N < Test::Unit::TestCase
       u("\xfc"),
       "\u3042",
       "ascii",
+
+      "\u3042".encode("UTF-16LE"),
+      "\u3042".encode("UTF-16BE"),
     ].each do |str|
       assert_equal(str, eval(str.dump), "[ruby-dev:33142]")
     end
@@ -385,9 +412,9 @@ class TestM17N < Test::Unit::TestCase
       assert_equal(nil, r =~ s("a"))
       assert_equal(nil, r =~ u("a"))
       assert_equal(0, r =~ a("\xc2\xa1"))
-      assert_raise(ArgumentError) { r =~ e("\xc2\xa1") }
-      assert_raise(ArgumentError) { r =~ s("\xc2\xa1") }
-      assert_raise(ArgumentError) { r =~ u("\xc2\xa1") }
+      assert_raise(Encoding::CompatibilityError) { r =~ e("\xc2\xa1") }
+      assert_raise(Encoding::CompatibilityError) { r =~ s("\xc2\xa1") }
+      assert_raise(Encoding::CompatibilityError) { r =~ u("\xc2\xa1") }
     }
   end
 
@@ -402,10 +429,10 @@ class TestM17N < Test::Unit::TestCase
       assert_equal(0, r =~ e("a"))
       assert_equal(0, r =~ s("a"))
       assert_equal(0, r =~ u("a"))
-      assert_raise(ArgumentError) { r =~ a("\xc2\xa1") }
+      assert_raise(Encoding::CompatibilityError) { r =~ a("\xc2\xa1") }
       assert_equal(nil, r =~ e("\xc2\xa1"))
-      assert_raise(ArgumentError) { r =~ s("\xc2\xa1") }
-      assert_raise(ArgumentError) { r =~ u("\xc2\xa1") }
+      assert_raise(Encoding::CompatibilityError) { r =~ s("\xc2\xa1") }
+      assert_raise(Encoding::CompatibilityError) { r =~ u("\xc2\xa1") }
     }
 
     [/\xc2\xa1/e, eval(e(%{/\xc2\xa1/})), eval(e(%q{/\xc2\xa1/}))].each {|r|
@@ -413,10 +440,10 @@ class TestM17N < Test::Unit::TestCase
       assert_equal(nil, r =~ e("a"))
       assert_equal(nil, r =~ s("a"))
       assert_equal(nil, r =~ u("a"))
-      assert_raise(ArgumentError) { r =~ a("\xc2\xa1") }
+      assert_raise(Encoding::CompatibilityError) { r =~ a("\xc2\xa1") }
       assert_equal(0, r =~ e("\xc2\xa1"))
-      assert_raise(ArgumentError) { r =~ s("\xc2\xa1") }
-      assert_raise(ArgumentError) { r =~ u("\xc2\xa1") }
+      assert_raise(Encoding::CompatibilityError) { r =~ s("\xc2\xa1") }
+      assert_raise(Encoding::CompatibilityError) { r =~ u("\xc2\xa1") }
     }
   end
 
@@ -430,7 +457,7 @@ class TestM17N < Test::Unit::TestCase
   def test_regexp_windows_31j
     begin
       Regexp.new("\xa1".force_encoding("windows-31j")) =~ "\xa1\xa1".force_encoding("euc-jp")
-    rescue ArgumentError
+    rescue Encoding::CompatibilityError
       err = $!
     end
     assert_match(/windows-31j/i, err.message)
@@ -438,18 +465,18 @@ class TestM17N < Test::Unit::TestCase
 
   def test_regexp_embed
     r = eval(e("/\xc2\xa1/"))
-    assert_raise(ArgumentError) { eval(s("/\xc2\xa1\#{r}/s")) }
-    assert_raise(ArgumentError) { eval(s("/\#{r}\xc2\xa1/s")) }
+    assert_raise(RegexpError) { eval(s("/\xc2\xa1\#{r}/s")) }
+    assert_raise(RegexpError) { eval(s("/\#{r}\xc2\xa1/s")) }
 
     r = /\xc2\xa1/e
-    assert_raise(ArgumentError) { eval(s("/\xc2\xa1\#{r}/s")) }
-    assert_raise(ArgumentError) { eval(s("/\#{r}\xc2\xa1/s")) }
+    assert_raise(RegexpError) { eval(s("/\xc2\xa1\#{r}/s")) }
+    assert_raise(RegexpError) { eval(s("/\#{r}\xc2\xa1/s")) }
 
     r = eval(e("/\xc2\xa1/"))
-    assert_raise(ArgumentError) { /\xc2\xa1#{r}/s }
+    assert_raise(RegexpError) { /\xc2\xa1#{r}/s }
 
     r = /\xc2\xa1/e
-    assert_raise(ArgumentError) { /\xc2\xa1#{r}/s }
+    assert_raise(RegexpError) { /\xc2\xa1#{r}/s }
 
     r1 = Regexp.new('foo'.force_encoding("ascii-8bit"))
     r2 = eval('/bar#{r1}/'.force_encoding('ascii-8bit'))
@@ -642,40 +669,40 @@ class TestM17N < Test::Unit::TestCase
 
   def test_dynamic_ascii_regexp
     assert_warning(%r{regexp match /.../n against to}) {
-      assert_regexp_generic_ascii(/#{}/n)
+      assert_regexp_generic_ascii(/#{ }/n)
     }
-    assert_regexp_fixed_ascii8bit(/#{}\xc2\xa1/n)
-    assert_regexp_fixed_ascii8bit(/\xc2\xa1#{}/n)
+    assert_regexp_fixed_ascii8bit(/#{ }\xc2\xa1/n)
+    assert_regexp_fixed_ascii8bit(/\xc2\xa1#{ }/n)
     assert_nothing_raised { s1, s2 = a('\xc2'), a('\xa1'); /#{s1}#{s2}/ }
   end
 
   def test_dynamic_eucjp_regexp
-    assert_regexp_fixed_eucjp(/#{}/e)
-    assert_regexp_fixed_eucjp(/#{}\xc2\xa1/e)
-    assert_regexp_fixed_eucjp(/\xc2\xa1#{}/e)
-    assert_raise(SyntaxError) { eval('/\xc2#{}/e') }
-    assert_raise(SyntaxError) { eval('/#{}\xc2/e') }
-    assert_raise(SyntaxError) { eval('/\xc2#{}\xa1/e') }
+    assert_regexp_fixed_eucjp(/#{ }/e)
+    assert_regexp_fixed_eucjp(/#{ }\xc2\xa1/e)
+    assert_regexp_fixed_eucjp(/\xc2\xa1#{ }/e)
+    assert_raise(SyntaxError) { eval('/\xc2#{ }/e') }
+    assert_raise(SyntaxError) { eval('/#{ }\xc2/e') }
+    assert_raise(SyntaxError) { eval('/\xc2#{ }\xa1/e') }
     assert_raise(ArgumentError) { s1, s2 = e('\xc2'), e('\xa1'); /#{s1}#{s2}/ }
   end
 
   def test_dynamic_sjis_regexp
-    assert_regexp_fixed_sjis(/#{}/s)
-    assert_regexp_fixed_sjis(/#{}\xc2\xa1/s)
-    assert_regexp_fixed_sjis(/\xc2\xa1#{}/s)
-    assert_raise(SyntaxError) { eval('/\x81#{}/s') }
-    assert_raise(SyntaxError) { eval('/#{}\x81/s') }
-    assert_raise(SyntaxError) { eval('/\x81#{}\xa1/s') }
+    assert_regexp_fixed_sjis(/#{ }/s)
+    assert_regexp_fixed_sjis(/#{ }\xc2\xa1/s)
+    assert_regexp_fixed_sjis(/\xc2\xa1#{ }/s)
+    assert_raise(SyntaxError) { eval('/\x81#{ }/s') }
+    assert_raise(SyntaxError) { eval('/#{ }\x81/s') }
+    assert_raise(SyntaxError) { eval('/\x81#{ }\xa1/s') }
     assert_raise(ArgumentError) { s1, s2 = s('\x81'), s('\xa1'); /#{s1}#{s2}/ }
   end
 
   def test_dynamic_utf8_regexp
-    assert_regexp_fixed_utf8(/#{}/u)
-    assert_regexp_fixed_utf8(/#{}\xc2\xa1/u)
-    assert_regexp_fixed_utf8(/\xc2\xa1#{}/u)
-    assert_raise(SyntaxError) { eval('/\xc2#{}/u') }
-    assert_raise(SyntaxError) { eval('/#{}\xc2/u') }
-    assert_raise(SyntaxError) { eval('/\xc2#{}\xa1/u') }
+    assert_regexp_fixed_utf8(/#{ }/u)
+    assert_regexp_fixed_utf8(/#{ }\xc2\xa1/u)
+    assert_regexp_fixed_utf8(/\xc2\xa1#{ }/u)
+    assert_raise(SyntaxError) { eval('/\xc2#{ }/u') }
+    assert_raise(SyntaxError) { eval('/#{ }\xc2/u') }
+    assert_raise(SyntaxError) { eval('/\xc2#{ }\xa1/u') }
     assert_raise(ArgumentError) { s1, s2 = u('\xc2'), u('\xa1'); /#{s1}#{s2}/ }
   end
 
@@ -708,23 +735,23 @@ class TestM17N < Test::Unit::TestCase
     assert_raise(SyntaxError) { eval(s(%{/\\u{6666}\\xc2\\xa1/})) }
     assert_nothing_raised { eval(u(%{/\\u{6666}\\xc2\\xa1/})) }
 
-    assert_raise(SyntaxError) { eval(a(%{/\xc2\xa1#{}\\u{6666}/})) }
-    assert_raise(SyntaxError) { eval(e(%{/\xc2\xa1#{}\\u{6666}/})) }
-    assert_raise(SyntaxError) { eval(s(%{/\xc2\xa1#{}\\u{6666}/})) }
-    assert_nothing_raised { eval(u(%{/\xc2\xa1#{}\\u{6666}/})) }
-    assert_raise(SyntaxError) { eval(a(%{/\\u{6666}#{}\xc2\xa1/})) }
-    assert_raise(SyntaxError) { eval(e(%{/\\u{6666}#{}\xc2\xa1/})) }
-    assert_raise(SyntaxError) { eval(s(%{/\\u{6666}#{}\xc2\xa1/})) }
-    assert_nothing_raised { eval(u(%{/\\u{6666}#{}\xc2\xa1/})) }
+    assert_raise(SyntaxError) { eval(a(%{/\xc2\xa1#{ }\\u{6666}/})) }
+    assert_raise(SyntaxError) { eval(e(%{/\xc2\xa1#{ }\\u{6666}/})) }
+    assert_raise(SyntaxError) { eval(s(%{/\xc2\xa1#{ }\\u{6666}/})) }
+    assert_nothing_raised { eval(u(%{/\xc2\xa1#{ }\\u{6666}/})) }
+    assert_raise(SyntaxError) { eval(a(%{/\\u{6666}#{ }\xc2\xa1/})) }
+    assert_raise(SyntaxError) { eval(e(%{/\\u{6666}#{ }\xc2\xa1/})) }
+    assert_raise(SyntaxError) { eval(s(%{/\\u{6666}#{ }\xc2\xa1/})) }
+    assert_nothing_raised { eval(u(%{/\\u{6666}#{ }\xc2\xa1/})) }
 
-    assert_raise(SyntaxError) { eval(a(%{/\\xc2\\xa1#{}\\u{6666}/})) }
-    assert_raise(SyntaxError) { eval(e(%{/\\xc2\\xa1#{}\\u{6666}/})) }
-    assert_raise(SyntaxError) { eval(s(%{/\\xc2\\xa1#{}\\u{6666}/})) }
-    assert_nothing_raised { eval(u(%{/\\xc2\\xa1#{}\\u{6666}/})) }
-    assert_raise(SyntaxError) { eval(a(%{/\\u{6666}#{}\\xc2\\xa1/})) }
-    assert_raise(SyntaxError) { eval(e(%{/\\u{6666}#{}\\xc2\\xa1/})) }
-    assert_raise(SyntaxError) { eval(s(%{/\\u{6666}#{}\\xc2\\xa1/})) }
-    assert_nothing_raised { eval(u(%{/\\u{6666}#{}\\xc2\\xa1/})) }
+    assert_raise(SyntaxError) { eval(a(%{/\\xc2\\xa1#{ }\\u{6666}/})) }
+    assert_raise(SyntaxError) { eval(e(%{/\\xc2\\xa1#{ }\\u{6666}/})) }
+    assert_raise(SyntaxError) { eval(s(%{/\\xc2\\xa1#{ }\\u{6666}/})) }
+    assert_nothing_raised { eval(u(%{/\\xc2\\xa1#{ }\\u{6666}/})) }
+    assert_raise(SyntaxError) { eval(a(%{/\\u{6666}#{ }\\xc2\\xa1/})) }
+    assert_raise(SyntaxError) { eval(e(%{/\\u{6666}#{ }\\xc2\\xa1/})) }
+    assert_raise(SyntaxError) { eval(s(%{/\\u{6666}#{ }\\xc2\\xa1/})) }
+    assert_nothing_raised { eval(u(%{/\\u{6666}#{ }\\xc2\\xa1/})) }
   end
 
   def test_str_allocate
@@ -746,36 +773,23 @@ class TestM17N < Test::Unit::TestCase
     #assert_raise(ArgumentError) { s("%c") % 0xc2a1 }
     assert_strenc("\u{c2a1}", 'UTF-8', u("%c") % 0xc2a1)
     assert_strenc("\u{c2}", 'UTF-8', u("%c") % 0xc2)
-    assert_raise(ArgumentError) {
+    assert_raise(Encoding::CompatibilityError) {
       "%s%s" % [s("\xc2\xa1"), e("\xc2\xa1")]
     }
   end
 
   def test_sprintf_p
-    assert_strenc('""', 'ASCII-8BIT', a("%p") % a(""))
-    assert_strenc('""', 'EUC-JP', e("%p") % e(""))
-    assert_strenc('""', 'Windows-31J', s("%p") % s(""))
-    assert_strenc('""', 'UTF-8', u("%p") % u(""))
-
-    assert_strenc('"a"', 'ASCII-8BIT', a("%p") % a("a"))
-    assert_strenc('"a"', 'EUC-JP', e("%p") % e("a"))
-    assert_strenc('"a"', 'Windows-31J', s("%p") % s("a"))
-    assert_strenc('"a"', 'UTF-8', u("%p") % u("a"))
-
-    assert_strenc('"\xC2\xA1"', 'ASCII-8BIT', a("%p") % a("\xc2\xa1"))
-    assert_strenc("\"\xC2\xA1\"", 'EUC-JP', e("%p") % e("\xc2\xa1"))
-    #assert_strenc("\"\xC2\xA1\"", 'Windows-31J', s("%p") % s("\xc2\xa1"))
-    assert_strenc("\"\xC2\xA1\"", 'UTF-8', u("%p") % u("\xc2\xa1"))
-
-    assert_strenc('"\xC2\xA1"', 'US-ASCII', "%10p" % a("\xc2\xa1"))
-    assert_strenc("       \"\xC2\xA1\"", 'EUC-JP', "%10p" % e("\xc2\xa1"))
-    #assert_strenc("       \"\xC2\xA1\"", 'Windows-31J', "%10p" % s("\xc2\xa1"))
-    assert_strenc("       \"\xC2\xA1\"", 'UTF-8', "%10p" % u("\xc2\xa1"))
-
-    assert_strenc('"\x00"', 'ASCII-8BIT', a("%p") % a("\x00"))
-    assert_strenc('"\x00"', 'EUC-JP', e("%p") % e("\x00"))
-    assert_strenc('"\x00"', 'Windows-31J', s("%p") % s("\x00"))
-    assert_strenc('"\x00"', 'UTF-8', u("%p") % u("\x00"))
+    enc = "".inspect.encoding
+    asc = Encoding::US_ASCII
+    Encoding.list.each do |e|
+      format = "%p".force_encoding(e)
+      ['', 'a', "\xC2\xA1", "\x00"].each do |s|
+        s.force_encoding(e)
+        assert_strenc(s.inspect, e.ascii_compatible? && enc == asc ? e : enc, format % s)
+      end
+      s = "\xC2\xA1".force_encoding(e)
+      assert_strenc('%10s' % s.inspect, enc, "%10p" % s)
+    end
   end
 
   def test_sprintf_s
@@ -866,22 +880,27 @@ class TestM17N < Test::Unit::TestCase
 
   def test_str_aref_substr
     assert_equal(a("\xa1\xc2"), a("\xc2\xa1\xc2\xa2\xc2\xa3")[a("\xa1\xc2")])
-    assert_raise(ArgumentError) { a("\xc2\xa1\xc2\xa2\xc2\xa3")[e("\xa1\xc2")] }
+    assert_raise(Encoding::CompatibilityError) { a("\xc2\xa1\xc2\xa2\xc2\xa3")[e("\xa1\xc2")] }
 
     assert_equal(nil, e("\xc2\xa1\xc2\xa2\xc2\xa3")[e("\xa1\xc2")])
-    assert_raise(ArgumentError) { e("\xc2\xa1\xc2\xa2\xc2\xa3")[s("\xa1\xc2")] }
+    assert_raise(Encoding::CompatibilityError) { e("\xc2\xa1\xc2\xa2\xc2\xa3")[s("\xa1\xc2")] }
 
     assert_equal(s("\xa1\xc2"), s("\xc2\xa1\xc2\xa2\xc2\xa3")[s("\xa1\xc2")])
-    assert_raise(ArgumentError) { s("\xc2\xa1\xc2\xa2\xc2\xa3")[u("\xa1\xc2")] }
+    assert_raise(Encoding::CompatibilityError) { s("\xc2\xa1\xc2\xa2\xc2\xa3")[u("\xa1\xc2")] }
 
     assert_equal(nil, u("\xc2\xa1\xc2\xa2\xc2\xa3")[u("\xa1\xc2")])
-    assert_raise(ArgumentError) { u("\xc2\xa1\xc2\xa2\xc2\xa3")[a("\xa1\xc2")] }
+    assert_raise(Encoding::CompatibilityError) { u("\xc2\xa1\xc2\xa2\xc2\xa3")[a("\xa1\xc2")] }
     assert_nil(e("\xa1\xa2\xa3\xa4")[e("\xa2\xa3")])
+
+    bug2379 = '[ruby-core:26787]'
+    assert_equal("\u{439}", "\u{439}"[0, 30], bug2379)
+    assert_equal("\u{439}", "a\u{439}"[1, 30], bug2379)
+    assert_equal("\u{439}", "a\u{439}bcdefghijklmnop"[1, 1][0, 1], bug2379)
   end
 
   def test_aset
     s = e("\xa3\xb0\xa3\xb1\xa3\xb2\xa3\xb3\xa3\xb4")
-    assert_raise(ArgumentError){s["\xb0\xa3"] = "foo"}
+    assert_raise(Encoding::CompatibilityError){s["\xb0\xa3"] = "foo"}
   end
 
   def test_str_center
@@ -917,13 +936,13 @@ class TestM17N < Test::Unit::TestCase
   def test_count
     assert_equal(0, e("\xa1\xa2").count("z"))
     s = e("\xa3\xb0\xa3\xb1\xa3\xb2\xa3\xb3\xa3\xb4")
-    assert_raise(ArgumentError){s.count(a("\xa3\xb0"))}
+    assert_raise(Encoding::CompatibilityError){s.count(a("\xa3\xb0"))}
   end
 
   def test_delete
     assert_equal(1, e("\xa1\xa2").delete("z").length)
     s = e("\xa3\xb0\xa3\xb1\xa3\xb2\xa3\xb3\xa3\xb4")
-    assert_raise(ArgumentError){s.delete(a("\xa3\xb2"))}
+    assert_raise(Encoding::CompatibilityError){s.delete(a("\xa3\xb2"))}
 
     a = "\u3042\u3044\u3046\u3042\u3044\u3046"
     a.delete!("\u3042\u3044", "^\u3044")
@@ -942,7 +961,7 @@ class TestM17N < Test::Unit::TestCase
     assert_nil(e("\xa1\xa2\xa3\xa4").index(e("\xa3")))
     assert_nil(e("\xa1\xa2\xa3\xa4").rindex(e("\xa3")))
     s = e("\xa3\xb0\xa3\xb1\xa3\xb2\xa3\xb3\xa3\xb4")
-    assert_raise(ArgumentError){s.rindex(a("\xb1\xa3"))}
+    assert_raise(Encoding::CompatibilityError){s.rindex(a("\xb1\xa3"))}
   end
 
   def test_next
@@ -959,6 +978,21 @@ class TestM17N < Test::Unit::TestCase
     assert_equal(Encoding::EUC_JP, "\xa4\xa2".force_encoding("euc-jp").gsub(/./, '\&').encoding)
   end
 
+  def test_sub2
+    s = "\x80".force_encoding("ASCII-8BIT")
+    r = Regexp.new("\x80".force_encoding("ASCII-8BIT"))
+    s2 = s.sub(r, "")
+    assert(s2.empty?)
+    assert(s2.ascii_only?)
+  end
+
+  def test_sub3
+    repl = "\x81".force_encoding("sjis")
+    assert_equal(false, repl.valid_encoding?)
+    s = "a@".sub(/a/, repl)
+    assert(s.valid_encoding?)
+  end
+
   def test_insert
     s = e("\xa3\xb0\xa3\xb1\xa3\xb2\xa3\xb3\xa3\xb4")
     assert_equal(e("\xa3\xb0\xa3\xb1\xa3\xb2\xa3\xb3\xa3\xb4a"), s.insert(-1, "a"))
@@ -985,7 +1019,7 @@ class TestM17N < Test::Unit::TestCase
   def test_upto
     s1 = e("\xa1\xa2")
     s2 = s("\xa1\xa2")
-    assert_raise(ArgumentError){s1.upto(s2) {|x| break }}
+    assert_raise(Encoding::CompatibilityError){s1.upto(s2) {|x| break }}
   end
 
   def test_casecmp
@@ -1005,12 +1039,12 @@ class TestM17N < Test::Unit::TestCase
   end
 
   def test_plus
-    assert_raise(ArgumentError){u("\xe3\x81\x82") + a("\xa1")}
+    assert_raise(Encoding::CompatibilityError){u("\xe3\x81\x82") + a("\xa1")}
   end
 
   def test_chomp
     s = e("\xa3\xb0\xa3\xb1\xa3\xb2\xa3\xb3\xa3\xb4")
-    assert_raise(ArgumentError){s.chomp(s("\xa3\xb4"))}
+    assert_raise(Encoding::CompatibilityError){s.chomp(s("\xa3\xb4"))}
   end
 
   def test_gsub
@@ -1023,7 +1057,7 @@ class TestM17N < Test::Unit::TestCase
     t = s.gsub(/b/, "\xa1\xa1".force_encoding("euc-jp"))
     assert_equal(Encoding::ASCII_8BIT, s.encoding)
 
-    assert_raise(ArgumentError) {
+    assert_raise(Encoding::CompatibilityError) {
       "abc".gsub(/[ac]/) {
          $& == "a" ? "\xc2\xa1".force_encoding("euc-jp") :
                      "\xc2\xa1".force_encoding("utf-8")
@@ -1040,11 +1074,17 @@ class TestM17N < Test::Unit::TestCase
     s1 = s("\x81\x40")
     s2 = "@"
     assert_equal(false, s1.end_with?(s2), "#{encdump s1}.end_with?(#{encdump s2})")
+    each_encoding("\u3042\u3044", "\u3044") do |_s1, _s2|
+      assert_equal(true, _s1.end_with?(_s2), "#{encdump _s1}.end_with?(#{encdump _s2})")
+    end
+    each_encoding("\u3042a\u3044", "a\u3044") do |_s1, _s2|
+      assert_equal(true, _s1.end_with?(_s2), "#{encdump _s1}.end_with?(#{encdump _s2})")
+    end
   end
 
   def test_each_line
     s = e("\xa3\xb0\xa3\xb1\xa3\xb2\xa3\xb3\xa3\xb4")
-    assert_raise(ArgumentError){s.each_line(a("\xa3\xb1")) {|l| }}
+    assert_raise(Encoding::CompatibilityError){s.each_line(a("\xa3\xb1")) {|l| }}
     s = e("\xa4\xa2\nfoo")
 
     actual = []
@@ -1059,6 +1099,12 @@ class TestM17N < Test::Unit::TestCase
     assert_equal(a, s.each_char.to_a, "[ruby-dev:33211] #{encdump s}.each_char.to_a")
   end
 
+  def test_str_concat
+    assert_equal(1, "".concat(0xA2).size)
+    assert_equal("A\x84\x31\xA4\x39".force_encoding("GB18030"),
+                 "A".force_encoding("GB18030") << 0x8431A439)
+  end
+
   def test_regexp_match
     assert_equal([0,0], //.match("\xa1\xa1".force_encoding("euc-jp"),-1).offset(0))
     assert_equal(0, // =~ :a)
@@ -1068,6 +1114,10 @@ class TestM17N < Test::Unit::TestCase
     assert_equal(e("\xa1\xa2\xa1\xa3").split(//),
                  [e("\xa1\xa2"), e("\xa1\xa3")],
                  '[ruby-dev:32452]')
+
+    each_encoding("abc,def", ",", "abc", "def") do |str, sep, *expected|
+      assert_equal(expected, str.split(sep, -1))
+    end
   end
 
   def test_nonascii_method_name
@@ -1092,10 +1142,10 @@ class TestM17N < Test::Unit::TestCase
   end
 
   def test_symbol_op
-    ops = %w[
+    ops = %w"
       .. ... + - +(binary) -(binary) * / % ** +@ -@ | ^ & ! <=> > >= < <= ==
       === != =~ !~ ~ ! [] []= << >> :: `
-    ] #`
+    "
     ops.each do |op|
       assert_equal(Encoding::US_ASCII, op.intern.encoding, "[ruby-dev:33449]")
     end
@@ -1105,6 +1155,7 @@ class TestM17N < Test::Unit::TestCase
     0.upto(255) {|b|
       assert_equal([b].pack("C"), b.chr)
     }
+    assert_equal("\x84\x31\xA4\x39".force_encoding("GB18030"), 0x8431A439.chr("GB18030"))
   end
 
   def test_marshal
@@ -1114,9 +1165,10 @@ class TestM17N < Test::Unit::TestCase
   end
 
   def test_env
+    locale_encoding = Encoding.find("locale")
     ENV.each {|k, v|
-      assert_equal(Encoding::ASCII_8BIT, k.encoding)
-      assert_equal(Encoding::ASCII_8BIT, v.encoding)
+      assert_equal(locale_encoding, k.encoding)
+      assert_equal(locale_encoding, v.encoding)
     }
   end
 
@@ -1149,15 +1201,15 @@ class TestM17N < Test::Unit::TestCase
   end
 
   def test_bignum_to_s
-    assert_equal(Encoding::US_ASCII, (1<<129).to_s.encoding)
+    assert_equal(Encoding::US_ASCII, (1 << 129).to_s.encoding)
   end
 
   def test_array_to_s
     assert_equal(Encoding::US_ASCII, [].to_s.encoding)
     assert_equal(Encoding::US_ASCII, [nil].to_s.encoding)
     assert_equal(Encoding::US_ASCII, [1].to_s.encoding)
-    assert_equal(Encoding::US_ASCII, [""].to_s.encoding)
-    assert_equal(Encoding::US_ASCII, ["a"].to_s.encoding)
+    assert_equal("".inspect.encoding, [""].to_s.encoding)
+    assert_equal("a".inspect.encoding, ["a"].to_s.encoding)
     assert_equal(Encoding::US_ASCII, [nil,1,"","a","\x20",[]].to_s.encoding)
   end
 
@@ -1192,32 +1244,50 @@ class TestM17N < Test::Unit::TestCase
     assert_equal(Encoding::ASCII_8BIT, eval("# -*- encoding: ASCII-8BIT -*-\n__ENCODING__".force_encoding("US-ASCII")))
   end
 
+  def test_magic_comment_vim
+    assert_equal(Encoding::US_ASCII, eval("# vim: filetype=ruby, fileencoding: US-ASCII, ts=3, sw=3\n__ENCODING__".force_encoding("ASCII-8BIT")))
+    assert_equal(Encoding::ASCII_8BIT, eval("# vim: filetype=ruby, fileencoding: ASCII-8BIT, ts=3, sw=3\n__ENCODING__".force_encoding("US-ASCII")))
+  end
+
+  def test_magic_comment_at_various_positions
+    # after shebang
+    assert_equal(Encoding::US_ASCII, eval("#!/usr/bin/ruby\n# -*- encoding: US-ASCII -*-\n__ENCODING__".force_encoding("ASCII-8BIT")))
+    assert_equal(Encoding::ASCII_8BIT, eval("#!/usr/bin/ruby\n# -*- encoding: ASCII-8BIT -*-\n__ENCODING__".force_encoding("US-ASCII")))
+    # wrong position
+    assert_equal(Encoding::ASCII_8BIT, eval("\n# -*- encoding: US-ASCII -*-\n__ENCODING__".force_encoding("ASCII-8BIT")))
+    assert_equal(Encoding::US_ASCII, eval("\n# -*- encoding: ASCII-8BIT -*-\n__ENCODING__".force_encoding("US-ASCII")))
+
+    # leading expressions
+    assert_equal(Encoding::ASCII_8BIT, eval("v=1 # -*- encoding: US-ASCII -*-\n__ENCODING__".force_encoding("ASCII-8BIT")))
+    assert_equal(Encoding::US_ASCII, eval("v=1 # -*- encoding: ASCII-8BIT -*-\n__ENCODING__".force_encoding("US-ASCII")))
+  end
+
   def test_regexp_usascii
     assert_regexp_usascii_literal('//', Encoding::US_ASCII)
-    assert_regexp_usascii_literal('/#{}/', Encoding::US_ASCII)
+    assert_regexp_usascii_literal('/#{ }/', Encoding::US_ASCII)
     assert_regexp_usascii_literal('/#{"a"}/', Encoding::US_ASCII)
     assert_regexp_usascii_literal('/#{%q"\x80"}/', Encoding::ASCII_8BIT)
     assert_regexp_usascii_literal('/#{"\x80"}/', nil, SyntaxError)
 
     assert_regexp_usascii_literal('/a/', Encoding::US_ASCII)
-    assert_regexp_usascii_literal('/a#{}/', Encoding::US_ASCII)
+    assert_regexp_usascii_literal('/a#{ }/', Encoding::US_ASCII)
     assert_regexp_usascii_literal('/a#{"a"}/', Encoding::US_ASCII)
     assert_regexp_usascii_literal('/a#{%q"\x80"}/', Encoding::ASCII_8BIT)
     assert_regexp_usascii_literal('/a#{"\x80"}/', nil, SyntaxError)
 
     assert_regexp_usascii_literal('/\x80/', Encoding::ASCII_8BIT)
-    assert_regexp_usascii_literal('/\x80#{}/', Encoding::ASCII_8BIT)
+    assert_regexp_usascii_literal('/\x80#{ }/', Encoding::ASCII_8BIT)
     assert_regexp_usascii_literal('/\x80#{"a"}/', Encoding::ASCII_8BIT)
     assert_regexp_usascii_literal('/\x80#{%q"\x80"}/', Encoding::ASCII_8BIT)
     assert_regexp_usascii_literal('/\x80#{"\x80"}/', nil, SyntaxError)
 
     assert_regexp_usascii_literal('/\u1234/', Encoding::UTF_8)
-    assert_regexp_usascii_literal('/\u1234#{}/', Encoding::UTF_8)
+    assert_regexp_usascii_literal('/\u1234#{ }/', Encoding::UTF_8)
     assert_regexp_usascii_literal('/\u1234#{"a"}/', Encoding::UTF_8)
     assert_regexp_usascii_literal('/\u1234#{%q"\x80"}/', nil, SyntaxError)
     assert_regexp_usascii_literal('/\u1234#{"\x80"}/', nil, SyntaxError)
     assert_regexp_usascii_literal('/\u1234\x80/', nil, SyntaxError)
-    assert_regexp_usascii_literal('/\u1234#{}\x80/', nil, ArgumentError)
+    assert_regexp_usascii_literal('/\u1234#{ }\x80/', nil, RegexpError)
   end
 
   def test_gbk
@@ -1240,6 +1310,16 @@ class TestM17N < Test::Unit::TestCase
     s = "\xa1\xa1\x8f".force_encoding("euc-jp")
     assert_equal(false, s.valid_encoding?)
     assert_equal(true, s.reverse.valid_encoding?)
+
+    bug4018 = '[ruby-core:33027]'
+    s = "\xa1\xa1".force_encoding("euc-jp")
+    assert_equal(true, s.valid_encoding?)
+    s << "\x8f".force_encoding("euc-jp")
+    assert_equal(false, s.valid_encoding?, bug4018)
+    s = "aa".force_encoding("utf-16be")
+    assert_equal(true, s.valid_encoding?)
+    s << "\xff".force_encoding("utf-16be")
+    assert_equal(false, s.valid_encoding?, bug4018)
   end
 
   def test_getbyte
@@ -1262,11 +1342,33 @@ class TestM17N < Test::Unit::TestCase
   end
 
   def test_compatible
-    assert_equal(nil, Encoding.compatible?("",0), "moved from btest/knownbug")
+    assert_nil Encoding.compatible?("",0)
+    assert_equal(Encoding::UTF_8, Encoding.compatible?(u(""), ua("abc")))
+    assert_equal(Encoding::UTF_8, Encoding.compatible?(Encoding::UTF_8, Encoding::UTF_8))
+    assert_equal(Encoding::UTF_8, Encoding.compatible?(Encoding::UTF_8, Encoding::US_ASCII))
+    assert_equal(Encoding::ASCII_8BIT,
+                 Encoding.compatible?(Encoding::ASCII_8BIT, Encoding::US_ASCII))
+    assert_nil Encoding.compatible?(Encoding::UTF_8, Encoding::ASCII_8BIT)
   end
 
   def test_force_encoding
     assert(("".center(1, "\x80".force_encoding("utf-8")); true),
            "moved from btest/knownbug, [ruby-dev:33807]")
+    a = "".force_encoding("ascii-8bit") << 0xC3 << 0xB6
+    assert_equal(1, a.force_encoding("utf-8").size, '[ruby-core:22437]')
+    b = "".force_encoding("ascii-8bit") << 0xC3.chr << 0xB6.chr
+    assert_equal(1, b.force_encoding("utf-8").size, '[ruby-core:22437]')
+  end
+
+  def test_combchar_codepoint
+    assert_equal([0x30BB, 0x309A], "\u30BB\u309A".codepoints.to_a)
+  end
+
+  def each_encoding(*strings)
+    Encoding.list.each do |enc|
+      next if enc.dummy?
+      strs = strings.map {|s| s.encode(enc)} rescue next
+      yield *strs
+    end
   end
 end
