@@ -5,6 +5,10 @@ require_relative 'envutil'
 $m0 = Module.nesting
 
 class TestModule < Test::Unit::TestCase
+  def _wrap_assertion
+    yield
+  end
+
   def assert_method_defined?(klass, mid, message="")
     message = build_message(message, "#{klass}\##{mid} expected to be defined.")
     _wrap_assertion do
@@ -51,7 +55,7 @@ class TestModule < Test::Unit::TestCase
   end
 
   def test_CMP_0
-    assert_equal -1, (String <=> Object)
+    assert_equal(-1, (String <=> Object))
     assert_equal 1, (Object <=> String)
     assert_nil(Array <=> String)
   end
@@ -68,6 +72,12 @@ class TestModule < Test::Unit::TestCase
     list.reject {|c| c.to_s.start_with?("JSON") }
   end
 
+  def remove_rake_mixins(list)
+    list.
+      reject {|c| c.to_s == "RakeFileUtils" }.
+      reject {|c| c.to_s.start_with?("FileUtils") }
+  end
+
   module Mixin
     MIXIN = 1
     def mixin
@@ -96,16 +106,19 @@ class TestModule < Test::Unit::TestCase
     def AClass.cm3
       "cm3"
     end
-    
+
     private_class_method :cm1, "cm3"
 
     def aClass
+      :aClass
     end
 
     def aClass1
+      :aClass1
     end
 
     def aClass2
+      :aClass2
     end
 
     private :aClass1
@@ -114,15 +127,23 @@ class TestModule < Test::Unit::TestCase
 
   class BClass < AClass
     def bClass1
+      :bClass1
     end
 
     private
 
     def bClass2
+      :bClass2
     end
 
     protected
     def bClass3
+      :bClass3
+    end
+  end
+  
+  class CClass < BClass
+    def self.cClass
     end
   end
 
@@ -195,23 +216,34 @@ class TestModule < Test::Unit::TestCase
     assert_equal([Mixin],            Mixin.ancestors)
 
     assert_equal([Object, Kernel, BasicObject],
-                 remove_json_mixins(remove_pp_mixins(Object.ancestors)))
+                 remove_rake_mixins(remove_json_mixins(remove_pp_mixins(Object.ancestors))))
     assert_equal([String, Comparable, Object, Kernel, BasicObject],
-                 remove_json_mixins(remove_pp_mixins(String.ancestors)))
+                 remove_rake_mixins(remove_json_mixins(remove_pp_mixins(String.ancestors))))
   end
 
+  CLASS_EVAL = 2
+  @@class_eval = 'b'
+
   def test_class_eval
     Other.class_eval("CLASS_EVAL = 1")
     assert_equal(1, Other::CLASS_EVAL)
     assert(Other.constants.include?(:CLASS_EVAL))
-  end
+    assert_equal(2, Other.class_eval { CLASS_EVAL })
 
-  def test_class_variable_set
-    # TODO
-  end
+    Other.class_eval("@@class_eval = 'a'")
+    assert_equal('a', Other.class_variable_get(:@@class_eval))
+    assert_equal('b', Other.class_eval { @@class_eval })
 
-  def test_class_variable_get
-    # TODO
+    Other.class_eval do
+      module_function
+
+      def class_eval_test
+        "foo"
+      end
+    end
+    assert_equal("foo", Other.class_eval_test)
+
+    assert_equal([Other], Other.class_eval { |*args| args })
   end
 
   def test_const_defined?
@@ -244,9 +276,9 @@ class TestModule < Test::Unit::TestCase
     assert_equal([], Mixin.included_modules)
     assert_equal([Mixin], User.included_modules)
     assert_equal([Kernel],
-                 remove_json_mixins(remove_pp_mixins(Object.included_modules)))
+                 remove_rake_mixins(remove_json_mixins(remove_pp_mixins(Object.included_modules))))
     assert_equal([Comparable, Kernel],
-                 remove_json_mixins(remove_pp_mixins(String.included_modules)))
+                 remove_rake_mixins(remove_json_mixins(remove_pp_mixins(String.included_modules))))
   end
 
   def test_instance_methods
@@ -254,6 +286,9 @@ class TestModule < Test::Unit::TestCase
     assert_equal([:user, :mixin].sort, User.instance_methods(true).sort)
     assert_equal([:mixin], Mixin.instance_methods)
     assert_equal([:mixin], Mixin.instance_methods(true))
+    assert_equal([:cClass], (class << CClass; self; end).instance_methods(false))
+    assert_equal([], (class << BClass; self; end).instance_methods(false))
+    assert_equal([:cm2], (class << AClass; self; end).instance_methods(false))
     # Ruby 1.8 feature change:
     # #instance_methods includes protected methods.
     #assert_equal([:aClass], AClass.instance_methods(false))
@@ -353,6 +388,21 @@ class TestModule < Test::Unit::TestCase
     Object.module_eval "WALTER = 99"
     c2 = Module.constants
     assert_equal([:WALTER], c2 - c1)
+
+    assert_equal([], Module.constants(true))
+    assert_equal([], Module.constants(false))
+
+    src = <<-INPUT
+      ary = Module.constants
+      module M
+        WALTER = 99
+      end
+      class Module
+        include M
+      end
+      p Module.constants - ary, Module.constants(true), Module.constants(false)
+    INPUT
+    assert_in_out_err([], src, %w([:M] [:WALTER] []), [])
   end
 
   module M1
@@ -401,7 +451,7 @@ class TestModule < Test::Unit::TestCase
     assert_equal(false, o.respond_to?(:bar=))
   end
 
-  def test_const_get2
+  def test_const_get_evaled
     c1 = Class.new
     c2 = Class.new(c1)
 
@@ -431,39 +481,77 @@ class TestModule < Test::Unit::TestCase
     assert_raise(NameError) { c1.const_get(:foo) }
   end
 
-  def test_const_set2
+  def test_const_set_invalid_name
     c1 = Class.new
     assert_raise(NameError) { c1.const_set(:foo, :foo) }
   end
 
-  def test_const_get3
+  def test_const_get_invalid_name
     c1 = Class.new
     assert_raise(NameError) { c1.const_defined?(:foo) }
   end
 
-  def test_class_variable_get2
+  def test_const_get_no_inherited
+    bug3422 = '[ruby-core:30719]'
+    assert_in_out_err([], <<-INPUT, %w[1 NameError A], [], bug3422)
+    BasicObject::A = 1
+    puts [true, false].map {|inh|
+      begin
+        Object.const_get(:A, inh)
+      rescue NameError => e
+        [e.class, e.name]
+      end
+    }
+    INPUT
+  end
+
+  def test_const_get_inherited
+    bug3423 = '[ruby-core:30720]'
+    assert_in_out_err([], <<-INPUT, %w[NameError A NameError A], [], bug3423)
+    module Foo; A = 1; end
+    class Object; include Foo; end
+    class Bar; include Foo; end
+
+    puts [Object, Bar].map {|klass|
+      begin
+        klass.const_get(:A, false)
+      rescue NameError => e
+        [e.class, e.name]
+      end
+    }
+    INPUT
+  end
+
+  def test_class_variable_get
     c = Class.new
-    c.class_eval { @@foo = :foo }
+    c.class_eval('@@foo = :foo')
     assert_equal(:foo, c.class_variable_get(:@@foo))
     assert_raise(NameError) { c.class_variable_get(:@@bar) } # c.f. instance_variable_get
     assert_raise(NameError) { c.class_variable_get(:foo) }
   end
 
-  def test_class_variable_set2
+  def test_class_variable_set
     c = Class.new
     c.class_variable_set(:@@foo, :foo)
-    assert_equal(:foo, c.class_eval { @@foo })
+    assert_equal(:foo, c.class_eval('@@foo'))
     assert_raise(NameError) { c.class_variable_set(:foo, 1) }
   end
 
   def test_class_variable_defined
     c = Class.new
-    c.class_eval { @@foo = :foo }
+    c.class_eval('@@foo = :foo')
     assert_equal(true, c.class_variable_defined?(:@@foo))
     assert_equal(false, c.class_variable_defined?(:@@bar))
     assert_raise(NameError) { c.class_variable_defined?(:foo) }
   end
 
+  def test_remove_class_variable
+    c = Class.new
+    c.class_eval('@@foo = :foo')
+    c.class_eval { remove_class_variable(:@@foo) }
+    assert_equal(false, c.class_variable_defined?(:@@foo))
+  end
+
   def test_export_method
     m = Module.new
     assert_raise(NameError) do
@@ -486,7 +574,7 @@ class TestModule < Test::Unit::TestCase
 
     c = Class.new
     assert_raise(NameError) do
-      c.instance_eval { attr_reader :"$" }
+      c.instance_eval { attr_reader :"." }
     end
   end
 
@@ -513,10 +601,10 @@ class TestModule < Test::Unit::TestCase
       class << o; self; end.instance_eval { undef_method(:foo) }
     end
 
-    %w(object_id __send__ initialize).each do |m|
-      assert_in_out_err([], <<-INPUT, [], /warning: undefining `#{m}' may cause serious problem$/)
+    %w(object_id __send__ initialize).each do |n|
+      assert_in_out_err([], <<-INPUT, [], /warning: undefining `#{n}' may cause serious problems$/)
         $VERBOSE = false
-        Class.new.instance_eval { undef_method(:#{m}) }
+        Class.new.instance_eval { undef_method(:#{n}) }
       INPUT
     end
   end
@@ -527,7 +615,7 @@ class TestModule < Test::Unit::TestCase
       m.class_eval { alias foo bar }
     end
 
-    assert_in_out_err([], <<-INPUT, %w(2), /warning: discarding old foo$/)
+    assert_in_out_err([], <<-INPUT, %w(2), /discarding old foo$/)
       $VERBOSE = true
       c = Class.new
       c.class_eval do
@@ -540,10 +628,11 @@ class TestModule < Test::Unit::TestCase
   end
 
   def test_mod_constants
-    Module.const_set(:Foo, :foo)
-    assert_equal([:Foo], Module.constants(true))
-    assert_equal([:Foo], Module.constants(false))
-    Module.instance_eval { remove_const(:Foo) }
+    m = Module.new
+    m.const_set(:Foo, :foo)
+    assert_equal([:Foo], m.constants(true))
+    assert_equal([:Foo], m.constants(false))
+    m.instance_eval { remove_const(:Foo) }
   end
 
   def test_frozen_class
@@ -717,4 +806,149 @@ class TestModule < Test::Unit::TestCase
       }.call
     end
   end
+
+  def test_send
+    a = AClass.new
+    assert_equal(:aClass, a.__send__(:aClass))
+    assert_equal(:aClass1, a.__send__(:aClass1))
+    assert_equal(:aClass2, a.__send__(:aClass2))
+    b = BClass.new
+    assert_equal(:aClass, b.__send__(:aClass))
+    assert_equal(:aClass1, b.__send__(:aClass1))
+    assert_equal(:aClass2, b.__send__(:aClass2))
+    assert_equal(:bClass1, b.__send__(:bClass1))
+    assert_equal(:bClass2, b.__send__(:bClass2))
+    assert_equal(:bClass3, b.__send__(:bClass3))
+  end
+
+
+  def test_nonascii_name
+    c = eval("class ::C\u{df}; self; end")
+    assert_equal("C\u{df}", c.name, '[ruby-core:24600]')
+    c = eval("class C\u{df}; self; end")
+    assert_equal("TestModule::C\u{df}", c.name, '[ruby-core:24600]')
+  end
+
+  def test_method_added
+    memo = []
+    mod = Module.new do
+      mod = self
+      (class << self ; self ; end).class_eval do
+        define_method :method_added do |sym|
+          memo << sym
+          memo << mod.instance_methods(false)
+          memo << (mod.instance_method(sym) rescue nil)
+        end
+      end
+      def f
+      end
+      alias g f
+      attr_reader :a
+      attr_writer :a
+    end
+    assert_equal :f, memo.shift
+    assert_equal [:f], memo.shift, '[ruby-core:25536]'
+    assert_equal mod.instance_method(:f), memo.shift
+    assert_equal :g, memo.shift
+    assert_equal [:f, :g], memo.shift
+    assert_equal mod.instance_method(:f), memo.shift
+    assert_equal :a, memo.shift
+    assert_equal [:f, :g, :a], memo.shift
+    assert_equal mod.instance_method(:a), memo.shift
+    assert_equal :a=, memo.shift
+    assert_equal [:f, :g, :a, :a=], memo.shift
+    assert_equal mod.instance_method(:a=), memo.shift
+  end
+
+  def test_method_redefinition
+    feature2155 = '[ruby-dev:39400]'
+
+    line = __LINE__+4
+    stderr = EnvUtil.verbose_warning do
+      Module.new do
+        def foo; end
+        def foo; end
+      end
+    end
+    assert_match(/:#{line}: warning: method redefined; discarding old foo/, stderr)
+    assert_match(/:#{line-1}: warning: previous definition of foo/, stderr, feature2155)
+
+    stderr = EnvUtil.verbose_warning do
+      Module.new do
+        def foo; end
+        alias bar foo
+        def foo; end
+      end
+    end
+    assert_equal("", stderr)
+
+    stderr = EnvUtil.verbose_warning do
+      Module.new do
+        def foo; end
+        alias bar foo
+        alias bar foo
+      end
+    end
+    assert_equal("", stderr)
+
+    line = __LINE__+4
+    stderr = EnvUtil.verbose_warning do
+      Module.new do
+        define_method(:foo) do end
+        def foo; end
+      end
+    end
+    assert_match(/:#{line}: warning: method redefined; discarding old foo/, stderr)
+    assert_match(/:#{line-1}: warning: previous definition of foo/, stderr, feature2155)
+
+    stderr = EnvUtil.verbose_warning do
+      Module.new do
+        define_method(:foo) do end
+        alias bar foo
+        alias barf oo
+      end
+    end
+    assert_equal("", stderr)
+
+    stderr = EnvUtil.verbose_warning do
+      Module.new do
+        module_function
+        def foo; end
+        module_function :foo
+      end
+    end
+    assert_equal("", stderr, '[ruby-dev:39397]')
+
+    stderr = EnvUtil.verbose_warning do
+      Module.new do
+        def foo; end
+        undef foo
+      end
+    end
+    assert_equal("", stderr)
+  end
+
+  def test_protected_singleton_method
+    klass = Class.new
+    x = klass.new
+    class << x
+      protected
+
+      def foo
+      end
+    end
+    assert_raise(NoMethodError) do
+      x.foo
+    end
+    klass.send(:define_method, :bar) do
+      x.foo
+    end
+    assert_nothing_raised do
+      x.bar
+    end
+    y = klass.new
+    assert_raise(NoMethodError) do
+      y.bar
+    end
+  end
 end
