@@ -22,7 +22,7 @@ class TestString < Test::Unit::TestCase
   def test_s_new
     assert_equal("RUBY", S("RUBY"))
   end
-    
+
   def test_AREF # '[]'
     assert_equal("A",  S("AooBar")[0])
     assert_equal("B",  S("FooBaB")[-1])
@@ -175,9 +175,11 @@ class TestString < Test::Unit::TestCase
     def o.<=>(x); nil; end
     assert_nil("foo" <=> o)
 
+    class << o;remove_method :<=>;end
     def o.<=>(x); 1; end
     assert_equal(-1, "foo" <=> o)
 
+    class << o;remove_method :<=>;end
     def o.<=>(x); 2**100; end
     assert_equal(-(2**100), "foo" <=> o)
   end
@@ -200,6 +202,7 @@ class TestString < Test::Unit::TestCase
     def o.to_str; end
     def o.==(x); false; end
     assert_equal(false, "foo" == o)
+    class << o;remove_method :==;end
     def o.==(x); true; end
     assert_equal(true, "foo" == o)
   end
@@ -218,13 +221,20 @@ class TestString < Test::Unit::TestCase
     l = s.size
     s << "bar"
     assert_equal(l + 3, s.size)
+
+    bug = '[ruby-core:27583]'
+    assert_raise(RangeError, bug) {S("a".force_encoding(Encoding::UTF_8)) << -3}
+    assert_raise(RangeError, bug) {S("a".force_encoding(Encoding::UTF_8)) << -2}
+    assert_raise(RangeError, bug) {S("a".force_encoding(Encoding::UTF_8)) << -1}
+    assert_raise(RangeError, bug) {S("a".force_encoding(Encoding::UTF_8)) << 0x81308130}
+    assert_nothing_raised {S("a".force_encoding(Encoding::GB18030)) << 0x81308130}
   end
 
   def test_MATCH # '=~'
     assert_equal(10,  S("FeeFieFoo-Fum") =~ /Fum$/)
     assert_equal(nil, S("FeeFieFoo-Fum") =~ /FUM$/)
 
-    pre_1_7_1 do 
+    pre_1_7_1 do
       $= = true
       assert_equal(10,  S("FeeFieFoo-Fum") =~ /FUM$/)
       $= = false
@@ -279,7 +289,7 @@ class TestString < Test::Unit::TestCase
   def test_VERY_EQUAL # '==='
     # assert_equal(true, S("foo") === :foo)
     casetest(S("abcdef"), S("abcdef"))
-    
+
     pre_1_7_1 do
       $= = true
       casetest(S("CAT"), S('cat'))
@@ -317,12 +327,15 @@ class TestString < Test::Unit::TestCase
     b = a.dup
     assert_equal(S("Hello"), a.capitalize!)
     assert_equal(S("hello"), b)
-   
+
   end
 
+  Bug2463 = '[ruby-dev:39856]'
   def test_center
     assert_equal(S("hello"),       S("hello").center(4))
     assert_equal(S("   hello   "), S("hello").center(11))
+    assert_equal(S("ababaababa"), S("").center(10, "ab"), Bug2463)
+    assert_equal(S("ababaababab"), S("").center(11, "ab"), Bug2463)
   end
 
   def test_chomp
@@ -339,6 +352,8 @@ class TestString < Test::Unit::TestCase
     assert_equal(S("hello"), S("hello").chomp)
     assert_equal(S("hello"), S("hello!").chomp)
     $/ = save
+
+    assert_equal(S("a").hash, S("a\u0101").chomp(S("\u0101")).hash, '[ruby-core:22414]')
   end
 
   def test_chomp!
@@ -393,6 +408,8 @@ class TestString < Test::Unit::TestCase
     s = "foo\r"
     s.chomp!("")
     assert_equal("foo\r", s)
+
+    assert_equal(S("a").hash, S("a\u0101").chomp!(S("\u0101")).hash, '[ruby-core:22414]')
   end
 
   def test_chop
@@ -401,6 +418,7 @@ class TestString < Test::Unit::TestCase
     assert_equal(S("hello\n"), S("hello\n\r").chop)
     assert_equal(S(""),        S("\r\n").chop)
     assert_equal(S(""),        S("").chop)
+    assert_equal(S("a").hash,  S("a\u00d8").chop.hash)
   end
 
   def test_chop!
@@ -419,6 +437,10 @@ class TestString < Test::Unit::TestCase
     a = S("").chop!
     assert_nil(a)
 
+    a = S("a\u00d8")
+    a.chop!
+    assert_equal(S("a").hash, a.hash)
+
     a = S("hello\n")
     b = a.dup
     assert_equal(S("hello"),   a.chop!)
@@ -427,16 +449,20 @@ class TestString < Test::Unit::TestCase
 
   def test_clone
     for taint in [ false, true ]
-      for frozen in [ false, true ]
-        a = S("Cool")
-        a.taint  if taint
-        a.freeze if frozen
-        b = a.clone
+      for untrust in [ false, true ]
+        for frozen in [ false, true ]
+          a = S("Cool")
+          a.taint  if taint
+          a.untrust  if untrust
+          a.freeze if frozen
+          b = a.clone
 
-        assert_equal(a, b)
-        assert(a.__id__ != b.__id__)
-        assert_equal(a.frozen?, b.frozen?)
-        assert_equal(a.tainted?, b.tainted?)
+          assert_equal(a, b)
+          assert(a.__id__ != b.__id__)
+          assert_equal(a.frozen?, b.frozen?)
+          assert_equal(a.untrusted?, b.untrusted?)
+          assert_equal(a.tainted?, b.tainted?)
+        end
       end
     end
 
@@ -456,6 +482,8 @@ class TestString < Test::Unit::TestCase
     assert_equal(4, a.count(S("hello"), S("^l")))
     assert_equal(4, a.count(S("ej-m")))
     assert_equal(0, S("y").count(S("a\\-z")))
+    assert_equal(5, "abc\u{3042 3044 3046}".count("^a"))
+    assert_equal(5, "abc\u{3042 3044 3046}".count("^\u3042"))
 
     assert_raise(ArgumentError) { "foo".count }
   end
@@ -470,6 +498,15 @@ class TestString < Test::Unit::TestCase
     assert_equal(S("he"),   S("hello").delete(S("lo")))
     assert_equal(S("hell"), S("hello").delete(S("aeiou"), S("^e")))
     assert_equal(S("ho"),   S("hello").delete(S("ej-m")))
+
+    assert_equal("a".hash, "a\u0101".delete("\u0101").hash, '[ruby-talk:329267]')
+    assert_equal(true, "a\u0101".delete("\u0101").ascii_only?)
+    assert_equal(true, "a\u3041".delete("\u3041").ascii_only?)
+    assert_equal(false, "a\u3041\u3042".tr("\u3041", "a").ascii_only?)
+
+    assert_equal("a", "abc\u{3042 3044 3046}".delete("^a"))
+    assert_equal("bc\u{3042 3044 3046}", "abc\u{3042 3044 3046}".delete("a"))
+    assert_equal("\u3042", "abc\u{3042 3044 3046}".delete("^\u3042"))
   end
 
   def test_delete!
@@ -532,18 +569,22 @@ class TestString < Test::Unit::TestCase
 
   def test_dup
     for taint in [ false, true ]
-      for frozen in [ false, true ]
-        a = S("hello")
-        a.taint  if taint
-        a.freeze if frozen
-        b = a.dup 
+      for untrust in [ false, true ]
+        for frozen in [ false, true ]
+          a = S("hello")
+          a.taint  if taint
+          a.untrust  if untrust
+          a.freeze if frozen
+          b = a.dup
 
-        assert_equal(a, b)
-        assert(a.__id__ != b.__id__)
-        assert(!b.frozen?)
-        assert_equal(a.tainted?, b.tainted?)
+          assert_equal(a, b)
+          assert(a.__id__ != b.__id__)
+          assert(!b.frozen?)
+          assert_equal(a.tainted?, b.tainted?)
+          assert_equal(a.untrusted?, b.untrusted?)
+        end
       end
-    end     
+    end
   end
 
   def test_each
@@ -553,12 +594,12 @@ class TestString < Test::Unit::TestCase
     S("hello\nworld").lines.each {|x| res << x}
     assert_equal(S("hello\n"), res[0])
     assert_equal(S("world"),   res[1])
-    
+
     res=[]
     S("hello\n\n\nworld").lines(S('')).each {|x| res << x}
     assert_equal(S("hello\n\n\n"), res[0])
     assert_equal(S("world"),       res[1])
-    
+
     $/ = "!"
     res=[]
     S("hello!world").lines.each {|x| res << x}
@@ -587,14 +628,14 @@ class TestString < Test::Unit::TestCase
     S("hello\n\n\nworld").lines(S('')).each {|x| res << x}
     assert_equal(S("hello\n\n\n"), res[0])
     assert_equal(S("world"),       res[1])
-    
+
     $/ = "!"
 
     res=[]
     S("hello!world").lines.each {|x| res << x}
     assert_equal(S("hello!"), res[0])
     assert_equal(S("world"),  res[1])
-    
+
     $/ = save
 
     s = nil
@@ -618,12 +659,14 @@ class TestString < Test::Unit::TestCase
     assert_equal(S("h<e>ll<o>"), S("hello").gsub(/([aeiou])/, S('<\1>')))
     assert_equal(S("h e l l o "),
                  S("hello").gsub(/./) { |s| s[0].to_s + S(' ')})
-    assert_equal(S("HELL-o"), 
+    assert_equal(S("HELL-o"),
                  S("hello").gsub(/(hell)(.)/) { |s| $1.upcase + S('-') + $2 })
 
     a = S("hello")
     a.taint
+    a.untrust
     assert(a.gsub(/./, S('X')).tainted?)
+    assert(a.gsub(/./, S('X')).untrusted?)
 
     assert_equal("z", "abc".gsub(/./, "a" => "z"), "moved from btest/knownbug")
 
@@ -651,8 +694,10 @@ class TestString < Test::Unit::TestCase
 
     r = S('X')
     r.taint
+    r.untrust
     a.gsub!(/./, r)
-    assert(a.tainted?) 
+    assert(a.tainted?)
+    assert(a.untrusted?)
 
     a = S("hello")
     assert_nil(a.sub!(S('X'), S('Y')))
@@ -753,6 +798,8 @@ class TestString < Test::Unit::TestCase
   def test_ljust
     assert_equal(S("hello"),       S("hello").ljust(4))
     assert_equal(S("hello      "), S("hello").ljust(11))
+    assert_equal(S("ababababab"), S("").ljust(10, "ab"), Bug2463)
+    assert_equal(S("abababababa"), S("").ljust(11, "ab"), Bug2463)
   end
 
   def test_next
@@ -823,9 +870,11 @@ class TestString < Test::Unit::TestCase
 
     a = S("foo")
     a.taint
+    a.untrust
     b = a.replace(S("xyz"))
     assert_equal(S("xyz"), b)
     assert(b.tainted?)
+    assert(b.untrusted?)
 
     s = "foo" * 100
     s2 = ("bar" * 100).dup
@@ -835,6 +884,12 @@ class TestString < Test::Unit::TestCase
     s2 = ["foo"].pack("p")
     s.replace(s2)
     assert_equal(s2, s)
+
+    fs = "".freeze
+    assert_raise(RuntimeError) { fs.replace("a") }
+    assert_raise(RuntimeError) { fs.replace(fs) }
+    assert_raise(ArgumentError) { fs.replace() }
+    assert_raise(RuntimeError) { fs.replace(42) }
   end
 
   def test_reverse
@@ -889,6 +944,8 @@ class TestString < Test::Unit::TestCase
   def test_rjust
     assert_equal(S("hello"), S("hello").rjust(4))
     assert_equal(S("      hello"), S("hello").rjust(11))
+    assert_equal(S("ababababab"), S("").rjust(10, "ab"), Bug2463)
+    assert_equal(S("abababababa"), S("").rjust(11, "ab"), Bug2463)
   end
 
   def test_scan
@@ -908,6 +965,14 @@ class TestString < Test::Unit::TestCase
     res = []
     a.scan(/(...)/) { |w| res << w }
     assert_equal([[S("cru")], [S("el ")], [S("wor")]],res)
+
+    a = S("hello")
+    a.taint
+    a.untrust
+    res = []
+    a.scan(/./) { |w| res << w }
+    assert(res[0].tainted?, '[ruby-core:33338] #4087')
+    assert(res[0].untrusted?, '[ruby-core:33338] #4087')
   end
 
   def test_size
@@ -960,12 +1025,12 @@ class TestString < Test::Unit::TestCase
       assert_nil( a.slice!(6) )
     else
       assert_raise(IndexError) { a.slice!(6) }
-    end 
+    end
     assert_equal(S("FooBar"), a)
 
     if @aref_slicebang_silent
-      assert_nil( a.slice!(-7) ) 
-    else 
+      assert_nil( a.slice!(-7) )
+    else
       assert_raise(IndexError) { a.slice!(-7) }
     end
     assert_equal(S("FooBar"), a)
@@ -1111,6 +1176,11 @@ class TestString < Test::Unit::TestCase
   def test_strip
     assert_equal(S("x"), S("      x        ").strip)
     assert_equal(S("x"), S(" \n\r\t     x  \t\r\n\n      ").strip)
+
+    assert_equal("0b0 ".force_encoding("UTF-16BE"),
+                 "\x00 0b0 ".force_encoding("UTF-16BE").strip)
+    assert_equal("0\x000b0 ".force_encoding("UTF-16BE"),
+                 "0\x000b0 ".force_encoding("UTF-16BE").strip)
   end
 
   def test_strip!
@@ -1170,7 +1240,10 @@ class TestString < Test::Unit::TestCase
 
     a = S("hello")
     a.taint
-    assert(a.sub(/./, S('X')).tainted?)
+    a.untrust
+    x = a.sub(/./, S('X'))
+    assert(x.tainted?)
+    assert(x.untrusted?)
 
     o = Object.new
     def o.to_str; "bar"; end
@@ -1211,8 +1284,10 @@ class TestString < Test::Unit::TestCase
 
     r = S('X')
     r.taint
+    r.untrust
     a.sub!(/./, r)
-    assert(a.tainted?) 
+    assert(a.tainted?)
+    assert(a.untrusted?)
   end
 
   def test_succ
@@ -1306,6 +1381,15 @@ class TestString < Test::Unit::TestCase
     }
   end
 
+  def test_sum_long
+    s8421505 = "\xff" * 8421505
+    assert_equal(127, s8421505.sum(31))
+    assert_equal(2147483775, s8421505.sum(0))
+    s16843010 = ("\xff" * 16843010)
+    assert_equal(254, s16843010.sum(32))
+    assert_equal(4294967550, s16843010.sum(0))
+  end
+
   def test_swapcase
     assert_equal(S("hi&LOW"), S("HI&low").swapcase)
   end
@@ -1327,6 +1411,8 @@ class TestString < Test::Unit::TestCase
     assert_equal(5.9742e24, S("5.9742e24").to_f)
     assert_equal(98.6,      S("98.6 degrees").to_f)
     assert_equal(0.0,       S("degrees 100.0").to_f)
+    assert_equal([ 0.0].pack('G'), [S(" 0.0").to_f].pack('G'))
+    assert_equal([-0.0].pack('G'), [S("-0.0").to_f].pack('G'))
   end
 
   def test_to_i
@@ -1370,12 +1456,32 @@ class TestString < Test::Unit::TestCase
     a = S("me")
     assert_equal("me", a.to_s)
     assert_equal(a.__id__, a.to_s.__id__) if @cls == String
+
+    o = Object.new
+    def o.to_str
+      "at"
+    end
+    assert_equal("meat", a.concat(o))
+
+    o = Object.new
+    def o.to_str
+      foo_bar()
+    end
+    assert_match(/foo_bar/, assert_raise(NoMethodError) {a.concat(o)}.message)
   end
 
   def test_tr
     assert_equal(S("hippo"), S("hello").tr(S("el"), S("ip")))
     assert_equal(S("*e**o"), S("hello").tr(S("^aeiou"), S("*")))
     assert_equal(S("hal"),   S("ibm").tr(S("b-z"), S("a-z")))
+
+    a = "abc".force_encoding(Encoding::US_ASCII)
+    assert_equal(Encoding::US_ASCII, a.tr(S("z"), S("\u0101")).encoding, '[ruby-core:22326]')
+
+    assert_equal("a".hash, "a".tr("a", "\u0101").tr("\u0101", "a").hash, '[ruby-core:22328]')
+    assert_equal(true, "\u0101".tr("\u0101", "a").ascii_only?)
+    assert_equal(true, "\u3041".tr("\u3041", "a").ascii_only?)
+    assert_equal(false, "\u3041\u3042".tr("\u3041", "a").ascii_only?)
   end
 
   def test_tr!
@@ -1396,11 +1502,17 @@ class TestString < Test::Unit::TestCase
     a = S("ibm")
     assert_nil(a.tr!(S("B-Z"), S("A-Z")))
     assert_equal(S("ibm"), a)
+
+    a = "abc".force_encoding(Encoding::US_ASCII)
+    assert_nil(a.tr!(S("z"), S("\u0101")), '[ruby-core:22326]')
+    assert_equal(Encoding::US_ASCII, a.encoding, '[ruby-core:22326]')
   end
 
   def test_tr_s
     assert_equal(S("hypo"), S("hello").tr_s(S("el"), S("yp")))
     assert_equal(S("h*o"),  S("hello").tr_s(S("el"), S("*")))
+    assert_equal("a".hash, "\u0101\u0101".tr_s("\u0101", "a").hash)
+    assert_equal(true, "\u3041\u3041".tr("\u3041", "a").ascii_only?)
   end
 
   def test_tr_s!
@@ -1452,7 +1564,7 @@ class TestString < Test::Unit::TestCase
     assert_equal([ 65, 66, 67 ],  S("ABC").unpack("c3"))
     assert_equal([ -1, 66, 67 ],  S("\377BC").unpack("c*"))
 
-    
+
     assert_equal([S("4142"), S("0a"), S("1")], S("AB\n\x10").unpack(S("H4H2H1")))
     assert_equal([S("1424"), S("a0"), S("2")], S("AB\n\x02").unpack(S("h4h2h1")))
 
@@ -1525,6 +1637,30 @@ class TestString < Test::Unit::TestCase
     assert_equal(676, count)
   end
 
+  def test_upto_numeric
+    a     = S("00")
+    start = S("00")
+    count = 0
+    assert_equal(S("00"), a.upto(S("23")) {|s|
+                   assert_equal(start, s, "[ruby-dev:39361]")
+                   assert_equal(Encoding::US_ASCII, s.encoding)
+                   start.succ!
+                   count += 1
+                   })
+    assert_equal(24, count, "[ruby-dev:39361]")
+  end
+
+  def test_upto_nonalnum
+    first = S("\u3041")
+    last  = S("\u3093")
+    count = 0
+    assert_equal(first, first.upto(last) {|s|
+                   count += 1
+                   s.replace(last)
+                   })
+    assert_equal(83, count, "[ruby-dev:39626]")
+  end
+
   def test_mod_check
     assert_raise(RuntimeError) {
       s = ""
@@ -1617,7 +1753,7 @@ class TestString < Test::Unit::TestCase
   end
 
   def test_respond_to
-    o = Object.new 
+    o = Object.new
     def o.respond_to?(arg) [:to_str].include?(arg) ? nil : super end
     def o.to_str() "" end
     def o.==(other) "" == other end
@@ -1655,12 +1791,16 @@ class TestString < Test::Unit::TestCase
     assert_equal(%w(he l lo), "hello".partition(/l/))
     assert_equal(%w(he l lo), "hello".partition("l"))
     assert_raise(TypeError) { "hello".partition(1) }
+    def (hyphen = Object.new).to_str; "-"; end
+    assert_equal(%w(foo - bar), "foo-bar".partition(hyphen), '[ruby-core:23540]')
   end
 
   def test_rpartition
     assert_equal(%w(hel l o), "hello".rpartition(/l/))
     assert_equal(%w(hel l o), "hello".rpartition("l"))
     assert_raise(TypeError) { "hello".rpartition(1) }
+    def (hyphen = Object.new).to_str; "-"; end
+    assert_equal(%w(foo - bar), "foo-bar".rpartition(hyphen), '[ruby-core:23540]')
   end
 
   def test_setter
@@ -1685,6 +1825,7 @@ class TestString < Test::Unit::TestCase
       c.class_eval { attr 1 }
     end
 
+    class << o;remove_method :to_str;end
     def o.to_str; "foo"; end
     assert_nothing_raised do
       c.class_eval { attr o }
@@ -1695,4 +1836,78 @@ class TestString < Test::Unit::TestCase
   def test_gsub_enumerator
     assert_normal_exit %q{"abc".gsub(/./).next}, "[ruby-dev:34828]"
   end
+
+  def test_clear_nonasciicompat
+    assert_equal("", "\u3042".encode("ISO-2022-JP").clear)
+  end
+
+  def test_try_convert
+    assert_equal(nil, String.try_convert(1))
+    assert_equal("foo", String.try_convert("foo"))
+  end
+
+  def test_substr_negative_begin
+    assert_equal("\u3042", ("\u3042" * 100)[-1])
+  end
+
+  def test_compare_different_encoding_string
+    s1 = "\xff".force_encoding("UTF-8")
+    s2 = "\xff".force_encoding("ISO-2022-JP")
+    #assert_equal([-1, 1], [s1 <=> s2, s2 <=> s1].sort)
+  end
+
+  def test_casecmp
+    assert_equal(1, "\u3042B".casecmp("\u3042a"))
+  end
+
+  def test_upcase2
+    assert_equal("\u3042AB", "\u3042aB".upcase)
+  end
+
+  def test_downcase2
+    assert_equal("\u3042ab", "\u3042aB".downcase)
+  end
+
+  def test_rstrip
+    assert_equal("\u3042", "\u3042   ".rstrip)
+    assert_raise(Encoding::CompatibilityError) { "\u3042".encode("ISO-2022-JP").rstrip }
+  end
+
+  def test_symbol_table_overflow
+    return
+    assert_in_out_err([], <<-INPUT, [], /symbol table overflow \(symbol [a-z]{8}\) \(RuntimeError\)/)
+      ("aaaaaaaa".."zzzzzzzz").each {|s| s.to_sym }
+    INPUT
+  end
+
+  def test_shared_force_encoding
+    s = "\u{3066}\u{3059}\u{3068}".gsub(//, '')
+    h = {}
+    h[s] = nil
+    k = h.keys[0]
+    assert_equal(s, k, '[ruby-dev:39068]')
+    assert_equal(Encoding::UTF_8, k.encoding, '[ruby-dev:39068]')
+    s.dup.force_encoding(Encoding::ASCII_8BIT).gsub(//, '')
+    k = h.keys[0]
+    assert_equal(s, k, '[ruby-dev:39068]')
+    assert_equal(Encoding::UTF_8, k.encoding, '[ruby-dev:39068]')
+  end
+
+  def test_ascii_incomat_inspect
+    bug4081 = '[ruby-core:33283]'
+    [Encoding::UTF_16LE, Encoding::UTF_16BE,
+     Encoding::UTF_32LE, Encoding::UTF_32BE].each do |e|
+      assert_equal('"abc"', "abc".encode(e).inspect)
+      assert_equal('"\\u3042\\u3044\\u3046"', "\u3042\u3044\u3046".encode(e).inspect)
+      assert_equal('"ab\\"c"', "ab\"c".encode(e).inspect, bug4081)
+    end
+    begin
+      ext = Encoding.default_external
+      Encoding.default_external = "us-ascii"
+      i = "abc\"\\".force_encoding("utf-8").inspect
+    ensure
+      Encoding.default_external = ext
+    end
+    assert_equal('"abc\\"\\\\"', i, bug4081)
+  end
 end
