@@ -1,6 +1,8 @@
 require 'test/unit'
 require 'rational'
+require 'delegate'
 require 'timeout'
+require 'delegate'
 
 class TestTime < Test::Unit::TestCase
   def setup
@@ -12,12 +14,33 @@ class TestTime < Test::Unit::TestCase
     $VERBOSE = @verbose
   end
 
+  def test_new
+    assert_equal(Time.utc(2000,2,10), Time.new(2000,2,10, 11,0,0, 3600*11))
+    assert_equal(Time.utc(2000,2,10), Time.new(2000,2,9, 13,0,0, -3600*11))
+    assert_equal(Time.utc(2000,2,10), Time.new(2000,2,10, 11,0,0, "+11:00"))
+    assert_equal(Rational(1,2), Time.new(2000,2,10, 11,0,5.5, "+11:00").subsec)
+    bug4090 = '[ruby-dev:42631]'
+    tm = [2001,2,28,23,59,30]
+    t = Time.new(*tm, "-12:00")
+    assert_equal([2001,2,28,23,59,30,-43200], [t.year, t.month, t.mday, t.hour, t.min, t.sec, t.gmt_offset], bug4090)
+  end
+
   def test_time_add()
     assert_equal(Time.utc(2000, 3, 21, 3, 30) + 3 * 3600,
                  Time.utc(2000, 3, 21, 6, 30))
     assert_equal(Time.utc(2000, 3, 21, 3, 30) + (-3 * 3600),
                  Time.utc(2000, 3, 21, 0, 30))
     assert_equal(0, (Time.at(1.1) + 0.9).usec)
+
+    assert((Time.utc(2000, 4, 1) + 24).utc?)
+    assert(!(Time.local(2000, 4, 1) + 24).utc?)
+
+    t = Time.new(2000, 4, 1, 0, 0, 0, "+01:00") + 24
+    assert(!t.utc?)
+    assert_equal(3600, t.utc_offset)
+    t = Time.new(2000, 4, 1, 0, 0, 0, "+02:00") + 24
+    assert(!t.utc?)
+    assert_equal(7200, t.utc_offset)
   end
 
   def test_time_subt()
@@ -100,33 +123,88 @@ class TestTime < Test::Unit::TestCase
   end
 
   def test_at
+    assert_equal(100000, Time.at("0.1".to_r).usec)
+    assert_equal(10000, Time.at("0.01".to_r).usec)
+    assert_equal(1000, Time.at("0.001".to_r).usec)
+    assert_equal(100, Time.at("0.0001".to_r).usec)
+    assert_equal(10, Time.at("0.00001".to_r).usec)
+    assert_equal(1, Time.at("0.000001".to_r).usec)
+    assert_equal(100000000, Time.at("0.1".to_r).nsec)
+    assert_equal(10000000, Time.at("0.01".to_r).nsec)
+    assert_equal(1000000, Time.at("0.001".to_r).nsec)
+    assert_equal(100000, Time.at("0.0001".to_r).nsec)
+    assert_equal(10000, Time.at("0.00001".to_r).nsec)
+    assert_equal(1000, Time.at("0.000001".to_r).nsec)
+    assert_equal(100, Time.at("0.0000001".to_r).nsec)
+    assert_equal(10, Time.at("0.00000001".to_r).nsec)
+    assert_equal(1, Time.at("0.000000001".to_r).nsec)
     assert_equal(100000, Time.at(0.1).usec)
     assert_equal(10000, Time.at(0.01).usec)
     assert_equal(1000, Time.at(0.001).usec)
     assert_equal(100, Time.at(0.0001).usec)
     assert_equal(10, Time.at(0.00001).usec)
-    assert_equal(1, Time.at(0.000001).usec)
+    assert_equal(3, Time.at(0.000003).usec)
     assert_equal(100000000, Time.at(0.1).nsec)
     assert_equal(10000000, Time.at(0.01).nsec)
     assert_equal(1000000, Time.at(0.001).nsec)
     assert_equal(100000, Time.at(0.0001).nsec)
     assert_equal(10000, Time.at(0.00001).nsec)
-    assert_equal(1000, Time.at(0.000001).nsec)
-    assert_equal(100, Time.at(0.0000001).nsec)
+    assert_equal(3000, Time.at(0.000003).nsec)
+    assert_equal(199, Time.at(0.0000002).nsec)
     assert_equal(10, Time.at(0.00000001).nsec)
     assert_equal(1, Time.at(0.000000001).nsec)
+
     assert_equal(0, Time.at(1e-10).nsec)
     assert_equal(0, Time.at(4e-10).nsec)
-    assert_equal(1, Time.at(6e-10).nsec)
+    assert_equal(0, Time.at(6e-10).nsec)
     assert_equal(1, Time.at(14e-10).nsec)
-    assert_equal(2, Time.at(16e-10).nsec)
+    assert_equal(1, Time.at(16e-10).nsec)
     if negative_time_t?
-      assert_equal(0, Time.at(-1e-10).nsec)
-      assert_equal(0, Time.at(-4e-10).nsec)
+      assert_equal(999999999, Time.at(-1e-10).nsec)
+      assert_equal(999999999, Time.at(-4e-10).nsec)
       assert_equal(999999999, Time.at(-6e-10).nsec)
-      assert_equal(999999999, Time.at(-14e-10).nsec)
+      assert_equal(999999998, Time.at(-14e-10).nsec)
       assert_equal(999999998, Time.at(-16e-10).nsec)
     end
+
+    t = Time.at(-4611686019).utc
+    assert_equal(1823, t.year)
+
+    t = Time.at(4611686018, 999999).utc
+    assert_equal(2116, t.year)
+    assert_equal("0.999999".to_r, t.subsec)
+
+    t = Time.at(2**40 + "1/3".to_r, 9999999999999).utc
+    assert_equal(36812, t.year)
+    
+    t = Time.at(-0x3fff_ffff_ffff_ffff)
+    assert_equal(-146138510344, t.year)
+    t = Time.at(-0x4000_0000_0000_0000)
+    assert_equal(-146138510344, t.year)
+    t = Time.at(-0x4000_0000_0000_0001)
+    assert_equal(-146138510344, t.year)
+    t = Time.at(-0x5000_0000_0000_0001)
+    assert_equal(-182673138422, t.year)
+    t = Time.at(-0x6000_0000_0000_0000)
+    assert_equal(-219207766501, t.year)
+
+    t = Time.at(0).utc
+    assert_equal([1970,1,1, 0,0,0], [t.year, t.mon, t.day, t.hour, t.min, t.sec])
+    t = Time.at(-86400).utc
+    assert_equal([1969,12,31, 0,0,0], [t.year, t.mon, t.day, t.hour, t.min, t.sec])
+    t = Time.at(-86400 * (400 * 365 + 97)).utc
+    assert_equal([1970-400,1,1, 0,0,0], [t.year, t.mon, t.day, t.hour, t.min, t.sec])
+    t = Time.at(-86400 * (400 * 365 + 97)*1000).utc
+    assert_equal([1970-400*1000,1,1, 0,0,0], [t.year, t.mon, t.day, t.hour, t.min, t.sec])
+    t = Time.at(-86400 * (400 * 365 + 97)*2421).utc
+    assert_equal([1970-400*2421,1,1, 0,0,0], [t.year, t.mon, t.day, t.hour, t.min, t.sec])
+    t = Time.at(-86400 * (400 * 365 + 97)*1000000).utc
+    assert_equal([1970-400*1000000,1,1, 0,0,0], [t.year, t.mon, t.day, t.hour, t.min, t.sec])
+
+    t = Time.at(-30613683110400).utc
+    assert_equal([-968139,1,1, 0,0,0], [t.year, t.mon, t.day, t.hour, t.min, t.sec])
+    t = Time.at(-30613683110401).utc
+    assert_equal([-968140,12,31, 23,59,59], [t.year, t.mon, t.day, t.hour, t.min, t.sec])
   end
 
   def test_at2
@@ -141,7 +219,7 @@ class TestTime < Test::Unit::TestCase
   end
 
   def test_utc_subsecond
-    assert_equal(100000, Time.utc(2007,1,1,0,0,1.1).usec)
+    assert_equal(500000, Time.utc(2007,1,1,0,0,1.5).usec)
     assert_equal(100000, Time.utc(2007,1,1,0,0,Rational(11,10)).usec)
   end
 
@@ -166,6 +244,18 @@ class TestTime < Test::Unit::TestCase
     assert_marshal_roundtrip(Time.at(0, 0.120))
   end
 
+  def test_marshal_nsec_191
+    # generated by ruby 1.9.1p376
+    m = "\x04\bIu:\tTime\r \x80\x11\x80@\xE2\x01\x00\x06:\rsubmicro\"\ax\x90"
+    t = Marshal.load(m)
+    assert_equal(Time.at(Rational(123456789, 1000000000)), t, "[ruby-dev:40133]")
+  end
+
+  def test_marshal_rational
+    assert_marshal_roundtrip(Time.at(0, Rational(1,3)))
+    assert_not_match(/Rational/, Marshal.dump(Time.at(0, Rational(1,3))))
+  end
+
   def test_marshal_ivar
     t = Time.at(123456789, 987654.321)
     t.instance_eval { @var = 135 }
@@ -173,6 +263,20 @@ class TestTime < Test::Unit::TestCase
     assert_marshal_roundtrip(Marshal.load(Marshal.dump(t)))
   end
 
+  def test_marshal_timezone
+    bug = '[ruby-dev:40063]'
+
+    t1 = Time.gm(2000)
+    m = Marshal.dump(t1.getlocal("-02:00"))
+    t2 = Marshal.load(m)
+    assert_equal(t1, t2)
+    assert_equal(-7200, t2.utc_offset, bug)
+    m = Marshal.dump(t1.getlocal("+08:15"))
+    t2 = Marshal.load(m)
+    assert_equal(t1, t2)
+    assert_equal(29700, t2.utc_offset, bug)
+  end
+
   # Sat Jan 01 00:00:00 UTC 2000
   T2000 = Time.at(946684800).gmtime
 
@@ -188,21 +292,19 @@ class TestTime < Test::Unit::TestCase
 
   def test_at3
     assert_equal(T2000, Time.at(T2000))
-    assert_raise(RangeError) do
-      Time.at(2**31-1, 1_000_000)
-      Time.at(2**63-1, 1_000_000)
-    end
-    assert_raise(RangeError) do
-      Time.at(-2**31, -1_000_000)
-      Time.at(-2**63, -1_000_000)
-    end
+#    assert_raise(RangeError) do
+#      Time.at(2**31-1, 1_000_000)
+#      Time.at(2**63-1, 1_000_000)
+#    end
+#    assert_raise(RangeError) do
+#      Time.at(-2**31, -1_000_000)
+#      Time.at(-2**63, -1_000_000)
+#    end
   end
 
   def test_utc_or_local
     assert_equal(T2000, Time.gm(2000))
     assert_equal(T2000, Time.gm(0, 0, 0, 1, 1, 2000, :foo, :bar, false, :baz))
-    assert_equal(T2000, Time.gm(0))
-    assert_equal(T2000, Time.gm(100))
     assert_equal(T2000, Time.gm(2000, "jan"))
     assert_equal(T2000, Time.gm(2000, "1"))
     assert_equal(T2000, Time.gm(2000, 1, 1, 0, 0, 0, 0))
@@ -215,14 +317,22 @@ class TestTime < Test::Unit::TestCase
     end
     assert_raise(ArgumentError) { Time.gm(2000, 1, 1, 0, 0, -(2**31), :foo, :foo) }
     o = Object.new
-    def o.divmod(x); nil; end
+    def o.to_r; nil; end
     assert_raise(TypeError) { Time.gm(2000, 1, 1, 0, 0, o, :foo, :foo) }
-    def o.divmod(x); [-1, 0]; end
-    assert_raise(ArgumentError) { Time.gm(2000, 1, 1, 0, 0, o, :foo, :foo) }
+    def o.to_r; ""; end
+    assert_raise(TypeError) { Time.gm(2000, 1, 1, 0, 0, o, :foo, :foo) }
+    def o.to_r; Rational(11); end
+    assert_equal(11, Time.gm(2000, 1, 1, 0, 0, o).sec)
+    o = Object.new
+    def o.to_int; 10; end
+    assert_equal(10, Time.gm(2000, 1, 1, 0, 0, o).sec)
     assert_raise(ArgumentError) { Time.gm(2000, 13) }
 
     t = Time.local(2000)
     assert_equal(t.gmt_offset, T2000 - t)
+
+    assert_equal(-4427700000, Time.utc(-4427700000,12,1).year)
+    assert_equal(-2**30+10, Time.utc(-2**30+10,1,1).year)
   end
 
   def test_time_interval
@@ -284,16 +394,24 @@ class TestTime < Test::Unit::TestCase
     t1 = Time.gm(2000)
     t2 = t1.getlocal
     assert_equal(t1, t2)
+    t3 = t1.getlocal("-02:00")
+    assert_equal(t1, t3)
+    assert_equal(-7200, t3.utc_offset)
     t1.localtime
     assert_equal(t1, t2)
     assert_equal(t1.gmt?, t2.gmt?)
+    assert_equal(t1, t3)
 
     t1 = Time.local(2000)
     t2 = t1.getgm
     assert_equal(t1, t2)
+    t3 = t1.getlocal("-02:00")
+    assert_equal(t1, t3)
+    assert_equal(-7200, t3.utc_offset)
     t1.gmtime
     assert_equal(t1, t2)
     assert_equal(t1.gmt?, t2.gmt?)
+    assert_equal(t1, t3)
   end
 
   def test_asctime
@@ -315,6 +433,31 @@ class TestTime < Test::Unit::TestCase
     assert_equal(T2000 + 1, T2000.succ)
   end
 
+  def test_plus_type
+    t0 = Time.utc(2000,1,1)
+    n0 = t0.to_i
+    n1 = n0+1
+    t1 = Time.at(n1)
+    assert_equal(t1, t0 + 1)
+    assert_equal(t1, t0 + 1.0)
+    assert_equal(t1, t0 + Rational(1,1))
+    assert_equal(t1, t0 + SimpleDelegator.new(1))
+    assert_equal(t1, t0 + SimpleDelegator.new(1.0))
+    assert_equal(t1, t0 + SimpleDelegator.new(Rational(1,1)))
+    assert_raise(TypeError) { t0 + nil }
+    assert_raise(TypeError) { t0 + "1" }
+    assert_raise(TypeError) { t0 + SimpleDelegator.new("1") }
+    assert_equal(0.5, (t0 + 1.5).subsec)
+    assert_equal(Rational(1,3), (t0 + Rational(4,3)).subsec)
+    assert_equal(0.5, (t0 + SimpleDelegator.new(1.5)).subsec)
+    assert_equal(Rational(1,3), (t0 + SimpleDelegator.new(Rational(4,3))).subsec)
+  end
+
+  def test_minus
+    t = Time.at(-4611686018).utc - 100
+    assert_equal(1823, t.year)
+  end
+
   def test_readers
     assert_equal(0, T2000.sec)
     assert_equal(0, T2000.min)
@@ -326,6 +469,7 @@ class TestTime < Test::Unit::TestCase
     assert_equal(1, T2000.yday)
     assert_equal(false, T2000.isdst)
     assert_equal("UTC", T2000.zone)
+    assert_equal(Encoding.find("locale"), T2000.zone.encoding)
     assert_equal(0, T2000.gmt_offset)
     assert(!T2000.sunday?)
     assert(!T2000.monday?)
@@ -347,6 +491,7 @@ class TestTime < Test::Unit::TestCase
     assert_equal(t.yday, Time.at(946684800).yday)
     assert_equal(t.isdst, Time.at(946684800).isdst)
     assert_equal(t.zone, Time.at(946684800).zone)
+    assert_equal(Encoding.find("locale"), Time.at(946684800).zone.encoding)
     assert_equal(t.gmt_offset, Time.at(946684800).gmt_offset)
     assert_equal(t.sunday?, Time.at(946684800).sunday?)
     assert_equal(t.monday?, Time.at(946684800).monday?)
@@ -396,7 +541,7 @@ class TestTime < Test::Unit::TestCase
     assert_equal("123456", t.strftime("%6N"))
     assert_equal("123456789", t.strftime("%9N"))
     assert_equal("1234567890", t.strftime("%10N"))
-    assert_equal("", t.strftime("%0N"))
+    assert_equal("123456789", t.strftime("%0N"))
     assert_equal("000", t.strftime("%3S"))
     assert_equal("946684800", t.strftime("%s"))
     assert_equal("946684800", t.utc.strftime("%s"))
@@ -444,5 +589,100 @@ class TestTime < Test::Unit::TestCase
     assert_equal(" 2", t.strftime("%l"))
     assert_equal("02", t.strftime("%0l"))
     assert_equal(" 2", t.strftime("%_l"))
+
+    # [ruby-dev:37155]
+    t = Time.mktime(1970, 1, 18)
+    assert_equal("0", t.strftime("%w"))
+    assert_equal("7", t.strftime("%u"))
+
+    # [ruby-dev:37160]
+    assert_equal("\t", T2000.strftime("%t"))
+    assert_equal("\t", T2000.strftime("%0t"))
+    assert_equal("\t", T2000.strftime("%1t"))
+    assert_equal("  \t", T2000.strftime("%3t"))
+    assert_equal("00\t", T2000.strftime("%03t"))
+    assert_equal("\n", T2000.strftime("%n"))
+    assert_equal("\n", T2000.strftime("%0n"))
+    assert_equal("\n", T2000.strftime("%1n"))
+    assert_equal("  \n", T2000.strftime("%3n"))
+    assert_equal("00\n", T2000.strftime("%03n"))
+
+    # [ruby-dev:37162]
+    assert_equal("SAT", T2000.strftime("%#a"))
+    assert_equal("SATURDAY", T2000.strftime("%#A"))
+    assert_equal("JAN", T2000.strftime("%#b"))
+    assert_equal("JANUARY", T2000.strftime("%#B"))
+    assert_equal("JAN", T2000.strftime("%#h"))
+    assert_equal("FRIDAY", Time.local(2008,1,4).strftime("%#A"))
+
+    t = Time.utc(2000,3,14, 6,53,"58.979323846".to_r) # Pi Day
+    assert_equal("03/14/2000  6:53:58.97932384600000000000000000000",
+                 t.strftime("%m/%d/%Y %l:%M:%S.%29N"))
+    assert_equal("03/14/2000  6:53:58.9793238460",
+                 t.strftime("%m/%d/%Y %l:%M:%S.%10N"))
+    assert_equal("03/14/2000  6:53:58.979323846",
+                 t.strftime("%m/%d/%Y %l:%M:%S.%9N"))
+    assert_equal("03/14/2000  6:53:58.97932384",
+                 t.strftime("%m/%d/%Y %l:%M:%S.%8N"))
+
+    t = Time.utc(1592,3,14, 6,53,"58.97932384626433832795028841971".to_r) # Pi Day
+    assert_equal("03/14/1592  6:53:58.97932384626433832795028841971",
+                 t.strftime("%m/%d/%Y %l:%M:%S.%29N"))
+    assert_equal("03/14/1592  6:53:58.9793238462",
+                 t.strftime("%m/%d/%Y %l:%M:%S.%10N"))
+    assert_equal("03/14/1592  6:53:58.979323846",
+                 t.strftime("%m/%d/%Y %l:%M:%S.%9N"))
+    assert_equal("03/14/1592  6:53:58.97932384",
+                 t.strftime("%m/%d/%Y %l:%M:%S.%8N"))
+
+    # [ruby-core:33985]
+    assert_equal("3000000000", Time.at(3000000000).strftime('%s'))
+  end
+
+  def test_delegate
+    d1 = SimpleDelegator.new(t1 = Time.utc(2000))
+    d2 = SimpleDelegator.new(t2 = Time.utc(2001))
+    assert_equal(-1, t1 <=> t2)
+    assert_equal(1, t2 <=> t1)
+    assert_equal(-1, d1 <=> d2)
+    assert_equal(1, d2 <=> d1)
+  end
+
+  def test_to_r
+    assert_kind_of(Rational, Time.new(2000,1,1,0,0,Rational(4,3)).to_r)
+    assert_kind_of(Rational, Time.utc(1970).to_r)
+  end
+
+  def test_round
+    t = Time.utc(1999,12,31, 23,59,59)
+    t2 = (t+0.4).round
+    assert_equal([59,59,23, 31,12,1999, 5,365,false,"UTC"], t2.to_a)
+    assert_equal(0, t2.subsec)
+    t2 = (t+0.49).round
+    assert_equal([59,59,23, 31,12,1999, 5,365,false,"UTC"], t2.to_a)
+    assert_equal(0, t2.subsec)
+    t2 = (t+0.5).round
+    assert_equal([0,0,0, 1,1,2000, 6,1,false,"UTC"], t2.to_a)
+    assert_equal(0, t2.subsec)
+    t2 = (t+1.4).round
+    assert_equal([0,0,0, 1,1,2000, 6,1,false,"UTC"], t2.to_a)
+    assert_equal(0, t2.subsec)
+    t2 = (t+1.49).round
+    assert_equal([0,0,0, 1,1,2000, 6,1,false,"UTC"], t2.to_a)
+    assert_equal(0, t2.subsec)
+    t2 = (t+1.5).round
+    assert_equal([1,0,0, 1,1,2000, 6,1,false,"UTC"], t2.to_a)
+    assert_equal(0, t2.subsec)
+
+    t2 = (t+0.123456789).round(4)
+    assert_equal([59,59,23, 31,12,1999, 5,365,false,"UTC"], t2.to_a)
+    assert_equal(Rational(1235,10000), t2.subsec)
+
+    off = 0.0
+    100.times {|i|
+      t2 = (t+off).round(1)
+      assert_equal(Rational(i % 10, 10), t2.subsec)
+      off += 0.1
+    }
   end
 end
