@@ -1,4 +1,4 @@
-require File.join(File.expand_path(File.dirname(__FILE__)), 'gemutilities')
+require_relative 'gemutilities'
 require 'rubygems/commands/environment_command'
 
 class TestGemCommandsEnvironmentCommand < RubyGemTestCase
@@ -12,6 +12,7 @@ class TestGemCommandsEnvironmentCommand < RubyGemTestCase
   def test_execute
     orig_sources = Gem.sources.dup
     Gem.sources.replace %w[http://gems.example.com]
+    Gem.configuration['gemcutter_key'] = 'blah'
 
     @cmd.send :handle_options, %w[]
 
@@ -24,7 +25,7 @@ class TestGemCommandsEnvironmentCommand < RubyGemTestCase
     assert_match %r|INSTALLATION DIRECTORY: #{Regexp.escape @gemhome}|,
                  @ui.output
     assert_match %r|RUBYGEMS PREFIX: |, @ui.output
-    assert_match %r|RUBY EXECUTABLE:.*#{Gem::ConfigMap[:RUBY_INSTALL_NAME]}|,
+    assert_match %r|RUBY EXECUTABLE:.*#{Gem::ConfigMap[:ruby_install_name]}|,
                  @ui.output
     assert_match %r|EXECUTABLE DIRECTORY:|, @ui.output
     assert_match %r|RUBYGEMS PLATFORMS:|, @ui.output
@@ -32,6 +33,7 @@ class TestGemCommandsEnvironmentCommand < RubyGemTestCase
     assert_match %r|GEM PATHS:|, @ui.output
     assert_match %r|- #{Regexp.escape @gemhome}|, @ui.output
     assert_match %r|GEM CONFIGURATION:|, @ui.output
+    assert_match %r|"gemcutter_key" => "\*\*\*\*"|, @ui.output
     assert_match %r|:verbose => |, @ui.output
     assert_match %r|REMOTE SOURCES:|, @ui.output
     assert_equal '', @ui.error
@@ -125,7 +127,7 @@ class TestGemCommandsEnvironmentCommand < RubyGemTestCase
       @cmd.execute
     end
 
-    assert_equal "#{Gem::RubyGemsVersion}\n", @ui.output
+    assert_equal "#{Gem::VERSION}\n", @ui.output
     assert_equal '', @ui.error
   end
 
