@@ -3,7 +3,7 @@
 
   thread_pthread.c -
 
-  $Author: nobu $
+  $Author: yugui $
 
   Copyright (C) 2004-2007 Koichi Sasada
 
@@ -34,7 +34,7 @@ native_mutex_lock(pthread_mutex_t *lock)
 {
     int r;
     if ((r = pthread_mutex_lock(lock)) != 0) {
-	rb_bug("pthread_mutex_lock: %d", r);
+	rb_bug_errno("pthread_mutex_lock", r);
     }
 }
 
@@ -43,7 +43,7 @@ native_mutex_unlock(pthread_mutex_t *lock)
 {
     int r;
     if ((r = pthread_mutex_unlock(lock)) != 0) {
-	rb_bug("native_mutex_unlock return non-zero: %d", r);
+	rb_bug_errno("pthread_mutex_unlock", r);
     }
 }
 
@@ -56,7 +56,7 @@ native_mutex_trylock(pthread_mutex_t *lock)
 	    return EBUSY;
 	}
 	else {
-	    rb_bug("native_mutex_trylock return non-zero: %d", r);
+	    rb_bug_errno("pthread_mutex_trylock", r);
 	}
     }
     return 0;
@@ -67,16 +67,21 @@ native_mutex_initialize(pthread_mutex_t *lock)
 {
     int r = pthread_mutex_init(lock, 0);
     if (r != 0) {
-	rb_bug("native_mutex_initialize return non-zero: %d", r);
+	rb_bug_errno("pthread_mutex_init", r);
     }
 }
 
+#define native_mutex_reinitialize_atfork(lock) (\
+	native_mutex_unlock(lock), \
+	native_mutex_initialize(lock), \
+	native_mutex_lock(lock))
+
 static void
 native_mutex_destroy(pthread_mutex_t *lock)
 {
     int r = pthread_mutex_destroy(lock);
     if (r != 0) {
-	rb_bug("native_mutex_destroy return non-zero: %d", r);
+	rb_bug_errno("pthread_mutex_destroy", r);
     }
 }
 
@@ -85,7 +90,7 @@ native_cond_initialize(pthread_cond_t *cond)
 {
     int r = pthread_cond_init(cond, 0);
     if (r != 0) {
-	rb_bug("native_cond_initialize return non-zero: %d", r);
+	rb_bug_errno("pthread_cond_init", r);
     }
 }
 
@@ -94,32 +99,67 @@ native_cond_destroy(pthread_cond_t *cond)
 {
     int r = pthread_cond_destroy(cond);
     if (r != 0) {
-	rb_bug("native_cond_destroy return non-zero: %d", r);
+	rb_bug_errno("pthread_cond_destroy", r);
     }
 }
 
 static void
 native_cond_signal(pthread_cond_t *cond)
 {
-    pthread_cond_signal(cond);
+    int r = pthread_cond_signal(cond);
+    if (r != 0) {
+	rb_bug_errno("pthread_cond_signal", r);
+    }
 }
 
 static void
 native_cond_broadcast(pthread_cond_t *cond)
 {
-    pthread_cond_broadcast(cond);
+    int r = pthread_cond_broadcast(cond);
+    if (r != 0) {
+	rb_bug_errno("native_cond_broadcast", r);
+    }
 }
 
 static void
 native_cond_wait(pthread_cond_t *cond, pthread_mutex_t *mutex)
 {
-    pthread_cond_wait(cond, mutex);
+    int r = pthread_cond_wait(cond, mutex);
+    if (r != 0) {
+	rb_bug_errno("pthread_cond_wait", r);
+    }
+}
+
+static int
+native_cond_timedwait(pthread_cond_t *cond, pthread_mutex_t *mutex, struct timespec *ts)
+{
+    int r;
+
+    /*
+     * An old Linux may return EINTR. Even though POSIX says
+     *   "These functions shall not return an error code of [EINTR]".
+     *   http://pubs.opengroup.org/onlinepubs/009695399/functions/pthread_cond_timedwait.html
+     * Let's hide it from arch generic code.
+     */
+    do {
+        r = pthread_cond_timedwait(cond, mutex, ts);
+    } while (r == EINTR);
+
+    if (r != 0 && r != ETIMEDOUT) {
+	rb_bug_errno("pthread_cond_timedwait", r);
+    }
+
+    return r;
 }
 
 
 #define native_cleanup_push pthread_cleanup_push
 #define native_cleanup_pop  pthread_cleanup_pop
-#define native_thread_yield() sched_yield()
+#ifdef HAVE_SCHED_YIELD
+#define native_thread_yield() (void)sched_yield()
+#else
+#define native_thread_yield() ((void)0)
+#endif
 
 #ifndef __CYGWIN__
 static void add_signal_thread_list(rb_thread_t *th);
@@ -148,7 +188,7 @@ ruby_thread_set_native(rb_thread_t *th)
     return pthread_setspecific(ruby_native_thread_key, th) == 0;
 }
 
-static void
+void
 Init_native_thread(void)
 {
     rb_thread_t *th = GET_THREAD();
@@ -170,6 +210,85 @@ native_thread_destroy(rb_thread_t *th)
 
 #define USE_THREAD_CACHE 0
 
+#if STACK_GROW_DIRECTION
+#define STACK_GROW_DIR_DETECTION
+#define STACK_DIR_UPPER(a,b) STACK_UPPER(0, a, b)
+#else
+#define STACK_GROW_DIR_DETECTION VALUE stack_grow_dir_detection
+#define STACK_DIR_UPPER(a,b) STACK_UPPER(&stack_grow_dir_detection, a, b)
+#endif
+
+#if defined HAVE_PTHREAD_GETATTR_NP || defined HAVE_PTHREAD_ATTR_GET_NP
+#define STACKADDR_AVAILABLE 1
+#elif defined HAVE_PTHREAD_GET_STACKADDR_NP && defined HAVE_PTHREAD_GET_STACKSIZE_NP
+#define STACKADDR_AVAILABLE 1
+#elif defined HAVE_THR_STKSEGMENT || defined HAVE_PTHREAD_STACKSEG_NP
+#define STACKADDR_AVAILABLE 1
+#elif defined HAVE_PTHREAD_GETTHRDS_NP
+#define STACKADDR_AVAILABLE 1
+#endif
+
+#ifdef STACKADDR_AVAILABLE
+static int
+get_stack(void **addr, size_t *size)
+{
+#define CHECK_ERR(expr)				\
+    {int err = (expr); if (err) return err;}
+#if defined HAVE_PTHREAD_GETATTR_NP || defined HAVE_PTHREAD_ATTR_GET_NP
+    pthread_attr_t attr;
+    size_t guard = 0;
+
+# ifdef HAVE_PTHREAD_GETATTR_NP
+    CHECK_ERR(pthread_getattr_np(pthread_self(), &attr));
+#   ifdef HAVE_PTHREAD_ATTR_GETSTACK
+    CHECK_ERR(pthread_attr_getstack(&attr, addr, size));
+#   else
+    CHECK_ERR(pthread_attr_getstackaddr(&attr, addr));
+    CHECK_ERR(pthread_attr_getstacksize(&attr, size));
+#   endif
+    if (pthread_attr_getguardsize(&attr, &guard) == 0) {
+	STACK_GROW_DIR_DETECTION;
+	STACK_DIR_UPPER((void)0, (void)(*addr = (char *)*addr + guard));
+	*size -= guard;
+    }
+# else
+    CHECK_ERR(pthread_attr_init(&attr));
+    CHECK_ERR(pthread_attr_get_np(pthread_self(), &attr));
+    CHECK_ERR(pthread_attr_getstackaddr(&attr, addr));
+    CHECK_ERR(pthread_attr_getstacksize(&attr, size));
+# endif
+    CHECK_ERR(pthread_attr_getguardsize(&attr, &guard));
+    *size -= guard;
+    pthread_attr_destroy(&attr);
+#elif defined HAVE_PTHREAD_GET_STACKADDR_NP && defined HAVE_PTHREAD_GET_STACKSIZE_NP
+    pthread_t th = pthread_self();
+    *addr = pthread_get_stackaddr_np(th);
+    *size = pthread_get_stacksize_np(th);
+#elif defined HAVE_THR_STKSEGMENT || defined HAVE_PTHREAD_STACKSEG_NP
+    stack_t stk;
+# if defined HAVE_THR_STKSEGMENT
+    CHECK_ERR(thr_stksegment(&stk));
+# else
+    CHECK_ERR(pthread_stackseg_np(pthread_self(), &stk));
+# endif
+    *addr = stk.ss_sp;
+    *size = stk.ss_size;
+#elif defined HAVE_PTHREAD_GETTHRDS_NP
+    pthread_t th = pthread_self();
+    struct __pthrdsinfo thinfo;
+    char reg[256];
+    int regsiz=sizeof(reg);
+    CHECK_ERR(pthread_getthrds_np(&th, PTHRDSINFO_QUERY_ALL,
+				  &thinfo, sizeof(thinfo),
+				  &reg, &regsiz));
+    *addr = thinfo.__pi_stackaddr;
+    *size = thinfo.__pi_stacksize;
+#endif
+    return 0;
+#undef CHECK_ERR
+}
+#endif
+
 static struct {
     rb_thread_id_t id;
     size_t stack_maxsize;
@@ -185,7 +304,7 @@ extern void *STACK_END_ADDRESS;
 
 #undef ruby_init_stack
 void
-ruby_init_stack(VALUE *addr
+ruby_init_stack(volatile VALUE *addr
 #ifdef __ia64
     , void *bsp
 #endif
@@ -196,10 +315,10 @@ ruby_init_stack(VALUE *addr
     native_main_thread.stack_start = STACK_END_ADDRESS;
 #else
     if (!native_main_thread.stack_start ||
-        STACK_UPPER(&addr,
+        STACK_UPPER((VALUE *)(void *)&addr,
                     native_main_thread.stack_start > addr,
                     native_main_thread.stack_start < addr)) {
-        native_main_thread.stack_start = addr;
+        native_main_thread.stack_start = (VALUE *)addr;
     }
 #endif
 #ifdef __ia64
@@ -208,22 +327,25 @@ ruby_init_stack(VALUE *addr
         native_main_thread.register_stack_start = (VALUE*)bsp;
     }
 #endif
-#ifdef HAVE_GETRLIMIT
     {
+	size_t size = 0;
+	size_t space = 0;
+#if defined(HAVE_PTHREAD_ATTR_GET_NP)
+	void* addr;
+	get_stack(&addr, &size);
+#elif defined(HAVE_GETRLIMIT)
 	struct rlimit rlim;
-
 	if (getrlimit(RLIMIT_STACK, &rlim) == 0) {
-	    unsigned int space = rlim.rlim_cur/5;
-
-	    if (space > 1024*1024) space = 1024*1024;
-	    native_main_thread.stack_maxsize = rlim.rlim_cur - space;
+	    size = (size_t)rlim.rlim_cur;
 	}
-    }
 #endif
+	space = size > 5 * 1024 * 1024 ? 1024 * 1024 : size / 5;
+	native_main_thread.stack_maxsize = size - space;
+    }
 }
 
 #define CHECK_ERR(expr) \
-    {int err = (expr); if (err) {rb_bug("err: %d - %s", err, #expr);}}
+    {int err = (expr); if (err) {rb_bug_errno(#expr, err);}}
 
 static int
 native_thread_init_stack(rb_thread_t *th)
@@ -235,17 +357,14 @@ native_thread_init_stack(rb_thread_t *th)
 	th->machine_stack_maxsize = native_main_thread.stack_maxsize;
     }
     else {
-#ifdef HAVE_PTHREAD_GETATTR_NP
-	pthread_attr_t attr;
+#ifdef STACKADDR_AVAILABLE
 	void *start;
-	CHECK_ERR(pthread_getattr_np(curr, &attr));
-# if defined HAVE_PTHREAD_ATTR_GETSTACK
-	CHECK_ERR(pthread_attr_getstack(&attr, &start, &th->machine_stack_maxsize));
-# elif defined HAVE_PTHREAD_ATTR_GETSTACKSIZE && defined HAVE_PTHREAD_ATTR_GETSTACKADDR
-	CHECK_ERR(pthread_attr_getstackaddr(&attr, &start));
-	CHECK_ERR(pthread_attr_getstacksize(&attr, &th->machine_stack_maxsize));
-# endif
-	th->machine_stack_start = start;
+	size_t size;
+
+	if (get_stack(&start, &size) == 0) {
+	    th->machine_stack_start = start;
+	    th->machine_stack_maxsize = size;
+	}
 #else
 	rb_raise(rb_eNotImpError, "ruby engine can initialize only in the main thread");
 #endif
@@ -268,6 +387,9 @@ thread_start_func_1(void *th_ptr)
 	rb_thread_t *th = th_ptr;
 	VALUE stack_start;
 
+#ifndef __CYGWIN__
+	native_thread_init_stack(th);
+#endif
 	/* run */
 	thread_start_func_2(th, &stack_start, rb_ia64_bsp());
     }
@@ -376,26 +498,37 @@ use_cached_thread(rb_thread_t *th)
     return result;
 }
 
+enum {
+#ifdef __SYMBIAN32__
+    RUBY_STACK_MIN_LIMIT = 64 * 1024,  /* 64KB: Let's be slightly more frugal on mobile platform */
+#else
+    RUBY_STACK_MIN_LIMIT = 512 * 1024, /* 512KB */
+#endif
+    RUBY_STACK_SPACE_LIMIT = 1024 * 1024
+};
+
+#ifdef PTHREAD_STACK_MIN
+#define RUBY_STACK_MIN ((RUBY_STACK_MIN_LIMIT < PTHREAD_STACK_MIN) ? \
+			PTHREAD_STACK_MIN * 2 : RUBY_STACK_MIN_LIMIT)
+#else
+#define RUBY_STACK_MIN (RUBY_STACK_MIN_LIMIT)
+#endif
+#define RUBY_STACK_SPACE (RUBY_STACK_MIN/5 > RUBY_STACK_SPACE_LIMIT ? \
+			  RUBY_STACK_SPACE_LIMIT : RUBY_STACK_MIN/5)
+
 static int
 native_thread_create(rb_thread_t *th)
 {
     int err = 0;
 
     if (use_cached_thread(th)) {
-	thread_debug("create (use cached thread): %p\n", th);
+	thread_debug("create (use cached thread): %p\n", (void *)th);
     }
     else {
 	pthread_attr_t attr;
-	size_t stack_size = 512 * 1024; /* 512KB */
-        size_t space;
+	const size_t stack_size = RUBY_STACK_MIN;
+	const size_t space = RUBY_STACK_SPACE;
 
-#ifdef PTHREAD_STACK_MIN
-	if (stack_size < PTHREAD_STACK_MIN) {
-	    stack_size = PTHREAD_STACK_MIN * 2;
-	}
-#endif
-        space = stack_size/5;
-        if (space > 1024*1024) space = 1024*1024;
         th->machine_stack_maxsize = stack_size - space;
 #ifdef __ia64
         th->machine_stack_maxsize /= 2;
@@ -409,21 +542,18 @@ native_thread_create(rb_thread_t *th)
 	CHECK_ERR(pthread_attr_setstacksize(&attr, stack_size));
 #endif
 
+#ifdef HAVE_PTHREAD_ATTR_SETINHERITSCHED
 	CHECK_ERR(pthread_attr_setinheritsched(&attr, PTHREAD_INHERIT_SCHED));
+#endif
 	CHECK_ERR(pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED));
 
 	err = pthread_create(&th->thread_id, &attr, thread_start_func_1, th);
-	thread_debug("create: %p (%d)", th, err);
+	thread_debug("create: %p (%d)", (void *)th, err);
 	CHECK_ERR(pthread_attr_destroy(&attr));
 
 	if (!err) {
 	    pthread_cond_init(&th->native_thread_data.sleep_cond, 0);
 	}
-	else {
-	    st_delete_wrap(th->vm->living_threads, th->self);
-	    th->status = THREAD_KILLED;
-	    rb_raise(rb_eThreadError, "can't create Thread (%d)", err);
-	}
     }
     return err;
 }
@@ -437,6 +567,9 @@ native_thread_join(pthread_t th)
     }
 }
 
+
+#if USE_NATIVE_THREAD_PRIORITY
+
 static void
 native_thread_apply_priority(rb_thread_t *th)
 {
@@ -463,15 +596,17 @@ native_thread_apply_priority(rb_thread_t *th)
 #endif
 }
 
+#endif /* USE_NATIVE_THREAD_PRIORITY */
+
 static void
 ubf_pthread_cond_signal(void *ptr)
 {
     rb_thread_t *th = (rb_thread_t *)ptr;
-    thread_debug("ubf_pthread_cond_signal (%p)\n", th);
+    thread_debug("ubf_pthread_cond_signal (%p)\n", (void *)th);
     pthread_cond_signal(&th->native_thread_data.sleep_cond);
 }
 
-#ifndef __CYGWIN__
+#if !defined(__CYGWIN__) && !defined(__SYMBIAN32__)
 static void
 ubf_select_each(rb_thread_t *th)
 {
@@ -492,6 +627,8 @@ ubf_select(void *ptr)
 #define ubf_select 0
 #endif
 
+#define PER_NANO 1000000000
+
 static void
 native_sleep(rb_thread_t *th, struct timeval *tv)
 {
@@ -502,10 +639,10 @@ native_sleep(rb_thread_t *th, struct timeval *tv)
 	gettimeofday(&tvn, NULL);
 	ts.tv_sec = tvn.tv_sec + tv->tv_sec;
 	ts.tv_nsec = (tvn.tv_usec + tv->tv_usec) * 1000;
-        if (ts.tv_nsec >= 1000000000){
+	if (ts.tv_nsec >= PER_NANO){
 	    ts.tv_sec += 1;
-	    ts.tv_nsec -= 1000000000;
-        }
+	    ts.tv_nsec -= PER_NANO;
+	}
     }
 
     thread_debug("native_sleep %ld\n", tv ? tv->tv_sec : -1);
@@ -524,8 +661,8 @@ native_sleep(rb_thread_t *th, struct timeval *tv)
 		int r;
 		thread_debug("native_sleep: pthread_cond_wait start\n");
 		r = pthread_cond_wait(&th->native_thread_data.sleep_cond,
-				  &th->interrupt_lock);
-                if (r) rb_bug("pthread_cond_wait: %d", r);
+				      &th->interrupt_lock);
+                if (r) rb_bug_errno("pthread_cond_wait", r);
 		thread_debug("native_sleep: pthread_cond_wait end\n");
 	    }
 	    else {
@@ -534,7 +671,7 @@ native_sleep(rb_thread_t *th, struct timeval *tv)
 			     (unsigned long)ts.tv_sec, ts.tv_nsec);
 		r = pthread_cond_timedwait(&th->native_thread_data.sleep_cond,
 					   &th->interrupt_lock, &ts);
-		if (r && r != ETIMEDOUT) rb_bug("pthread_cond_timedwait: %d", r);
+		if (r && r != ETIMEDOUT) rb_bug_errno("pthread_cond_timedwait", r);
 
 		thread_debug("native_sleep: pthread_cond_timedwait end (%d)\n", r);
 	    }
@@ -636,23 +773,40 @@ remove_signal_thread_list(rb_thread_t *th)
 }
 
 static pthread_t timer_thread_id;
+static pthread_cond_t timer_thread_cond = PTHREAD_COND_INITIALIZER;
+static pthread_mutex_t timer_thread_lock = PTHREAD_MUTEX_INITIALIZER;
+
+static struct timespec *
+get_ts(struct timespec *ts, unsigned long nsec)
+{
+    struct timeval tv;
+    gettimeofday(&tv, 0);
+    ts->tv_sec = tv.tv_sec;
+    ts->tv_nsec = tv.tv_usec * 1000 + nsec;
+    if (ts->tv_nsec >= PER_NANO) {
+	ts->tv_sec++;
+	ts->tv_nsec -= PER_NANO;
+    }
+    return ts;
+}
 
 static void *
 thread_timer(void *dummy)
 {
-    while (system_working) {
-#ifdef HAVE_NANOSLEEP
-	struct timespec req, rem;
-	req.tv_sec = 0;
-	req.tv_nsec = 10 * 1000 * 1000;	/* 10 ms */
-	nanosleep(&req, &rem);
-#else
-	struct timeval tv;
-	tv.tv_sec = 0;
-	tv.tv_usec = 10000;     	/* 10 ms */
-	select(0, NULL, NULL, NULL, &tv);
-#endif
-#ifndef __CYGWIN__
+    struct timespec ts;
+
+    native_mutex_lock(&timer_thread_lock);
+    native_cond_broadcast(&timer_thread_cond);
+#define WAIT_FOR_10MS() native_cond_timedwait(&timer_thread_cond, &timer_thread_lock, get_ts(&ts, PER_NANO/100))
+    while (system_working > 0) {
+	int err = WAIT_FOR_10MS();
+	if (err == ETIMEDOUT);
+	else if (err == 0) {
+	    if (rb_signal_buff_size() == 0) break;
+	}
+	else rb_bug_errno("thread_timer/timedwait", err);
+
+#if !defined(__CYGWIN__) && !defined(__SYMBIAN32__)
 	if (signal_thread_list_anchor.next) {
 	    FGLOCK(&signal_thread_list_lock, {
 		struct signal_thread_list *list;
@@ -666,6 +820,7 @@ thread_timer(void *dummy)
 #endif
 	timer_thread_function(dummy);
     }
+    native_mutex_unlock(&timer_thread_lock);
     return NULL;
 }
 
@@ -683,12 +838,74 @@ rb_thread_create_timer_thread(void)
 	pthread_attr_setstacksize(&attr,
 				  PTHREAD_STACK_MIN + (THREAD_DEBUG ? BUFSIZ : 0));
 #endif
-	err = pthread_create(&timer_thread_id, &attr, thread_timer, GET_VM());
+	native_mutex_lock(&timer_thread_lock);
+	err = pthread_create(&timer_thread_id, &attr, thread_timer, 0);
 	if (err != 0) {
-	    rb_bug("rb_thread_create_timer_thread: return non-zero (%d)", err);
+	    native_mutex_unlock(&timer_thread_lock);
+	    fprintf(stderr, "[FATAL] Failed to create timer thread (errno: %d)\n", err);
+	    exit(EXIT_FAILURE);
 	}
+	native_cond_wait(&timer_thread_cond, &timer_thread_lock);
+	native_mutex_unlock(&timer_thread_lock);
     }
     rb_disable_interrupt(); /* only timer thread recieve signal */
 }
 
+static int
+native_stop_timer_thread(void)
+{
+    int stopped;
+    native_mutex_lock(&timer_thread_lock);
+    stopped = --system_working <= 0;
+    if (stopped) {
+	native_cond_signal(&timer_thread_cond);
+    }
+    native_mutex_unlock(&timer_thread_lock);
+    if (stopped) {
+	native_thread_join(timer_thread_id);
+    }
+    return stopped;
+}
+
+static void
+native_reset_timer_thread(void)
+{
+    timer_thread_id = 0;
+}
+
+#ifdef HAVE_SIGALTSTACK
+int
+ruby_stack_overflowed_p(const rb_thread_t *th, const void *addr)
+{
+    void *base;
+    size_t size;
+    const size_t water_mark = 1024 * 1024;
+    STACK_GROW_DIR_DETECTION;
+
+    if (th) {
+	size = th->machine_stack_maxsize;
+	base = (char *)th->machine_stack_start - STACK_DIR_UPPER(0, size);
+    }
+#ifdef STACKADDR_AVAILABLE
+    else if (get_stack(&base, &size) == 0) {
+	STACK_DIR_UPPER((void)(base = (char *)base + size), (void)0);
+    }
+#endif
+    else {
+	return 0;
+    }
+    size /= 5;
+    if (size > water_mark) size = water_mark;
+    if (STACK_DIR_UPPER(1, 0)) {
+	if (size > ~(size_t)base+1) size = ~(size_t)base+1;
+	if (addr > base && addr <= (void *)((char *)base + size)) return 1;
+    }
+    else {
+	if (size > (size_t)base) size = (size_t)base;
+	if (addr > (void *)((char *)base - size) && addr <= base) return 1;
+    }
+    return 0;
+}
+#endif
+
 #endif /* THREAD_SYSTEM_DEPENDENT_IMPLEMENTATION */
